@@ -1,13 +1,11 @@
 /*
- * $Id$
+ * These functions handle authentication of users to a Citadel server.
  *
- * WebcitAuth; Handles authentication of users to a Citadel server.
+ * Copyright (c) 1996-2011 by the citadel.org team
  *
- * Copyright (c) 1996-2010 by the citadel.org team
- *
- * This program is free software; you can redistribute it and/or modify
+ * This program is open source software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "webcit.h"
@@ -26,8 +23,6 @@
 
 extern uint32_t hashlittle( const void *key, size_t length, uint32_t initval);
 
-void display_reg(int during_login);
-
 /*
  * Access level definitions.  This is initialized from a function rather than a
  * static array so that the strings may be localized.
@@ -35,13 +30,27 @@ void display_reg(int during_login);
 char *axdefs[7]; 
 
 void initialize_axdefs(void) {
-	axdefs[0] = _("Deleted");       /* an erased user */
-	axdefs[1] = _("New User");      /* a new user */
-	axdefs[2] = _("Problem User");  /* a trouble maker */
-	axdefs[3] = _("Local User");    /* user with normal privileges */
-	axdefs[4] = _("Network User");  /* a user that may access network resources */
-	axdefs[5] = _("Preferred User");/* a moderator */
-	axdefs[6] = _("Aide");          /* chief */
+
+	/* an erased user */
+	axdefs[0] = _("Deleted");       
+
+	/* a new user */
+	axdefs[1] = _("New User");      
+
+	/* a trouble maker */
+	axdefs[2] = _("Problem User");  
+
+	/* user with normal privileges */
+	axdefs[3] = _("Local User");    
+
+	/* a user that may access network resources */
+	axdefs[4] = _("Network User");  
+
+	/* a moderator */
+	axdefs[5] = _("Preferred User");
+
+	/* chief */
+	axdefs[6] = _("Aide");          
 }
 
 
@@ -54,66 +63,13 @@ void display_login(void)
 {
 	begin_burst();
 	output_headers(1, 0, 0, 0, 1, 0);
-	do_template("login", NULL);
+	do_template("login");
 	end_burst();
 }
 
 
 
 
-/* 
- * Display the openid-enabled login screen
- * mesg = the error message if last attempt failed.
- */
-void display_openid_login(char *mesg)
-{
-  begin_burst();
-  output_headers(1, 0, 0, 0, 1, 0);
-  do_template("openid_login", NULL);
-  end_burst();
-}
-
-
-void display_openid_name_request(const StrBuf *claimed_id, const StrBuf *username) 
-{
-	StrBuf *Buf = NULL;
-
-	output_headers(1, 1, 2, 0, 0, 0);
-	wc_printf("<div id=\"login_screen\">\n");
-
-	Buf = NewStrBufPlain(NULL, StrLength(claimed_id));
-	StrEscAppend(Buf, claimed_id, NULL, 0, 0);
-	svprintf(HKEY("VERIFIED"), WCS_STRING, _("Your OpenID <tt>%s</tt> was successfully verified."),
-		 ChrPtr(Buf));
-	SVPutBuf("CLAIMED_ID", Buf, 0);
-
-
-	if (StrLength(username) > 0) {
-			Buf = NewStrBufPlain(NULL, StrLength(username));
-			StrEscAppend(Buf, username, NULL, 0, 0);
-			svprintf(HKEY("REASON"), WCS_STRING,
-				 _("However, the user name '%s' conflicts with an existing user."), 
-				 ChrPtr(Buf));
-			FreeStrBuf(&Buf);
-	}
-	else {
-		svput("REASON", WCS_STRING, "");
-	}
-
-	svput("ACTION_REQUESTED", WCS_STRING, _("Please specify the user name you would like to use."));
-
-	svput("USERNAME_BOX", WCS_STRING, _("User name:"));
-	svput("NEWUSER_BUTTON", WCS_STRING, _("New User"));
-	svput("EXIT_BUTTON", WCS_STRING, _("Exit"));
-
-	svprintf(HKEY("BOXTITLE"), WCS_STRING, _("%s - powered by <a href=\"http://www.citadel.org\">Citadel</a>"),
-		 ChrPtr(WC->serv_info->serv_humannode));
-
-	do_template("openid_manual_create", NULL);
-	wDumpContent(2);
-}
-
-
 
 /* Initialize the session
  *
@@ -155,7 +111,7 @@ void become_logged_in(const StrBuf *user, const StrBuf *pass, StrBuf *serv_respo
 	}
 
 	WCC->axlevel = StrBufExtract_int(serv_response, 1, '|');
-	if (WCC->axlevel >= 6) { /* TODO: make this a define, else it might trick us later */
+	if (WCC->axlevel >= 6) {
 		WCC->is_aide = 1;
 	}
 
@@ -168,7 +124,7 @@ void become_logged_in(const StrBuf *user, const StrBuf *pass, StrBuf *serv_respo
 		const char *pch;
 
 		pch = ChrPtr(Buf) + 4;
-		WCC->new_mail  = StrBufExtractNext_long(Buf, &pch, '|');
+		/*WCC->new_mail  =*/ StrBufExtractNext_long(Buf, &pch, '|');
 		WCC->need_regi = StrBufExtractNext_long(Buf, &pch, '|');
 		WCC->need_vali = StrBufExtractNext_long(Buf, &pch, '|');
 		if (WCC->cs_inet_email == NULL)
@@ -178,98 +134,56 @@ void become_logged_in(const StrBuf *user, const StrBuf *pass, StrBuf *serv_respo
 	get_preference("floordiv_expanded", &FloorDiv);
 	WCC->floordiv_expanded = FloorDiv;
 	FreeStrBuf(&Buf);
+	FlushRoomlist();
 }
 
 
 /* 
- * Perform authentication using a user name and password
+ * modal/ajax version of 'login' (username and password)
  */
-void do_login(void)
-{
-	wcsession *WCC = WC;
-	StrBuf *Buf;
+void ajax_login_username_password(void) {
+	StrBuf *Buf = NewStrBuf();
 
-	if (havebstr("language")) {
-		set_selected_language(bstr("language"));
-		go_selected_language();
-	}
-
-	if (havebstr("exit_action")) {
-		do_logout();
-		return;
-	}
-	Buf = NewStrBuf();
-	if (havebstr("login_action")) {
-		serv_printf("USER %s", bstr("name"));
-		StrBuf_ServGetln(Buf);
-		if (GetServerStatus(Buf, NULL) == 3) {
-			serv_printf("PASS %s", bstr("pass"));
-			StrBuf_ServGetln(Buf);
-			if (GetServerStatus(Buf, NULL) == 2) {
-				become_logged_in(sbstr("name"), sbstr("pass"), Buf);
-			} else {
-				snprintf(WCC->ImportantMessage, 
-					 sizeof (WCC->ImportantMessage), 
-					 "%s", 
-					 &(ChrPtr(Buf))[4]);
-				display_login();
-				FreeStrBuf(&Buf);
-				return;
-			}
-		} else {
-			snprintf(WCC->ImportantMessage, 
-				 sizeof (WCC->ImportantMessage), 
-				 "%s", 
-				 &(ChrPtr(Buf))[4]);
-			display_login();
-			FreeStrBuf(&Buf);
-			return;
-		}
-	}
-	if (havebstr("newuser_action")) {
-		if (!havebstr("pass")) {
-			snprintf(WCC->ImportantMessage, 
-				 sizeof (WCC->ImportantMessage), 
-				 "%s", 
-				 _("Blank passwords are not allowed."));
-			display_login();
-			FreeStrBuf(&Buf);
-			return;
-		}
-		serv_printf("NEWU %s", bstr("name"));
+	serv_printf("USER %s", bstr("name"));
+	StrBuf_ServGetln(Buf);
+	if (GetServerStatus(Buf, NULL) == 3) {
+		serv_printf("PASS %s", bstr("pass"));
 		StrBuf_ServGetln(Buf);
 		if (GetServerStatus(Buf, NULL) == 2) {
 			become_logged_in(sbstr("name"), sbstr("pass"), Buf);
-			serv_printf("SETP %s", bstr("pass"));
-			StrBuf_ServGetln(Buf); /* Don't care? */
-		} else {
-			snprintf(WCC->ImportantMessage, 
-				 sizeof (WCC->ImportantMessage), 
-				 "%s", 
-				 &(ChrPtr(Buf))[4]);
-			display_login();
-			FreeStrBuf(&Buf);
-			return;
 		}
 	}
-	if (WCC->logged_in) {
-		if (WCC->need_regi) {
-			display_reg(1);
-		} else if (WCC->need_vali) {
-			validate();
-		} else {
-			do_welcome();
-		}
-	} else {
-		snprintf(WCC->ImportantMessage, 
-			 sizeof (WCC->ImportantMessage), 
-			 "%s", 
-			 _("Your password was not accepted."));
-		display_login();
-	}
+
+	/* The client is expecting to read back a citadel protocol response */
+	wc_printf("%s", ChrPtr(Buf));
 	FreeStrBuf(&Buf);
 }
 
+
+
+/* 
+ * modal/ajax version of 'new user' (username and password)
+ */
+void ajax_login_newuser(void) {
+	StrBuf *NBuf = NewStrBuf();
+	StrBuf *SBuf = NewStrBuf();
+
+	serv_printf("NEWU %s", bstr("name"));
+	StrBuf_ServGetln(NBuf);
+	if (GetServerStatus(NBuf, NULL) == 2) {
+		become_logged_in(sbstr("name"), sbstr("pass"), NBuf);
+		serv_printf("SETP %s", bstr("pass"));
+		StrBuf_ServGetln(SBuf);
+	}
+
+	/* The client is expecting to read back a citadel protocol response */
+	wc_printf("%s", ChrPtr(NBuf));
+	FreeStrBuf(&NBuf);
+	FreeStrBuf(&SBuf);
+}
+
+
+
 /* 
  * Try to create an account manually after an OpenID was verified
  */
@@ -277,38 +191,54 @@ void openid_manual_create(void)
 {
 	StrBuf *Buf;
 
+	/* Did the user change his mind?  Pack up and go home. */
 	if (havebstr("exit_action")) {
-		do_logout();
+		begin_burst();
+		output_headers(1, 0, 0, 0, 1, 0);
+		do_template("authpopup_finished");
+		end_burst();
 		return;
 	}
 
-	if (havebstr("newuser_action")) {
-		Buf = NewStrBuf();
-		serv_printf("OIDC %s", bstr("name"));
-		StrBuf_ServGetln(Buf);
-		if (GetServerStatus(Buf, NULL) == 2) {
-			StrBuf *gpass;
 
-			gpass = NewStrBuf();
-			serv_puts("SETP GENERATE_RANDOM_PASSWORD");
-			StrBuf_ServGetln(gpass);
-			StrBufCutLeft(gpass, 4);
-			become_logged_in(sbstr("name"), gpass, Buf);
-			FreeStrBuf(&gpass);
-		}
-		FreeStrBuf(&Buf);
+	/* Ok, let's give this a try.  Can we create the new user? */
+
+	Buf = NewStrBuf();
+	serv_printf("OIDC %s", bstr("name"));
+	StrBuf_ServGetln(Buf);
+	if (GetServerStatus(Buf, NULL) == 2) {
+		StrBuf *gpass;
+
+		gpass = NewStrBuf();
+		serv_puts("SETP GENERATE_RANDOM_PASSWORD");
+		StrBuf_ServGetln(gpass);
+		StrBufCutLeft(gpass, 4);
+		become_logged_in(sbstr("name"), gpass, Buf);
+		FreeStrBuf(&gpass);
 	}
+	FreeStrBuf(&Buf);
 
+	/* Did we manage to log in?  If so, continue with the normal flow... */
 	if (WC->logged_in) {
-		if (WC->need_regi) {
-			display_reg(1);
-		} else if (WC->need_vali) {
-			validate();
-		} else {
-			do_welcome();
+		if (WC->logged_in) {
+			begin_burst();
+			output_headers(1, 0, 0, 0, 1, 0);
+			do_template("authpopup_finished");
+			end_burst();
 		}
 	} else {
-		display_openid_name_request(sbstr("openid_url"), sbstr("name"));
+		/* Still no good!  Go back to teh dialog to select a username */
+		const StrBuf *Buf;
+		putbstr("__claimed_id", NewStrBufDup(sbstr("openid_url")));
+		Buf = sbstr("name");
+		if (StrLength(Buf) > 0)
+			putbstr("__username", NewStrBufDup(Buf));
+		begin_burst();
+		output_headers(1, 0, 0, 0, 1, 0);
+		wc_printf("<html><body>");
+		do_template("openid_manual_create");
+		wc_printf("</body></html>");
+		end_burst();
 	}
 
 }
@@ -320,43 +250,32 @@ void openid_manual_create(void)
  */
 void do_openid_login(void)
 {
-	wcsession *WCC = WC;
 	char buf[4096];
 
-	if (havebstr("language")) {
-		set_selected_language(bstr("language"));
-		go_selected_language();
-	}
+	snprintf(buf, sizeof buf,
+		"OIDS %s|%s/finalize_openid_login|%s",
+		bstr("openid_url"),
+		ChrPtr(site_prefix),
+		ChrPtr(site_prefix)
+	);
 
-	if (havebstr("exit_action")) {
-		do_logout();
+	serv_puts(buf);
+	serv_getln(buf, sizeof buf);
+	if (buf[0] == '2') {
+		syslog(LOG_DEBUG, "OpenID server contacted; redirecting to %s\n", &buf[4]);
+		http_redirect(&buf[4]);
 		return;
 	}
-	if (havebstr("login_action")) {
-		snprintf(buf, sizeof buf,
-			"OIDS %s|%s://%s/finalize_openid_login|%s://%s",
-			bstr("openid_url"),
-			 (is_https ? "https" : "http"), ChrPtr(WCC->Hdr->HR.http_host),
-			 (is_https ? "https" : "http"), ChrPtr(WCC->Hdr->HR.http_host)
-		);
 
-		serv_puts(buf);
-		serv_getln(buf, sizeof buf);
-		if (buf[0] == '2') {
-			lprintf(CTDL_DEBUG, "OpenID server contacted; redirecting to %s\n", &buf[4]);
-			http_redirect(&buf[4]);
-			return;
-		}
-		else {
-			display_openid_login(&buf[4]);
-			return;
-		}
-	}
-
-	/* If we get to this point then something failed. */
-	display_openid_login(_("Your password was not accepted."));
+	begin_burst();
+	output_headers(1, 0, 0, 0, 1, 0);
+	wc_printf("<html><body>");
+	escputs(&buf[4]);
+	wc_printf("</body></html>");
+	end_burst();
 }
 
+
 /* 
  * Complete the authentication using OpenID
  * This function handles the positive or negative assertion from the user's Identity Provider
@@ -365,7 +284,6 @@ void finalize_openid_login(void)
 {
 	StrBuf *Buf;
 	wcsession *WCC = WC;
-	int already_logged_in = (WCC->logged_in) ;
 	int linecount = 0;
 	StrBuf *result = NULL;
 	StrBuf *username = NULL;
@@ -428,41 +346,62 @@ void finalize_openid_login(void)
 		}
 	}
 
-	/* If we were already logged in, this was an attempt to associate an OpenID account */
-	if (already_logged_in) {
+	/*
+	 * Is this an attempt to associate a new OpenID with an account that is already logged in?
+	 */
+	if ( (WCC->logged_in) && (havebstr("attach_existing")) ) {
 		display_openids();
-		FreeStrBuf(&result);
-		FreeStrBuf(&username);
-		FreeStrBuf(&password);
-		FreeStrBuf(&claimed_id);
-		FreeStrBuf(&logged_in_response);
-		return;
 	}
 
 	/* If this operation logged us in, either by connecting with an existing account or by
 	 * auto-creating one using Simple Registration Extension, we're already on our way.
 	 */
-	if (!strcasecmp(ChrPtr(result), "authenticate")) {
+	else if (!strcasecmp(ChrPtr(result), "authenticate")) {
 		become_logged_in(username, password, logged_in_response);
+
+		/* Did we manage to log in?  If so, continue with the normal flow... */
+		if (WC->logged_in) {
+			begin_burst();
+			output_headers(1, 0, 0, 0, 1, 0);
+			do_template("authpopup_finished");
+			end_burst();
+		} else {
+			begin_burst();
+			output_headers(1, 0, 0, 0, 1, 0);
+			wc_printf("<html><body>");
+			wc_printf(_("An error has occurred."));
+			wc_printf("</body></html>");
+			end_burst();
+		}
 	}
 
-	/* The specified OpenID was verified but the desired user name was either not specified via SRI
+	/* The specified OpenID was verified but the desired user name was either not specified via SRE
 	 * or conflicts with an existing user.  Either way the user will need to specify a new name.
 	 */
-
 	else if (!strcasecmp(ChrPtr(result), "verify_only")) {
-		display_openid_name_request(claimed_id, username);
+		putbstr("__claimed_id", claimed_id);
+		claimed_id = NULL;
+		if (StrLength(username) > 0) {
+			putbstr("__username", username);
+			username = NULL;
+		}
+		begin_burst();
+		output_headers(1, 0, 0, 0, 1, 0);
+		wc_printf("<html><body>");
+		do_template("openid_manual_create");
+		wc_printf("</body></html>");
+		end_burst();
 	}
 
-	/* Did we manage to log in?  If so, continue with the normal flow... */
-	if (WC->logged_in) {
-		if (WC->need_regi) {
-			display_reg(1);
-		} else {
-			do_welcome();
-		}
-	} else {
-		display_openid_login(_("Your password was not accepted."));
+	/* Something went VERY wrong if we get to this point */
+	else {
+		syslog(1, "finalize_openid_login() failed to do anything.  This is a code problem.\n");
+		begin_burst();
+		output_headers(1, 0, 0, 0, 1, 0);
+		wc_printf("<html><body>");
+		wc_printf(_("An error has occurred."));
+		wc_printf("</body></html>");
+		end_burst();
 	}
 
 	FreeStrBuf(&result);
@@ -493,23 +432,17 @@ void do_welcome(void)
 		if (!setup_wizard) {
 			int len;
 			sprintf(wizard_filename, "setupwiz.%s.%s",
-				ctdlhost, ctdlport);
-			len = strlen(wizard_filename);
-			for (i=0; i<len; ++i) {
-				if (	(wizard_filename[i]==' ')
-					|| (wizard_filename[i] == '/')
-				) {
-					wizard_filename[i] = '_';
-				}
-			}
-	
+				abs(HashLittle(ctdlhost, strlen(ctdlhost))),
+				abs(HashLittle(ctdlport, strlen(ctdlport)))
+			);
+
 			fp = fopen(wizard_filename, "r");
 			if (fp != NULL) {
 				fgets(buf, sizeof buf, fp);
 				buf[strlen(buf)-1] = 0;
 				fclose(fp);
 				if (atoi(buf) == serv_info.serv_rev_level) {
-					setup_wizard = 1; /**< already run */
+					setup_wizard = 1;	/* already run */
 				}
 			}
 		}
@@ -534,7 +467,7 @@ void do_welcome(void)
 	if (StrLength(Buf) == 0) {
 		StrBufAppendBufPlain(Buf, "dotgoto?room=_BASEROOM_", -1, 0);
 	}
-	lprintf(9, "Redirecting to user's start page: %s\n", ChrPtr(Buf));
+	syslog(9, "Redirecting to user's start page: %s\n", ChrPtr(Buf));
 	http_redirect(ChrPtr(Buf));
 }
 
@@ -543,14 +476,14 @@ void do_welcome(void)
  * Disconnect from the Citadel server, and end this WebCit session
  */
 void end_webcit_session(void) {
-	
 	serv_puts("QUIT");
 	WC->killthis = 1;
 	/* close() of citadel socket will be done by do_housekeeping() */
 }
 
+
 /* 
- * execute the logout
+ * Log out the session with the Citadel server
  */
 void do_logout(void)
 {
@@ -559,27 +492,37 @@ void do_logout(void)
 
 	FlushStrBuf(WCC->wc_username);
 	FlushStrBuf(WCC->wc_password);
-	FlushStrBuf(WCC->CurRoom.name);
 	FlushStrBuf(WCC->wc_fullname);
+	FlushRoomlist();
 
-	/* FIXME: this is to suppress the iconbar displaying, because we aren't
-	   actually logged out yet */
+	serv_puts("LOUT");
+	serv_getln(buf, sizeof buf);
 	WCC->logged_in = 0;
-	
-	/** Calling output_headers() this way causes the cookies to be un-set */
+
+	FlushStrBuf(WCC->CurRoom.name);
+
+	/* Calling output_headers() this way causes the cookies to be un-set */
 	output_headers(1, 1, 0, 1, 0, 0);
 
+	/* For sites in guest mode, redirect to the landing page after we're logged out */
+	if (WC->serv_info->serv_supports_guest) {
+		wc_printf("	<script type=\"text/javascript\">	"
+			"	window.location='/';			"
+			"	</script>				"
+		);
+	}
+
 	wc_printf("<div id=\"logout_screen\">");
         wc_printf("<div class=\"box\">");
         wc_printf("<div class=\"boxlabel\">");
 	wc_printf(_("Log off"));
-        wc_printf("</div><div class=\"boxcontent\">");	
+        wc_printf("</div><div class=\"boxcontent\">");
 	serv_puts("MESG goodbye");
 	serv_getln(buf, sizeof buf);
 
 	if (WCC->serv_sock >= 0) {
 		if (buf[0] == '1') {
-			fmout("CENTER");
+			fmout("'CENTER'");
 		} else {
 			wc_printf("Goodbye\n");
 		}
@@ -590,7 +533,7 @@ void do_logout(void)
 			"this problem to your system administrator.")
 		);
 		wc_printf("<a href=\"http://www.citadel.org/doku.php/"
-			"faq:mastering_your_os:net#netstat\">%s</a>", 
+			"faq:mastering_your_os:net#netstat\">%s</a>",
 			_("Read More..."));
 	}
 
@@ -598,22 +541,39 @@ void do_logout(void)
 		"<span class=\"button_link\"><a href=\".\">");
 	wc_printf(_("Log in again"));
 	wc_printf("</a></span>");
+	wc_printf("</div></div></div>\n");
+	if (WC->serv_info->serv_supports_guest) {
+		display_default_landing_page();
+		return;
+	}
 
-	/* The "close window" link is commented out because some browsers don't
-	 * allow it to work.
-	 *
-	wc_printf("&nbsp;&nbsp;&nbsp;<span class=\"button_link\">"
-		"<a href=\"javascript:window.close();\">");
-	wc_printf(_("Close window"));
-	wc_printf("</a></span>");
-	 */
-
-	wc_printf("</div></div></div></div>\n");
 	wDumpContent(2);
 	end_webcit_session();
 }
 
 
+/* 
+ * Special page for monitoring scripts etc
+ */
+void monitor(void)
+{
+	output_headers(0, 0, 0, 0, 0, 0);
+
+	hprintf("Content-type: text/plain\r\n"
+		"Server: " PACKAGE_STRING "\r\n"
+		"Connection: close\r\n"
+	);
+	begin_burst();
+
+	wc_printf("Connection to Citadel server at %s:%s : %s\r\n",
+		ctdlhost, ctdlport,
+		(WC->connected ? "SUCCESS" : "FAIL")
+	);
+
+	wDumpContent(0);
+}
+
+
 /*
  * validate new users
  */
@@ -624,14 +584,11 @@ void validate(void)
 	char buf[SIZ];
 	int a;
 
-	output_headers(1, 1, 2, 0, 0, 0);
-	wc_printf("<div id=\"banner\">\n");
-	wc_printf("<h1>");
-	wc_printf(_("Validate new users"));
-	wc_printf("</h1>");
-	wc_printf("</div>\n");
+	output_headers(1, 1, 1, 0, 0, 0);
 
-	wc_printf("<div id=\"content\" class=\"service\">\n");
+        do_template("box_begin_1");
+        StrBufAppendBufPlain(WC->WBuf, _("Validate new users"), -1, 0);
+        do_template("box_begin_2");
 
 	/* If the user just submitted a validation, process it... */
 	safestrncpy(buf, bstr("user"), sizeof buf);
@@ -661,8 +618,7 @@ void validate(void)
 		return;
 	}
 
-	wc_printf("<div class=\"fix_scrollbar_bug\">"
-		"<table class=\"auth_validate\"><tr><td>\n");
+	wc_printf("<table class=\"auth_validate\"><tr><td>\n");
 	wc_printf("<div id=\"validate\">");
 
 	safestrncpy(user, &buf[4], sizeof user);
@@ -681,7 +637,7 @@ void validate(void)
 				int haveChar = 0;
 				int haveNum = 0;
 				int haveOther = 0;
-				int count = 0;
+				int haveLong = 0;
 				pch = buf;
 				while (!IsEmptyStr(pch))
 				{
@@ -693,10 +649,13 @@ void validate(void)
 						haveOther = 1;
 					pch ++;
 				}
-				count = pch - buf;
-				if (count > 7)
-					count = 0;
-				switch (count){
+				if (pch - buf > 7)
+					haveLong = 1;
+				switch (haveLong + 
+					haveChar + 
+					haveNum + 
+					haveOther)
+				{
 				case 0:
 					pch = _("very weak");
 					break;
@@ -730,27 +689,27 @@ void validate(void)
 					atoi(buf), axdefs[atoi(buf)]);
 		} while (strcmp(buf, "000"));
 	} else {
-		wc_printf("<H1>%s</H1>%s<br />\n", user, &cmd[4]);
+		wc_printf("<H1>%s</H1>%s<br>\n", user, &cmd[4]);
 	}
 
 	wc_printf("<hr />");
 	wc_printf(_("Select access level for this user:"));
-	wc_printf("<br />\n");
+	wc_printf("<br>\n");
 	for (a = 0; a <= 6; ++a) {
 		wc_printf("<a href=\"validate?nonce=%d?user=", WC->nonce);
 		urlescputs(user);
 		wc_printf("&axlevel=%d\">%s</A>&nbsp;&nbsp;&nbsp;\n",
 			a, axdefs[a]);
 	}
-	wc_printf("<br />\n");
+	wc_printf("<br>\n");
 
 	wc_printf("</div>\n");
-	wc_printf("</td></tr></table></div>\n");
+	wc_printf("</td></tr></table>\n");
+	do_template("box_end");
 	wDumpContent(1);
 }
 
 
-
 /*
  * Display form for registration.
  *
@@ -768,9 +727,9 @@ void display_reg(int during_login)
 	Buf = NewStrBuf();
 	memset(&Room, 0, sizeof(folder));
 	if (goto_config_room(Buf, &Room) != 0) {
-		lprintf(9, "display_reg() exiting because goto_config_room() failed\n");
+		syslog(9, "display_reg() exiting because goto_config_room() failed\n");
 		if (during_login) {
-			do_welcome();
+			pop_destination();
 		}
 		else {
 			display_main_menu();
@@ -784,9 +743,9 @@ void display_reg(int during_login)
 	FreeStrBuf(&Buf);
 	vcard_msgnum = locate_user_vcard_in_this_room(&VCMsg, &VCAtt);
 	if (vcard_msgnum < 0L) {
-		lprintf(9, "display_reg() exiting because locate_user_vcard_in_this_room() failed\n");
+		syslog(9, "display_reg() exiting because locate_user_vcard_in_this_room() failed\n");
 		if (during_login) {
-			do_welcome();
+			pop_destination();
 		}
 		else {
 			display_main_menu();
@@ -795,27 +754,25 @@ void display_reg(int during_login)
 	}
 
 	if (during_login) {
-		do_edit_vcard(vcard_msgnum, "1", VCMsg, VCAtt, "do_welcome", USERCONFIGROOM);
+		do_edit_vcard(vcard_msgnum, "1", VCMsg, VCAtt, "pop", USERCONFIGROOM);
 	}
 	else {
 		StrBuf *ReturnTo;
-		ReturnTo = NewStrBufPlain(HKEY("display_main_menu?gotofirst="));
+		ReturnTo = NewStrBufPlain(HKEY("display_main_menu?go="));
 		StrBufAppendBuf(ReturnTo, WC->CurRoom.name, 0);
 		do_edit_vcard(vcard_msgnum, "1", VCMsg, VCAtt, ChrPtr(ReturnTo), USERCONFIGROOM);
 		FreeStrBuf(&ReturnTo);
 	}
 
-	/* FIXME - don't we have to free VCMsg and VCAtt ?? */
 }
 
 
-
-
 /*
  * display form for changing your password
  */
 void display_changepw(void)
 {
+	wcsession *WCC = WC;
 	WCTemplputParams SubTP;
 	char buf[SIZ];
 	StrBuf *Buf;
@@ -825,14 +782,14 @@ void display_changepw(void)
 	memset(&SubTP, 0, sizeof(WCTemplputParams));
 	SubTP.Filter.ContextType = CTX_STRBUF;
 	SubTP.Context = Buf;
-	DoTemplate(HKEY("beginbox"), NULL, &SubTP);
+	DoTemplate(HKEY("box_begin"), NULL, &SubTP);
 
 	FreeStrBuf(&Buf);
 
-	if (!IsEmptyStr(WC->ImportantMessage)) {
+	if (StrLength(WCC->ImportantMsg) > 0) {
 		wc_printf("<span class=\"errormsg\">"
-			"%s</span><br />\n", WC->ImportantMessage);
-		safestrncpy(WC->ImportantMessage, "", sizeof WC->ImportantMessage);
+			  "%s</span><br>\n", ChrPtr(WCC->ImportantMsg));
+		FlushStrBuf(WCC->ImportantMsg);
 	}
 
 	serv_puts("MESG changepw");
@@ -861,7 +818,7 @@ void display_changepw(void)
 	wc_printf("</div>\n");
 	wc_printf("</form>\n");
 
-	do_template("endbox", NULL);
+	do_template("box_end");
 	wDumpContent(1);
 }
 
@@ -871,13 +828,11 @@ void display_changepw(void)
  */
 void changepw(void)
 {
-	char buf[SIZ];
+	StrBuf *Line;
 	char newpass1[32], newpass2[32];
 
 	if (!havebstr("change_action")) {
-		safestrncpy(WC->ImportantMessage, 
-			_("Cancelled.  Password was not changed."),
-			sizeof WC->ImportantMessage);
+		AppendImportantMessage(_("Cancelled.  Password was not changed."), -1);
 		display_main_menu();
 		return;
 	}
@@ -886,53 +841,80 @@ void changepw(void)
 	safestrncpy(newpass2, bstr("newpass2"), sizeof newpass2);
 
 	if (strcasecmp(newpass1, newpass2)) {
-		safestrncpy(WC->ImportantMessage, 
-			_("They don't match.  Password was not changed."),
-			sizeof WC->ImportantMessage);
+		AppendImportantMessage(_("They don't match.  Password was not changed."), -1);
 		display_changepw();
 		return;
 	}
 
 	if (IsEmptyStr(newpass1)) {
-		safestrncpy(WC->ImportantMessage, 
-			_("Blank passwords are not allowed."),
-			sizeof WC->ImportantMessage);
+		AppendImportantMessage(_("Blank passwords are not allowed."), -1);
 		display_changepw();
 		return;
 	}
 
+	Line = NewStrBuf();
 	serv_printf("SETP %s", newpass1);
-	serv_getln(buf, sizeof buf);
-	sprintf(WC->ImportantMessage, "%s", &buf[4]);
-	if (buf[0] == '2') {
+	StrBuf_ServGetln(Line);
+	if (GetServerStatusMsg(Line, NULL, 1, 0) == 2) {
 		if (WC->wc_password == NULL)
-			WC->wc_password = NewStrBufPlain(buf, -1);
+			WC->wc_password = NewStrBufPlain(
+				ChrPtr(Line) + 4, 
+				StrLength(Line) - 4);
 		else {
 			FlushStrBuf(WC->wc_password);
-			StrBufAppendBufPlain(WC->wc_password,  buf, -1, 0);
+			StrBufAppendBufPlain(WC->wc_password,  
+					     ChrPtr(Line) + 4, 
+					     StrLength(Line) - 4, 0);
 		}
 		display_main_menu();
 	}
 	else {
 		display_changepw();
 	}
+	FreeStrBuf(&Line);
 }
 
+
+int ConditionalHaveAccessCreateRoom(StrBuf *Target, WCTemplputParams *TP)
+{
+	StrBuf *Buf;	
+
+	Buf = NewStrBuf();
+	serv_puts("CRE8 0");
+	StrBuf_ServGetln(Buf);
+
+	if (GetServerStatus(Buf, NULL) == 2) {
+		StrBufCutLeft(Buf, 4);
+		AppendImportantMessage(SKEY(Buf));
+		FreeStrBuf(&Buf);
+		return 0;
+	}
+	FreeStrBuf(&Buf);
+	return 1;
+}
+
+
 int ConditionalAide(StrBuf *Target, WCTemplputParams *TP)
 {
 	wcsession *WCC = WC;
-	return (WCC != NULL)? (WC->is_aide == 0) : 0;
+	return (WCC != NULL) ? ((WCC->logged_in == 0)||(WC->is_aide == 0)) : 0;
 }
 
+
 int ConditionalIsLoggedIn(StrBuf *Target, WCTemplputParams *TP) 
 {
 	wcsession *WCC = WC;
-	return (WCC != NULL)? (WCC->logged_in == 0) : 0;
+	return (WCC != NULL) ? (WCC->logged_in == 0) : 0;
+
 }
 
 
-void _display_openid_login(void) {
-	display_openid_login(NULL);
+/* 
+ * toggle the session over to a different language
+ */
+void switch_language(void) {
+	set_selected_language(bstr("lang"));
+	pop_destination();
 }
 
 
@@ -952,10 +934,11 @@ void Header_HandleAuth(StrBuf *Line, ParsedHttpHdrs *hdr)
 			hdr->HR.got_auth = AUTH_BASIC;
 		}
 		else 
-			lprintf(1, "Authentication scheme not supported! [%s]\n", ChrPtr(Line));
+			syslog(1, "Authentication scheme not supported! [%s]\n", ChrPtr(Line));
 	}
 }
 
+
 void CheckAuthBasic(ParsedHttpHdrs *hdr)
 {
 /*
@@ -965,13 +948,9 @@ void CheckAuthBasic(ParsedHttpHdrs *hdr)
 */
 	StrBufAppendBufPlain(hdr->HR.plainauth, HKEY(":"), 0);
 	StrBufAppendBuf(hdr->HR.plainauth, hdr->HR.user_agent, 0);
-	hdr->HR.SessionKey = hashlittle(SKEY(hdr->HR.plainauth), 89479832);
-/*
-	lprintf(1, "CheckAuthBasic: calculated sessionkey %ld\n", 
-		hdr->HR.SessionKey);
-*/
 }
 
+
 void GetAuthBasic(ParsedHttpHdrs *hdr)
 {
 	const char *Pos = NULL;
@@ -983,6 +962,7 @@ void GetAuthBasic(ParsedHttpHdrs *hdr)
 	StrBufExtract_NextToken(hdr->c_password, hdr->HR.plainauth, &Pos, ':');
 }
 
+
 void Header_HandleCookie(StrBuf *Line, ParsedHttpHdrs *hdr)
 {
 	const char *pch;
@@ -1009,6 +989,7 @@ void Header_HandleCookie(StrBuf *Line, ParsedHttpHdrs *hdr)
 	hdr->HR.got_auth = AUTH_COOKIE;
 }
 
+
 void 
 HttpNewModule_AUTH
 (ParsedHttpHdrs *httpreq)
@@ -1018,6 +999,8 @@ HttpNewModule_AUTH
 	httpreq->c_roomname = NewStrBuf();
 	httpreq->c_language = NewStrBuf();
 }
+
+
 void 
 HttpDetachModule_AUTH
 (ParsedHttpHdrs *httpreq)
@@ -1028,6 +1011,7 @@ HttpDetachModule_AUTH
 	FLUSHStrBuf(httpreq->c_language);
 }
 
+
 void 
 HttpDestroyModule_AUTH
 (ParsedHttpHdrs *httpreq)
@@ -1038,31 +1022,37 @@ HttpDestroyModule_AUTH
 	FreeStrBuf(&httpreq->c_language);
 }
 
+
 void 
 InitModule_AUTH
 (void)
 {
+	initialize_axdefs();
 	RegisterHeaderHandler(HKEY("COOKIE"), Header_HandleCookie);
 	RegisterHeaderHandler(HKEY("AUTHORIZATION"), Header_HandleAuth);
 
-	WebcitAddUrlHandler(HKEY(""), "", 0, do_welcome, ANONYMOUS|COOKIEUNNEEDED); /* no url pattern at all? Show login. */
+	/* no url pattern at all? Show login. */
+	WebcitAddUrlHandler(HKEY(""), "", 0, do_welcome, ANONYMOUS|COOKIEUNNEEDED);
+
 	WebcitAddUrlHandler(HKEY("do_welcome"), "", 0, do_welcome, ANONYMOUS|COOKIEUNNEEDED);
-	WebcitAddUrlHandler(HKEY("login"), "", 0, do_login, ANONYMOUS|COOKIEUNNEEDED);
-	WebcitAddUrlHandler(HKEY("display_openid_login"), "", 0, _display_openid_login, ANONYMOUS);
 	WebcitAddUrlHandler(HKEY("openid_login"), "", 0, do_openid_login, ANONYMOUS);
 	WebcitAddUrlHandler(HKEY("finalize_openid_login"), "", 0, finalize_openid_login, ANONYMOUS);
 	WebcitAddUrlHandler(HKEY("openid_manual_create"), "", 0, openid_manual_create, ANONYMOUS);
-	WebcitAddUrlHandler(HKEY("do_logout"), "", 0, do_logout, ANONYMOUS|COOKIEUNNEEDED|FORCE_SESSIONCLOSE);
 	WebcitAddUrlHandler(HKEY("validate"), "", 0, validate, 0);
+	WebcitAddUrlHandler(HKEY("do_welcome"), "", 0, do_welcome, 0);
 	WebcitAddUrlHandler(HKEY("display_reg"), "", 0, _display_reg, 0);
 	WebcitAddUrlHandler(HKEY("display_changepw"), "", 0, display_changepw, 0);
 	WebcitAddUrlHandler(HKEY("changepw"), "", 0, changepw, 0);
 	WebcitAddUrlHandler(HKEY("termquit"), "", 0, do_logout, 0);
-
+	WebcitAddUrlHandler(HKEY("do_logout"), "", 0, do_logout, ANONYMOUS|COOKIEUNNEEDED|FORCE_SESSIONCLOSE);
+	WebcitAddUrlHandler(HKEY("monitor"), "", 0, monitor, ANONYMOUS|COOKIEUNNEEDED|FORCE_SESSIONCLOSE);
+	WebcitAddUrlHandler(HKEY("ajax_login_username_password"), "", 0, ajax_login_username_password, AJAX|ANONYMOUS);
+	WebcitAddUrlHandler(HKEY("ajax_login_newuser"), "", 0, ajax_login_newuser, AJAX|ANONYMOUS);
+	WebcitAddUrlHandler(HKEY("switch_language"), "", 0, switch_language, ANONYMOUS);
 	RegisterConditional(HKEY("COND:AIDE"), 2, ConditionalAide, CTX_NONE);
 	RegisterConditional(HKEY("COND:LOGGEDIN"), 2, ConditionalIsLoggedIn, CTX_NONE);
-
-	return ;
+	RegisterConditional(HKEY("COND:MAY_CREATE_ROOM"), 2,  ConditionalHaveAccessCreateRoom, CTX_NONE);
+	return;
 }
 
 
