@@ -1,12 +1,10 @@
 /*
- * $Id$
- *
  * Copyright (c) 1996-2010 by the citadel.org team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
+ * This program is open source software.  You can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 
@@ -106,7 +104,7 @@ int ical_ctdl_is_overlap(
 		return(1);
 	}
 
-	/* lprintf (9, "Comparing t1start %d:%d t1end %d:%d t2start %d:%d t2end %d:%d \n",
+	/* syslog(9, "Comparing t1start %d:%d t1end %d:%d t2start %d:%d t2end %d:%d \n",
 		t1start.hour, t1start.minute, t1end.hour, t1end.minute,
 		t2start.hour, t2start.minute, t2end.hour, t2end.minute);
 	*/
@@ -115,11 +113,11 @@ int ical_ctdl_is_overlap(
 
 	/* If event 1 ends before event 2 starts, we're in the clear. */
 	if (icaltime_compare(t1end, t2start) <= 0) return(0);
-	/* lprintf(9, "first passed\n"); */
+	/* syslog(9, "first passed\n"); */
 
 	/* If event 2 ends before event 1 starts, we're also ok. */
 	if (icaltime_compare(t2end, t1start) <= 0) return(0);
-	/* lprintf(9, "second passed\n"); */
+	/* syslog(9, "second passed\n"); */
 
 	/* Otherwise, they overlap. */
 	return(1);
@@ -211,6 +209,7 @@ void check_attendee_availability(icalcomponent *vevent) {
 	char attendee_string[SIZ];
 	char annotated_attendee_string[SIZ];
 	char annotation[SIZ];
+	const char *ch;
 
 	if (vevent == NULL) {
 		return;
@@ -250,12 +249,11 @@ void check_attendee_availability(icalcomponent *vevent) {
 	for (attendee = icalcomponent_get_first_property(vevent, ICAL_ATTENDEE_PROPERTY);
 	    attendee != NULL;
 	    attendee = icalcomponent_get_next_property(vevent, ICAL_ATTENDEE_PROPERTY)) {
-
-		strcpy(attendee_string, icalproperty_get_attendee(attendee));
-		if (!strncasecmp(attendee_string, "MAILTO:", 7)) {
+		ch = icalproperty_get_attendee(attendee);
+		if ((ch != NULL) && !strncasecmp(ch, "MAILTO:", 7)) {
 
 			/** screen name or email address */
-			strcpy(attendee_string, &attendee_string[7]);
+			safestrncpy(attendee_string, ch + 7, sizeof(attendee_string));
 			striplt(attendee_string);
 
 			check_individual_attendee(attendee_string,
