@@ -1,14 +1,12 @@
 /*
- * $Id$
- *
  * Handles the HTML display of calendar items.
  *
- * Copyright (c) 1996-2010 by the citadel.org team
+ * Copyright (c) 1996-2011 by the citadel.org team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
+ * This program is open source software.  You can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "webcit.h"
@@ -42,7 +40,7 @@ void embeddable_mini_calendar(int year, int month)
 	long weekstart = 0;
 	char url[256];
 	char div_id[256];
-	
+
 	snprintf(div_id, sizeof div_id, "mini_calendar_%d", rand() );
 
 	/* Determine what day to start.  If an impossible value is found, start on Sunday.
@@ -96,7 +94,7 @@ void embeddable_mini_calendar(int year, int month)
 	wc_printf("<a href=\"javascript:minical_change_month(%d,%d);\">&raquo;</a>",
 		(int)(tm.tm_year)+1900, tm.tm_mon + 1);
 
-	wc_printf("<table border=0 cellpadding=1 cellspacing=1 class=\"mini_calendar_days\">"
+	wc_printf("<table border=\"0\" cellpadding=\"1\" cellspacing=\"1\" class=\"mini_calendar_days\">"
 		"<tr>");
 	colheader_time = thetime;
 	for (i=0; i<7; ++i) {
@@ -121,7 +119,7 @@ void embeddable_mini_calendar(int year, int month)
 			}
 
 			if (tm.tm_mon == month-1) {
-				snprintf(url, sizeof url, "readfwd?calview=day&year=%d&month=%d&day=%d", 
+				snprintf(url, sizeof url, "readfwd?calview=day?year=%d?month=%d?day=%d",
 					tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday);
 				wc_printf("<td><a href=\"%s\">%d</a></td>", url, tm.tm_mday);
 			}
@@ -190,7 +188,7 @@ void calendar_month_view_display_events(int year, int month, int day)
 	time_t tt;
 
 	if (GetCount(WCC->disp_cal_items) == 0) {
-		wc_printf("<br /><br /><br />\n");
+		wc_printf("<br>\n");
 		return;
 	}
 
@@ -261,6 +259,14 @@ void calendar_month_view_display_events(int year, int month, int day)
 		 * If we determined that this event occurs today, then display it.
 	 	 */
 		if (show_event) {
+
+			/* time_t logtt = icaltime_as_timet(t);
+			syslog(LOG_DEBUG, "Match on %04d-%02d-%02d for event %x%s on %s",
+				year, month, day,
+				(int)Cal, ((all_day_event) ? " (all day)" : ""),
+				ctime(&logtt)
+			); */
+
 			p = icalcomponent_get_first_property(Cal->cal, ICAL_SUMMARY_PROPERTY);
 			if (p == NULL) {
 				p = icalproperty_new_summary(_("Untitled Event"));
@@ -269,7 +275,7 @@ void calendar_month_view_display_events(int year, int month, int day)
 			if (p != NULL) {
 
 				if (all_day_event) {
-					wc_printf("<table border=0 cellpadding=2><TR>"
+					wc_printf("<table border=\"0\" cellpadding=\"2\"><TR>"
 						"<td bgcolor=\"#CCCCDD\">"
 						);
 				}
@@ -288,26 +294,26 @@ void calendar_month_view_display_events(int year, int month, int day)
 
 				wc_printf("<span class=\"tooltip\"><span class=\"btttop\"></span><span class=\"bttmiddle\">");
 
-				wc_printf("<i>%s: %s</i><br />", _("From"), Cal->from);
+				wc_printf("<i>%s: %s</i><br>", _("From"), Cal->from);
 				wc_printf("<i>%s</i> ",          _("Summary:"));
 				escputs((char *)icalproperty_get_comment(p));
-				wc_printf("<br />");
-				
+				wc_printf("<br>");
+
 				q = icalcomponent_get_first_property(
 					Cal->cal,
 					ICAL_LOCATION_PROPERTY);
 				if (q) {
 					wc_printf("<i>%s</i> ", _("Location:"));
 					escputs((char *)icalproperty_get_comment(q));
-					wc_printf("<br />");
+					wc_printf("<br>");
 				}
-				
+
 				/*
 				 * Only show start/end times if we're actually looking at the VEVENT
 				 * component.  Otherwise it shows bogus dates for e.g. timezones
 				 */
 				if (icalcomponent_isa(Cal->cal) == ICAL_VEVENT_COMPONENT) {
-					
+
 					q = icalcomponent_get_first_property(Cal->cal, ICAL_DTSTART_PROPERTY);
 					if (q != NULL) {
 						int no_end = 0;
@@ -369,31 +375,31 @@ void calendar_month_view_display_events(int year, int month, int day)
 								webcit_fmt_date(buf, 256, tt, DATEFMT_BRIEF);
 								wc_printf("<i>%s</i> %s<br>", _("Ending date/time:"), buf);
 							}
-							
+
 						}
 					}
-					
+
 				}
-				
+
 				q = icalcomponent_get_first_property(Cal->cal, ICAL_DESCRIPTION_PROPERTY);
 				if (q) {
 					wc_printf("<i>%s</i> ", _("Notes:"));
 					escputs((char *)icalproperty_get_comment(q));
-					wc_printf("<br />");
+					wc_printf("<br>");
 				}
-				
+
 				wc_printf("</span><span class=\"bttbottom\"></span></span>");
-				wc_printf("</a></font><br />\n");
-				
+				wc_printf("</a></font><br>\n");
+
 				if (all_day_event) {
 					wc_printf("</td></tr></table>");
 				}
-				
+
 			}
-			
+
 		}
-		
-		
+
+
 	}
 	DeleteHashPos(&Pos);
 }
@@ -418,20 +424,16 @@ void calendar_month_view_brief_events(time_t thetime, const char *daycolor) {
 	icalproperty *e;
 	struct icaltimetype t;
 	disp_cal *Cal;
-	int month, day, year;
 	int all_day_event = 0;
 	char *timeformat;
 	int time_format;
-	
+
 	time_format = get_time_format_cached ();
 
 	if (time_format == WC_TIMEFORMAT_24) timeformat="%k:%M";
 	else timeformat="%I:%M %p";
 
 	localtime_r(&thetime, &today_tm);
-	month = today_tm.tm_mon + 1;
-	day = today_tm.tm_mday;
-	year = today_tm.tm_year + 1900;
 
 	Pos = GetNewHashPos(WCC->disp_cal_items, 0);
 	while (GetNextHashPos(WCC->disp_cal_items, Pos, &hklen, &HashKey, &vCal)) {
@@ -454,11 +456,11 @@ void calendar_month_view_brief_events(time_t thetime, const char *daycolor) {
 			if ((event_tms.tm_year == today_tm.tm_year)
 				&& (event_tms.tm_mon == today_tm.tm_mon)
 			&& (event_tms.tm_mday == today_tm.tm_mday)) {
-			
-			
+
+
 			char sbuf[255];
 			char ebuf[255];
-			
+
 			p = icalcomponent_get_first_property(
 				Cal->cal,
 				ICAL_SUMMARY_PROPERTY);
@@ -483,20 +485,20 @@ void calendar_month_view_brief_events(time_t thetime, const char *daycolor) {
 					"<a class=\"event%s\" href=\"display_edit_event?msgnum=%ld?calview=calbrief?year=%s?month=%s?day=%s\">",
 					daycolor,
 					hours, minutes,
-					(Cal->unread)?"_unread":"_read",						
+					(Cal->unread)?"_unread":"_read",
 					daycolor,
 					Cal->cal_msgnum,
 					bstr("year"),
 					bstr("month"),
 					bstr("day")
 					);
-				
+
 				escputs((char *)
 					icalproperty_get_comment(p));
 				/* \todo: allso ammitime format */
 				wc_strftime(&sbuf[0], sizeof(sbuf), timeformat, &event_tms);
 				wc_strftime(&ebuf[0], sizeof(sbuf), timeformat, &event_tme);
-				
+
 				wc_printf("</a></font></td>"
 					"<td bgcolor='%s'>%s</td><td bgcolor='%s'>%s</td></tr>",
 					daycolor,
@@ -504,10 +506,10 @@ void calendar_month_view_brief_events(time_t thetime, const char *daycolor) {
 					daycolor,
 					ebuf);
 				}
-			
+
 			}
-			
-			
+
+
 		}
 	}
 	DeleteHashPos(&Pos);
@@ -571,21 +573,20 @@ void calendar_month_view(int year, int month, int day) {
 	}
 
 	/* Outer table (to get the background color) */
-	wc_printf("<div class=\"fix_scrollbar_bug\">"
-		"<table class=\"calendar\"> \n <tr><td>"); 
+	wc_printf("<table class=\"calendar\"> \n <tr><td>"); 
 
-	wc_printf("<table width=100%% border=0 cellpadding=0 cellspacing=0><tr>\n");
+	wc_printf("<table width=\"100%%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\"><tr>\n");
 
-	wc_printf("<td align=center>");
+	wc_printf("<td align=\"center\">");
 
 	localtime_r(&previous_month, &tm);
 	wc_printf("<a href=\"readfwd?calview=month?year=%d?month=%d?day=1\">",
 		(int)(tm.tm_year)+1900, tm.tm_mon + 1);
-	wc_printf("<img align=middle src=\"static/prevdate_32x.gif\" border=0></A>\n");
+	wc_printf("<img alt=\"%s\" align=\"middle\" src=\"static/webcit_icons/essen/32x32/back.png\" border=\"0\"></a>\n", _("previous"));
 
 	wc_strftime(colheader_label, sizeof colheader_label, "%B", &starting_tm);
 	wc_printf("&nbsp;&nbsp;"
-		"<font size=+1 color=\"#FFFFFF\">"
+		"<font size=\"+1\" color=\"#FFFFFF\">"
 		"%s %d"
 		"</font>"
 		"&nbsp;&nbsp;", colheader_label, year);
@@ -593,20 +594,20 @@ void calendar_month_view(int year, int month, int day) {
 	localtime_r(&next_month, &tm);
 	wc_printf("<a href=\"readfwd?calview=month?year=%d?month=%d?day=1\">",
 		(int)(tm.tm_year)+1900, tm.tm_mon + 1);
-	wc_printf("<img align=middle src=\"static/nextdate_32x.gif\" border=0></A>\n");
+	wc_printf("<img alt=\"%s\" align=\"middle\" src=\"static/webcit_icons/essen/32x32/forward.png\" border=\"0\"></A>\n", _("next"));
 
 	wc_printf("</td></tr></table>\n");
 
 	/* Inner table (the real one) */
-	wc_printf("<table width=100%% border=0 cellpadding=1 cellspacing=1 "
-		"bgcolor=#204B78 id=\"inner_month\"><tr>");
-	wc_printf("<th align=center width=2%%></th>");
+	wc_printf("<table width=\"100%%\" border=\"0\" cellpadding=\"1\" cellspacing=\"1\" "
+		"bgcolor='#204B78' id=\"inner_month\"><tr>");
+	wc_printf("<th align=\"center\" width=\"2%%\"></th>");
 	colheader_time = thetime;
 	for (i=0; i<7; ++i) {
 		colheader_time = thetime + (i * 86400) ;
 		localtime_r(&colheader_time, &colheader_tm);
 		wc_strftime(colheader_label, sizeof colheader_label, "%A", &colheader_tm);
-		wc_printf("<th align=center width=14%%>"
+		wc_printf("<th align=\"center\" width=\"14%%\">"
 			"<font color=\"#FFFFFF\">%s</font></th>", colheader_label);
 
 	}
@@ -664,8 +665,8 @@ void calendar_month_view(int year, int month, int day) {
 	}
 
 	wc_printf("</table>"			/* end of inner table */
-		"</td></tr></table>"		/* end of outer table */
-		"</div>\n");
+		"</td></tr></table>\n"		/* end of outer table */
+	);
 }
 
 /*
@@ -707,22 +708,21 @@ void calendar_brief_month_view(int year, int month, int day) {
 	}
 
 	/* Outer table (to get the background color) */
-	wc_printf("<div class=\"fix_scrollbar_bug\">"
-		"<table width=100%% border=0 cellpadding=0 cellspacing=0 "
-		"bgcolor=#204B78><TR><TD>\n");
+	wc_printf("<table width=\"100%%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\" "
+		"bgcolor=#204B78><tr><td>\n");
 
-	wc_printf("<table width=100%% border=0 cellpadding=0 cellspacing=0><tr>\n");
+	wc_printf("<table width=\"100%%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\"><tr>\n");
 
-	wc_printf("<td align=center>");
+	wc_printf("<td align=\"center\">");
 
 	localtime_r(&previous_month, &tm);
 	wc_printf("<a href=\"readfwd?calview=month?year=%d?month=%d?day=1\">",
 		(int)(tm.tm_year)+1900, tm.tm_mon + 1);
-	wc_printf("<img align=middle src=\"static/prevdate_32x.gif\" border=0></A>\n");
+	wc_printf("<img alt=\"%s\" align=\"middle\" src=\"static/webcit_icons/essen/32x32/back.png\" border=\"0\"></a>\n", _("previous"));
 
 	wc_strftime(month_label, sizeof month_label, "%B", &tm);
 	wc_printf("&nbsp;&nbsp;"
-		"<font size=+1 color=\"#FFFFFF\">"
+		"<font size=\"+1\" color=\"#FFFFFF\">"
 		"%s %d"
 		"</font>"
 		"&nbsp;&nbsp;", month_label, year);
@@ -730,12 +730,12 @@ void calendar_brief_month_view(int year, int month, int day) {
 	localtime_r(&next_month, &tm);
 	wc_printf("<a href=\"readfwd?calview=month?year=%d?month=%d?day=1\">",
 		(int)(tm.tm_year)+1900, tm.tm_mon + 1);
-	wc_printf("<img align=middle src=\"static/nextdate_32x.gif\" border=0></A>\n");
+	wc_printf("<img alt=\"%s\" align=\"middle\" src=\"static/webcit_icons/essen/32x32/forward.png\" border=\"0\"></a>\n", _("next"));
 
 	wc_printf("</td></tr></table>\n");
 
 	/* Inner table (the real one) */
-	wc_printf("<table width=100%% border=0 cellpadding=1 cellspacing=1 "
+	wc_printf("<table width=\"100%%\" border=\"0\" cellpadding=\"1\" cellspacing=\"1\" "
 		"bgcolor=#EEEECC><TR>");
 	wc_printf("</tr>\n");
 	wc_printf("<tr><td colspan=\"100%%\">\n");
@@ -752,8 +752,8 @@ void calendar_brief_month_view(int year, int month, int day) {
 		if ((i % 7) == 0) {
 			wc_strftime(&weeknumber[0], sizeof(weeknumber), "%U", &tm);
 			wc_printf("<table border='0' bgcolor=\"#EEEECC\" width='100%%'> <tr><th colspan='4'>%s %s</th></tr>"
-				"   <tr><td>%s</td><td width=70%%>%s</td><td>%s</td><td>%s</td></tr>\n",
-				_("Week"), 
+				"   <tr><td>%s</td><td width='70%%'>%s</td><td>%s</td><td>%s</td></tr>\n",
+				_("Week"),
 				weeknumber,
 				_("Hours"),
 				_("Subject"),
@@ -761,11 +761,11 @@ void calendar_brief_month_view(int year, int month, int day) {
 				_("End")
 				);
 		}
-		
+
 		daycolor=((tm.tm_mon != month-1) ? "DDDDDD" :
 			((tm.tm_wday==0 || tm.tm_wday==6) ? "EEEECC" :
 				"FFFFFF"));
-		
+
 		/* Day Header */
 		wc_strftime(weekday_name, sizeof weekday_name, "%A", &tm);
 		wc_printf("<tr><td bgcolor='%s' colspan='1' align='left'> %s,%i."
@@ -787,15 +787,15 @@ void calendar_brief_month_view(int year, int month, int day) {
 	}
 
 	wc_printf("</table>"			/* end of inner table */
-		"</td></tr></table>"		/* end of outer table */
-		"</div>\n");
+		"</td></tr></table>\n"		/* end of outer table */
+	);
 }
 
 /*
  * Calendar week view -- not implemented yet, this is a stub function
  */
 void calendar_week_view(int year, int month, int day) {
-	wc_printf("<center><i>week view FIXME</i></center><br />\n");
+	wc_printf("<center><i>week view FIXME</i></center><br>\n");
 }
 
 
@@ -954,16 +954,16 @@ void calendar_day_view_display_events(time_t thetime,
 				);
                                 escputs((char *) icalproperty_get_comment(p));
 				wc_printf("<span class=\"tooltip\"><span class=\"btttop\"></span><span class=\"bttmiddle\">");
-                                wc_printf("<i>%s</i><br />",      _("All day event"));
-				wc_printf("<i>%s: %s</i><br />",  _("From"), Cal->from);
+                                wc_printf("<i>%s</i><br>",      _("All day event"));
+				wc_printf("<i>%s: %s</i><br>",  _("From"), Cal->from);
                                 wc_printf("<i>%s</i> ",           _("Summary:"));
                                 escputs((char *) icalproperty_get_comment(p));
-                                wc_printf("<br />");
+                                wc_printf("<br>");
 				q = icalcomponent_get_first_property(Cal->cal,ICAL_LOCATION_PROPERTY);
                                 if (q) {
                                         wc_printf("<i>%s</i> ", _("Location:"));
                                         escputs((char *)icalproperty_get_comment(q));
-                                        wc_printf("<br />");
+                                        wc_printf("<br>");
 				}
 				if (!icaltime_compare(t, end_t)) { /* one day only */
 					webcit_fmt_date(buf, 256, event_tt, DATEFMT_LOCALEDATE);
@@ -979,7 +979,7 @@ void calendar_day_view_display_events(time_t thetime,
                                 if (q) {
                                         wc_printf("<i>%s</i> ", _("Notes:"));
                                         escputs((char *)icalproperty_get_comment(q));
-                                        wc_printf("<br />");
+                                        wc_printf("<br>");
                                 }
 				wc_printf("</span><span class=\"bttbottom\"></span></span>");
                                 wc_printf("</a> <span>(");
@@ -998,16 +998,16 @@ void calendar_day_view_display_events(time_t thetime,
 				);
 				escputs((char *) icalproperty_get_comment(p));
 				wc_printf("<span class=\"tooltip\"><span class=\"btttop\"></span><span class=\"bttmiddle\">");
-                                wc_printf("<i>%s</i><br />",     _("Ongoing event"));
-				wc_printf("<i>%s: %s</i><br />", _("From"), Cal->from);
+                                wc_printf("<i>%s</i><br>",     _("Ongoing event"));
+				wc_printf("<i>%s: %s</i><br>", _("From"), Cal->from);
                                 wc_printf("<i>%s</i> ",          _("Summary:"));
                                 escputs((char *) icalproperty_get_comment(p));
-                                wc_printf("<br />");
+                                wc_printf("<br>");
                                 q = icalcomponent_get_first_property(Cal->cal,ICAL_LOCATION_PROPERTY);
                                 if (q) {
                                         wc_printf("<i>%s</i> ", _("Location:"));
                                         escputs((char *)icalproperty_get_comment(q));
-                                        wc_printf("<br />");
+                                        wc_printf("<br>");
 								}
                                 webcit_fmt_date(buf, 256, event_tt, DATEFMT_BRIEF);
                                 wc_printf("<i>%s</i> %s<br>", _("Starting date/time:"), buf);
@@ -1017,7 +1017,7 @@ void calendar_day_view_display_events(time_t thetime,
                                 if (q) {
                                         wc_printf("<i>%s</i> ", _("Notes:"));
                                         escputs((char *)icalproperty_get_comment(q));
-                                        wc_printf("<br />");
+                                        wc_printf("<br>");
                                 }
                                 wc_printf("</span><span class=\"bttbottom\"></span></span>");
 				wc_printf("</a> <span>(");
@@ -1080,15 +1080,15 @@ void calendar_day_view_display_events(time_t thetime,
 				);
 				escputs((char *) icalproperty_get_comment(p));
 				wc_printf("<span class=\"tooltip\"><span class=\"btttop\"></span><span class=\"bttmiddle\">");
-				wc_printf("<i>%s: %s</i><br />", _("From"), Cal->from);
+				wc_printf("<i>%s: %s</i><br>", _("From"), Cal->from);
                                 wc_printf("<i>%s</i> ",          _("Summary:"));
                                 escputs((char *) icalproperty_get_comment(p));
-                                wc_printf("<br />");
+                                wc_printf("<br>");
                                 q = icalcomponent_get_first_property(Cal->cal,ICAL_LOCATION_PROPERTY);
                                 if (q) {
                                         wc_printf("<i>%s</i> ", _("Location:"));
                                         escputs((char *)icalproperty_get_comment(q));
-                                        wc_printf("<br />");
+                                        wc_printf("<br>");
 								}
 				if (!icaltime_compare(t, end_t)) { /* one day only */
 					webcit_fmt_date(buf, 256, event_tt, DATEFMT_BRIEF);
@@ -1104,7 +1104,7 @@ void calendar_day_view_display_events(time_t thetime,
                                 if (q) {
                                         wc_printf("<i>%s</i> ", _("Notes:"));
                                         escputs((char *)icalproperty_get_comment(q));
-                                        wc_printf("<br />");
+                                        wc_printf("<br>");
                                 }
 				wc_printf("</span><span class=\"bttbottom\"></span></span>");
 				wc_printf("</a></dd>\n");
@@ -1141,7 +1141,7 @@ void calendar_day_view(int year, int month, int day) {
 		daystart = 9;
 		dayend = 17;
 	}
-	
+
 	/* Today's date */
 	memset(&d_tm, 0, sizeof d_tm);
 	d_tm.tm_year = year - 1900;
@@ -1165,8 +1165,6 @@ void calendar_day_view(int year, int month, int day) {
 	++tomorrow.day;
 	tomorrow = icaltime_normalize(tomorrow);
 
-	wc_printf("<div class=\"fix_scrollbar_bug\">");
-
 	/* Inner table (the real one) */
 	wc_printf("<table class=\"calendar\" id=\"inner_day\"><tr> \n");
 
@@ -1181,18 +1179,14 @@ void calendar_day_view(int year, int month, int day) {
 	if (hourlabel > (timeline - 2)) hourlabel = timeline - 2;
 
 	for (hour = 0; hour < daystart; ++hour) {	/* could do HEIGHT=xx */
-		wc_printf("<dt class=\"extrahour\" 	"
-			"style=\"		"
-			"position: absolute; 	"
-			"top: %dpx; left: 0px; 	"
-			"height: %dpx;		"
-			"font-size: %dpx;	"
-			"\" >			"
+		wc_printf("<dt class=\"extrahour\">"
 			"<a href=\"display_edit_event?msgnum=0"
 			"?calview=day?year=%d?month=%d?day=%d?hour=%d?minute=0\">",
+/* TODO: what have these been used for?
 			(hour * extratimeline ),
 			extratimeline,
 			extrahourlabel,
+*/
 			year, month, day, hour
 			);
 
@@ -1212,18 +1206,14 @@ void calendar_day_view(int year, int month, int day) {
 	gap = daystart * extratimeline;
 
         for (hour = daystart; hour <= dayend; ++hour) {       /* could do HEIGHT=xx */
-                wc_printf("<dt class=\"hour\"     "
-                        "style=\"               "
-                        "position: absolute;    "
-                        "top: %ldpx; left: 0px; "
-                        "height: %dpx;          "
-			"font-size: %dpx;	"
-                        "\" >                   "
+                wc_printf("<dt class=\"hour\">"
                         "<a href=\"display_edit_event?msgnum=0?calview=day"
                         "?year=%d?month=%d?day=%d?hour=%d?minute=0\">",
+/*TODO: what have these been used for?
                         gap + ((hour - daystart) * timeline ),
 			timeline,
 			hourlabel,
+*/
                         year, month, day, hour
 			);
 
@@ -1243,18 +1233,14 @@ void calendar_day_view(int year, int month, int day) {
 	gap = gap + ((dayend - daystart + 1) * timeline);
 
         for (hour = (dayend + 1); hour < 24; ++hour) {       /* could do HEIGHT=xx */
-                wc_printf("<dt class=\"extrahour\"     "
-                        "style=\"               "
-                        "position: absolute;    "
-                        "top: %ldpx; left: 0px; "
-                        "height: %dpx;          "
-			"font-size: %dpx;	"
-                        "\" >                   "
+                wc_printf("<dt class=\"extrahour\">"
                         "<a href=\"display_edit_event?msgnum=0?calview=day"
                         "?year=%d?month=%d?day=%d?hour=%d?minute=0\">",
+/*TODO: what have these been used for?
                         gap + ((hour - dayend - 1) * extratimeline ),
 			extratimeline,
 			extrahourlabel,
+*/
                         year, month, day, hour
                 );
 
@@ -1289,52 +1275,48 @@ void calendar_day_view(int year, int month, int day) {
 
 	wc_printf("</td>");	/* end extra on the middle */
 
-	wc_printf("<td width=20%% align=center valign=top>");	/* begin stuff-on-the-right */
+	wc_printf("<td width='20%%' align=\"center\" valign=top>");	/* begin stuff-on-the-right */
 
 	/* Begin todays-date-with-left-and-right-arrows */
-	wc_printf("<table border=0 width=100%% "
-		"cellspacing=0 cellpadding=0 bgcolor=\"#FFFFFF\">\n");
+	wc_printf("<table border=\"0\" width=\"100%%\" "
+		"cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#FFFFFF\">\n");
 	wc_printf("<tr>");
 
-	/* Left arrow */	
-	wc_printf("<td align=center>");
+	/* Left arrow */
+	wc_printf("<td align=\"center\">");
 	wc_printf("<a href=\"readfwd?calview=day?year=%d?month=%d?day=%d\">",
 		yesterday.year, yesterday.month, yesterday.day);
-	wc_printf("<img align=middle src=\"static/prevdate_32x.gif\" border=0></A>");
+	wc_printf("<img alt=\"previous\" align=\"middle\" src=\"static/webcit_icons/essen/32x32/back.png\" border=\"0\"></a>");
 	wc_printf("</td>");
 
 	wc_strftime(d_str, sizeof d_str,
-		"<td align=center>"
-		"<font size=+2>%A</font><br />"
-		"<font size=+2>%B</font><br />"
-		"<font size=+3>%d</font><br />"
-		"<font size=+2>%Y</font><br />"
+		"<td align=\"center\">"
+		"<font size='+2'>%A</font><br>"
+		"<font size='+2'>%B</font><br>"
+		"<font size='+3'>%d</font><br>"
+		"<font size='+2'>%Y</font><br>"
 		"</td>",
 		&d_tm
 		);
 	wc_printf("%s", d_str);
 
 	/* Right arrow */
-	wc_printf("<td align=center>");
+	wc_printf("<td align=\"center\">");
 	wc_printf("<a href=\"readfwd?calview=day?year=%d?month=%d?day=%d\">",
 		tomorrow.year, tomorrow.month, tomorrow.day);
-	wc_printf("<img align=middle src=\"static/nextdate_32x.gif\""
-		" border=0></a>\n");
+	wc_printf("<img alt=\"%s\" align=\"middle\" src=\"static/webcit_icons/essen/32x32/forward.png\""
+		" border=\"0\"></a>\n", _("next"));
 	wc_printf("</td>");
 
 	wc_printf("</tr></table>\n");
 	/* End todays-date-with-left-and-right-arrows */
 
 	/* Embed a mini month calendar in this space */
-	wc_printf("<br />\n");
+	wc_printf("<br>\n");
 	embeddable_mini_calendar(year, month);
 
-	wc_printf("</font></center>\n");
-
 	wc_printf("</td></tr>");			/* end stuff-on-the-right */
-
-	wc_printf("</table>"			/* end of inner table */
-		"</div>");
+	wc_printf("</table>\n");			/* end of inner table */
 }
 
 
@@ -1403,7 +1385,7 @@ int calendar_summary_view(void) {
 						wc_printf("<a href=\"display_edit_task"
 							"?msgnum=%ld"
 							"?return_to_summary=1"
-							"?gotofirst=",
+							"?go=",
 							Cal->cal_msgnum
 						);
 						escputs(ChrPtr(WCC->CurRoom.name));
@@ -1416,7 +1398,7 @@ int calendar_summary_view(void) {
 							"?year=%d"
 							"?month=%d"
 							"?day=%d"
-							"?gotofirst=",
+							"?go=",
 							Cal->cal_msgnum,
 							today_tm.tm_year + 1900,
 							today_tm.tm_mon + 1,
@@ -1429,7 +1411,7 @@ int calendar_summary_view(void) {
 					if (!all_day_event) {
 						wc_printf(" (%s)", timestring);
 					}
-					wc_printf("</a><br />\n");
+					wc_printf("</a><br>\n");
 					++num_displayed;
 				}
 			}
@@ -1443,10 +1425,10 @@ int calendar_summary_view(void) {
 /*
  * Parse the URL variables in order to determine the scope and display of a calendar view
  */
-int calendar_GetParamsGetServerCall(SharedMessageStatus *Stat, 
-				    void **ViewSpecific, 
-				    long oper, 
-				    char *cmd, 
+int calendar_GetParamsGetServerCall(SharedMessageStatus *Stat,
+				    void **ViewSpecific,
+				    long oper,
+				    char *cmd,
 				    long len)
 {
 	wcsession *WCC = WC;
@@ -1464,7 +1446,7 @@ int calendar_GetParamsGetServerCall(SharedMessageStatus *Stat,
 	Stat->load_seen = 1;
 	strcpy(cmd, "MSGS ALL");
 	Stat->maxmsgs = 32767;
-	
+
 	/* In case no date was specified, go with today */
 	now = time(NULL);
 	localtime_r(&now, &tm);
@@ -1529,8 +1511,8 @@ int calendar_GetParamsGetServerCall(SharedMessageStatus *Stat,
 /*
  * Render a calendar view from data previously loaded into memory
  */
-int calendar_RenderView_or_Tail(SharedMessageStatus *Stat, 
-				void **ViewSpecific, 
+int calendar_RenderView_or_Tail(SharedMessageStatus *Stat,
+				void **ViewSpecific,
 				long oper)
 {
 	wcsession *WCC = WC;
@@ -1556,7 +1538,9 @@ int calendar_RenderView_or_Tail(SharedMessageStatus *Stat,
 	return 0;
 }
 
-
-
-
-
+void 
+InitModule_CALENDAR_VIEW
+(void)
+{
+	WebcitAddUrlHandler(HKEY("mini_calendar"), "", 0, ajax_mini_calendar, AJAX);
+}
