@@ -1,12 +1,10 @@
 /*
- * $Id$
- *
  * Output an HTML message, modifying it slightly to make sure it plays nice
  * with the rest of our web framework.
  *
- * Copyright (c) 2005-2010 by the citadel.org team
+ * Copyright (c) 2005-2011 by the citadel.org team
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is open source software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
@@ -138,7 +136,7 @@ void output_html(const char *supplied_charset, int treat_as_wiki, int msgnum, St
 			StrBufAppendPrintf(Target, _("realloc() error! couldn't get %d bytes: %s"),
 					buffer_length + 1,
 					strerror(errno));
-			StrBufAppendPrintf(Target, "</b><br /><br />\n");
+			StrBufAppendPrintf(Target, "</b><br><br>\n");
 			while (serv_getln(buf, sizeof buf), strcmp(buf, "000")) {
 				/** flush */
 			}
@@ -303,10 +301,10 @@ void output_html(const char *supplied_charset, int treat_as_wiki, int msgnum, St
 			&& (strcasecmp(charset, "UTF-8"))
 			&& (strcasecmp(charset, ""))
 	   ) {
-		lprintf(9, "Converting %s to UTF-8\n", charset);
+		syslog(9, "Converting %s to UTF-8\n", charset);
 		ctdl_iconv_open("UTF-8", charset, &ic);
 		if (ic == (iconv_t)(-1) ) {
-			lprintf(5, "%s:%d iconv_open() failed: %s\n",
+			syslog(5, "%s:%d iconv_open() failed: %s\n",
 					__FILE__, __LINE__, strerror(errno));
 		}
 	}
@@ -346,7 +344,7 @@ void output_html(const char *supplied_charset, int treat_as_wiki, int msgnum, St
 	 */
 
 	if (converted_msg == NULL) {
-		StrBufAppendPrintf(Target, "Error %d: %s<br />%s:%d", errno, strerror(errno), __FILE__, __LINE__);
+		StrBufAppendPrintf(Target, "Error %d: %s<br>%s:%d", errno, strerror(errno), __FILE__, __LINE__);
 		goto BAIL;
 	}
 
@@ -395,7 +393,12 @@ void output_html(const char *supplied_charset, int treat_as_wiki, int msgnum, St
 				StrBufAppendPrintf(converted_msg, new_window);
 				ptr = &ptr[8];
 			}
-			else if ( (treat_as_wiki) && (strncasecmp(ptr, "<a href=\"wiki?", 14)) ) {
+			else if (
+				(treat_as_wiki)
+				&& (strncasecmp(ptr, "<a href=\"wiki?", 14))
+				&& (strncasecmp(ptr, "<a href=\"dotgoto?", 17))
+				&& (strncasecmp(ptr, "<a href=\"knrooms?", 17))
+			) {
 				content_length += 64;
 				StrBufAppendPrintf(converted_msg, "<a href=\"wiki?page=");
 				ptr = &ptr[9];
@@ -414,9 +417,9 @@ void output_html(const char *supplied_charset, int treat_as_wiki, int msgnum, St
 			 * and then ended the message)
 			 */
 			if (!tag_end) {
-				lprintf(9, "tag_end is null and ptr is:\n");
-				lprintf(9, "%s\n", ptr);
-				lprintf(9, "Theoretical bytes remaining: %d\n", msgend - ptr);
+				syslog(9, "tag_end is null and ptr is:\n");
+				syslog(9, "%s\n", ptr);
+				syslog(9, "Theoretical bytes remaining: %d\n", (int)(msgend - ptr));
 			}
 
 			src=strstr(ptr, "src=\"cid:");
@@ -556,7 +559,7 @@ void output_html(const char *supplied_charset, int treat_as_wiki, int msgnum, St
 	StrBufAppendBuf(Target, converted_msg, 0);
 
 BAIL:	/** A little trailing vertical whitespace... */
-	StrBufAppendPrintf(Target, "<br /><br />\n");
+	StrBufAppendPrintf(Target, "<br><br>\n");
 
 	/** Now give back the memory */
 	FreeStrBuf(&converted_msg);
@@ -641,7 +644,7 @@ void url(char *buf, size_t bufsize)
 	start = NULL;
 	len = strlen(buf);
 	if (len > bufsize) {
-		lprintf(1, "URL: content longer than buffer!");
+		syslog(1, "URL: content longer than buffer!");
 		return;
 	}
 	end = buf + len;
@@ -677,7 +680,7 @@ void url(char *buf, size_t bufsize)
 	
 	UrlLen = end - start;
 	if (UrlLen > sizeof(urlbuf)){
-		lprintf(1, "URL: content longer than buffer!");
+		syslog(1, "URL: content longer than buffer!");
 		return;
 	}
 	memcpy(urlbuf, start, UrlLen);
@@ -690,7 +693,7 @@ void url(char *buf, size_t bufsize)
 			  "%ca href=%c%s%c TARGET=%c%s%c%c%s%c/A%c",
 			  LB, QU, urlbuf, QU, QU, TARGET, QU, RB, urlbuf, LB, RB);
 	if (outpos >= sizeof(outbuf) - Offset) {
-		lprintf(1, "URL: content longer than buffer!");
+		syslog(1, "URL: content longer than buffer!");
 		return;
 	}
 
@@ -698,7 +701,7 @@ void url(char *buf, size_t bufsize)
 	if (TrailerLen > 0)
 		memcpy(outbuf + Offset + outpos, end, TrailerLen);
 	if (Offset + outpos + TrailerLen > bufsize) {
-		lprintf(1, "URL: content longer than buffer!");
+		syslog(1, "URL: content longer than buffer!");
 		return;
 	}
 	memcpy (buf, outbuf, Offset + outpos + TrailerLen);
