@@ -1,7 +1,21 @@
 /*
- * $Id$
- *
  * Web forms for handling mailing list subscribe/unsubscribe requests.
+ *
+ * Copyright (c) 1996-2011 by the citadel.org team
+ *
+ * This program is open source software.  You can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "webcit.h"
@@ -9,6 +23,7 @@
 /*
  * List subscription handling
  */
+#ifndef EXPERIMENTAL_LISTSUB
 void do_listsub(void)
 {
 	char cmd[256];
@@ -33,7 +48,7 @@ void do_listsub(void)
 
 	wc_printf("<HTML><HEAD>\n"
 		"<meta name=\"MSSmartTagsPreventParsing\" content=\"TRUE\" />\n"
-		"<link href=\"static/webcit.css\" rel=\"stylesheet\" type=\"text/css\">\n"
+		"<link href=\"static/styles/webcit.css\" rel=\"stylesheet\" type=\"text/css\">\n"
 		"<TITLE>\n"
 	);
 	wc_printf(_("List subscription"));
@@ -48,20 +63,20 @@ void do_listsub(void)
 	wc_printf("<div align=center>");
 	wc_printf("<table border=0 width=75%%><tr><td>");
 
-	svput("BOXTITLE", WCS_STRING, _("List subscribe/unsubscribe"));
-	do_template("beginboxx", NULL);
+	do_template("box_begin_1");
+	StrBufAppendBufPlain(WC->WBuf, _("List subscribe/unsubscribe"), -1, 0);
+	do_template("box_begin_2");
 	wc_printf("<div align=center><br>");
 
 	/*
 	 * Subscribe command
 	 */
 	if (!strcasecmp(cmd, "subscribe")) {
-		serv_printf("SUBS subscribe|%s|%s|%s|%s://%s/listsub",
+		serv_printf("SUBS subscribe|%s|%s|%s|%s/listsub",
 			room,
 			email,
 			subtype,
-			(is_https ? "https" : "http"),
-			    ChrPtr(WC->Hdr->HR.http_host)
+			ChrPtr(site_prefix)
 		);
 		serv_getln(buf, sizeof buf);
 		if (buf[0] == '2') {
@@ -79,16 +94,16 @@ void do_listsub(void)
 				"your subscription.  This extra step is for "
 				"your protection, as it prevents others from "
 				"being able to subscribe you to lists "
-				"without your consent.<br /><br />"
+				"without your consent.<br><br>"
 				"Please click on the link which is being "
 				"e-mailed to you and your subscription will "
-				"be confirmed.<br />\n"),
+				"be confirmed.<br>\n"),
 				escaped_email, escaped_room);
 			wc_printf("<a href=\"listsub\">%s</A></CENTER>\n", _("Go back..."));
 		}
 		else {
 			wc_printf("<FONT SIZE=+1><B>ERROR: %s</B>"
-				"</FONT><br /><br />\n",
+				"</FONT><br><br>\n",
 				&buf[4]);
 			goto FORM;
 		}
@@ -98,11 +113,10 @@ void do_listsub(void)
 	 * Unsubscribe command
 	 */
 	else if (!strcasecmp(cmd, "unsubscribe")) {
-		serv_printf("SUBS unsubscribe|%s|%s|%s://%s/listsub",
-			    room,
-			    email,
-			    (is_https ? "https" : "http"),
-			    ChrPtr(WC->Hdr->HR.http_host)
+		serv_printf("SUBS unsubscribe|%s|%s|%s/listsub",
+			room,
+			email,
+			ChrPtr(site_prefix)
 		);
 		serv_getln(buf, sizeof buf);
 		if (buf[0] == '2') {
@@ -117,16 +131,16 @@ void do_listsub(void)
 				"your unsubscription.  This extra step is for "
 				"your protection, as it prevents others from "
 				"being able to unsubscribe you from "
-				"lists without your consent.<br /><br />"
+				"lists without your consent.<br><br>"
 				"Please click on the link which is being "
 				"e-mailed to you and your unsubscription will "
-				"be confirmed.<br />\n"
+				"be confirmed.<br>\n"
 				"<a href=\"listsub\">Back...</A></CENTER>\n"
 			);
 		}
 		else {
 			wc_printf("<FONT SIZE=+1><B>ERROR: %s</B>"
-				"</FONT><br /><br />\n",
+				"</FONT><br><br>\n",
 				&buf[4]);
 			goto FORM;
 		}
@@ -157,7 +171,7 @@ void do_listsub(void)
 				"The error returned by the server was: "
 			);
 		}
-		wc_printf("%s</CENTER><br />\n", &buf[4]);
+		wc_printf("%s</CENTER><br>\n", &buf[4]);
 	}
 
 	/*
@@ -211,26 +225,151 @@ FORM:		wc_printf("<form method=\"POST\" action=\"listsub\">\n");
 			" one additional web link to click on for final "
 			"confirmation.  This extra step is for your "
 			"protection, as it prevents others from being able to "
-			"subscribe or unsubscribe you to lists.<br />\n"
+			"subscribe or unsubscribe you to lists.<br>\n"
 		);
 
 	}
 
 	wc_printf("</div>");
-	do_template("endbox", NULL);
+	do_template("box_end");
 	wc_printf("</td></tr></table></div>");
 
 	wc_printf("</BODY></HTML>\n");
 	wDumpContent(0);
 	end_webcit_session();
 }
+#endif
 
+int Conditional_LISTSUB_EXECUTE_SUBSCRIBE(StrBuf *Target, WCTemplputParams *TP)
+{
+	int rc;
+	StrBuf *Line;
+	const char *ImpMsg;
+	const StrBuf *Room, *Email, *SubType;
+	
+	Room = sbstr("room");
+	if (Room == NULL)
+	{
+		ImpMsg = _("You need to specify the mailinglist to subscribe to.");
+		AppendImportantMessage(ImpMsg, -1);
+		return 0;
+	}
+	Email = sbstr("email");
+	if (Email == NULL)
+	{
+		ImpMsg = _("You need to specify the email address you'd like to subscribe with.");
+		AppendImportantMessage(ImpMsg, -1);
+		return 0;
+	}
+	SubType = sbstr("subtype");
 
+	Line = NewStrBuf();
+	serv_printf("SUBS subscribe|%s|%s|%s|%s/listsub",
+		    ChrPtr(Room),
+		    ChrPtr(Email),
+		    ChrPtr(SubType),
+		    ChrPtr(site_prefix)
+		);
+	StrBuf_ServGetln(Line);
+	rc = GetServerStatusMsg(Line, NULL, 1, 2);
+	FreeStrBuf(&Line);
+	if (rc == 2)
+		putbstr("__FAIL", NewStrBufPlain(HKEY("1")));
+	return rc == 2;
+}
+
+int Conditional_LISTSUB_EXECUTE_UNSUBSCRIBE(StrBuf *Target, WCTemplputParams *TP)
+{
+	int rc;
+	StrBuf *Line;
+	const char *ImpMsg;
+	const StrBuf *Room, *Email;
+	
+	Room = sbstr("room");
+	if (Room == NULL)
+	{
+		ImpMsg = _("You need to specify the mailinglist to subscribe to.");
+		AppendImportantMessage(ImpMsg, -1);
+		return 0;
+	}
+	Email = sbstr("email");
+	if (Email == NULL)
+	{
+		ImpMsg = _("You need to specify the email address you'd like to subscribe with.");
+		AppendImportantMessage(ImpMsg, -1);
+		return 0;
+	}
+
+	serv_printf("SUBS unsubscribe|%s|%s|%s/listsub",
+		    ChrPtr(Room),
+		    ChrPtr(Email),
+		    ChrPtr(site_prefix)
+		);
+	Line = NewStrBuf();
+	StrBuf_ServGetln(Line);
+	rc = GetServerStatusMsg(Line, NULL, 1, 2);
+	FreeStrBuf(&Line);
+	if (rc == 2)
+		putbstr("__FAIL", NewStrBufPlain(HKEY("1")));
+	return rc == 2;
+}
+
+int Conditional_LISTSUB_EXECUTE_CONFIRM_SUBSCRIBE(StrBuf *Target, WCTemplputParams *TP)
+{
+	int rc;
+	StrBuf *Line;
+	const char *ImpMsg;
+	const StrBuf *Room, *Token;
+	
+	Room = sbstr("room");
+	if (Room == NULL)
+	{
+		ImpMsg = _("You need to specify the mailinglist to subscribe to.");
+		AppendImportantMessage(ImpMsg, -1);
+		return 0;
+	}
+	Token = sbstr("token");
+	if (Room == NULL)
+	{
+		ImpMsg = _("You need to specify the mailinglist to subscribe to.");
+		AppendImportantMessage(ImpMsg, -1);
+		return 0;
+	}
+
+	Line = NewStrBuf();
+	serv_printf("SUBS confirm|%s|%s",
+		    ChrPtr(Room),
+		    ChrPtr(Token)
+		);
+	StrBuf_ServGetln(Line);
+	rc = GetServerStatusMsg(Line, NULL, 1, 2);
+	FreeStrBuf(&Line);
+	if (rc == 2)
+		putbstr("__FAIL", NewStrBufPlain(HKEY("1")));
+	return rc == 2;
+}
+
+#ifdef EXPERIMENTAL_LISTSUB
+void do_listsub(void)
+{
+	if (!havebstr("cmd"))
+	{
+		putbstr("cmd", NewStrBufPlain(HKEY("")));
+	}
+	output_headers(1, 0, 0, 0, 1, 0);
+	do_template("listsub_display");
+	end_burst();
+}
+#endif
 
 void 
 InitModule_LISTSUB
 (void)
 {
+	RegisterConditional(HKEY("COND:LISTSUB:EXECUTE:SUBSCRIBE"), 0, Conditional_LISTSUB_EXECUTE_SUBSCRIBE,  CTX_NONE);
+	RegisterConditional(HKEY("COND:LISTSUB:EXECUTE:UNSUBSCRIBE"), 0, Conditional_LISTSUB_EXECUTE_UNSUBSCRIBE,  CTX_NONE);
+	RegisterConditional(HKEY("COND:LISTSUB:EXECUTE:CONFIRM:SUBSCRIBE"), 0, Conditional_LISTSUB_EXECUTE_CONFIRM_SUBSCRIBE,  CTX_NONE);
+
 	WebcitAddUrlHandler(HKEY("listsub"), "", 0, do_listsub, ANONYMOUS|COOKIEUNNEEDED|FORCE_SESSIONCLOSE);
 
 
