@@ -1,10 +1,6 @@
-/*
- * $Id$
- *
- */
 
 #include "webcit.h"
-#include "groupdav.h"
+#include "dav.h"
 #include "webserver.h"
 
 int pastel_palette[9][3] = {
@@ -34,7 +30,6 @@ struct vnote *vnote_new_from_msg(long msgnum,int unread)
 	char mime_filename[256];
 	char mime_content_type[256];
 	char mime_disposition[256];
-	int mime_length;
 	char relevant_partnum[256];
 	int phase = 0;				/* 0 = citadel headers, 1 = mime headers, 2 = body */
 	char msg4_content_type[256] = "";
@@ -68,7 +63,6 @@ struct vnote *vnote_new_from_msg(long msgnum,int unread)
 				extract_token(mime_partnum, &bptr[5], 2, '|', sizeof mime_partnum);
 				extract_token(mime_disposition, &bptr[5], 3, '|', sizeof mime_disposition);
 				extract_token(mime_content_type, &bptr[5], 4, '|', sizeof mime_content_type);
-				mime_length = extract_int(&bptr[5], 5);
 
 				if (!strcasecmp(mime_content_type, "text/vnote")) {
 					strcpy(relevant_partnum, mime_partnum);
@@ -296,7 +290,7 @@ void ajax_update_note(void) {
  *
  * msgnum = Message number on the local server of the note to be displayed
  */
-////TODO: falscher hook
+/*TODO: wrong hook */
 int notes_LoadMsgFromServer(SharedMessageStatus *Stat, 
 			    void **ViewSpecific, 
 			    message_summary* Msg, 
@@ -406,7 +400,7 @@ void tmpl_vcard_put_bgcolor(StrBuf *Target, WCTemplputParams *TP)
 void tmpl_vcard_put_message(StrBuf *Target, WCTemplputParams *TP)
 {
 	struct vnote *v = (struct vnote *) CTX;
-	StrEscAppend(Target, NULL, v->body, 0, 0); ///TODO?
+	StrEscAppend(Target, NULL, v->body, 0, 0); /*TODO?*/
 }
 
 void tmpl_vcard_put_uid(StrBuf *Target, WCTemplputParams *TP)
