@@ -1,11 +1,9 @@
 /*
- * $Id$
- *
  * This module handles multiuser chat.
  *
- * Copyright (c) 1996-2010 by the citadel.org team
+ * Copyright (c) 1996-2011 by the citadel.org team
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is open source software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
@@ -17,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "webcit.h"
@@ -34,7 +32,7 @@ void do_chat(void)
 	WC->last_chat_user[0] = 0;
 
 	output_headers(1, 1, 1, 0, 0, 0);
-	do_template("roomchat", NULL);
+	do_template("roomchat");
 
 	serv_puts("RCHT enter");
 	serv_getln(buf, sizeof buf);
@@ -134,7 +132,7 @@ void chat_rwho(void) {
 			else {
 				wc_printf("<span class=\"chat_notmyname_class\">");
 			}
-			wc_printf("<img src=\"static/citadelchat_16x.gif\">");
+			wc_printf("<img src=\"static/webcit_icons/essen/16x16/chat.png\">");
 			escputs(buf);
 			wc_printf("</span><br>\n");
 		}
