@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * Administrative screen for site-wide configuration
  */
 
@@ -37,8 +35,7 @@ void LoadExpirePolicy(GPEXWhichPolicy which)
 		WCC->Policy[which].expire_value = StrBufExtractNext_long(Buf, &Pos, '|');
 	}
 	else if (State == 550)
-		StrBufAppendBufPlain(WCC->ImportantMsg,
-				     _("Higher access is required to access this function."), -1, 0);
+		AppendImportantMessage(_("Higher access is required to access this function."), -1);
 	FreeStrBuf(&Buf);
 }
 
@@ -56,8 +53,7 @@ void SaveExpirePolicyFromHTTP(GPEXWhichPolicy which)
 	StrBuf_ServGetln(Buf);
 	GetServerStatus(Buf, &State);
 	if (State == 550)
-		StrBufAppendBufPlain(WC->ImportantMsg,
-				     _("Higher access is required to access this function."), -1, 0);
+		AppendImportantMessage(_("Higher access is required to access this function."), -1);
 	FreeStrBuf(&Buf);
 }
 
@@ -111,7 +107,7 @@ void LoadZoneFiles(void)
 	Put(ZoneHash, HKEY("UTC"), ZName, HFreeStrBuf);
 	zones = icaltimezone_get_builtin_timezones();
 	for (z = 0; z < zones->num_elements; ++z) {
-		/* lprintf(9, "Location: %-40s tzid: %s\n",
+		/* syslog(9, "Location: %-40s tzid: %s\n",
 			icaltimezone_get_location(icalarray_element_at(zones, z)),
 			icaltimezone_get_tzid(icalarray_element_at(zones, z))
 		); */
@@ -201,7 +197,8 @@ CfgMapping ServerConfig[] = {
 	{CFG_STR, HKEY("c_xmpp_s2s_port")},
 	{CFG_STR, HKEY("c_pop3_fetch")},
 	{CFG_STR, HKEY("c_pop3_fastest")},
-	{CFG_YES , HKEY("c_spam_flag_only")}
+	{CFG_YES , HKEY("c_spam_flag_only")},
+	{CFG_YES , HKEY("c_guest_logins")}
 };
 
 
@@ -213,7 +210,6 @@ void load_siteconfig(void)
 	wcsession *WCC = WC;
 	StrBuf *Buf;
 	HashList *Cfg;
-	char buf[SIZ];
 	long len;
 	int i;
 	
@@ -221,10 +217,17 @@ void load_siteconfig(void)
 		WCC->ServCfg = NewHash(1, NULL);
 	Cfg = WCC->ServCfg;
 
-	serv_printf("CONF get");
-	serv_getln(buf, sizeof buf);
-	i = 0;
 	Buf = NewStrBuf();
+
+	serv_printf("CONF get");
+	StrBuf_ServGetln(Buf);
+	if (GetServerStatus(Buf, NULL) != 1) {
+		StrBufCutLeft(Buf, 4);
+		AppendImportantMessage(SKEY(Buf));
+		return;
+		
+	}
+	i = 0;
 	while (len = StrBuf_ServGetln(Buf),
 	       (len >= 0) && 
 	       (i <= (sizeof(ServerConfig) / sizeof(CfgMapping))) &&
@@ -241,6 +244,14 @@ void load_siteconfig(void)
 		else
 			Buf = NULL;			
 	}
+	if (strcmp(ChrPtr(Buf), "000")!=0)
+	{
+		/* WHOOOOPSI??? burn the lines we don't understand */
+		while ((len = StrBuf_ServGetln(Buf),
+			strcmp(ChrPtr(Buf), "000"))) {}
+		AppendImportantMessage(_("WARNING: Failed to parse Server Config; do you run a to new citserver?"), -1);
+		return;
+	}
 	FreeStrBuf(&Buf);
 
 	LoadExpirePolicy(sitepolicy);
@@ -256,20 +267,23 @@ void siteconfig(void)
 {
 	wcsession *WCC = WC;
 	int i;
-	char buf[256];
+	StrBuf *Line;
 
 	if (strlen(bstr("ok_button")) == 0) {
 		display_aide_menu();
 		return;
 	}
+	Line = NewStrBuf();
 	serv_printf("CONF set");
-	serv_getln(buf, sizeof buf);
-	if (buf[0] != '4') {
-		safestrncpy(WCC->ImportantMessage, &buf[4], sizeof WCC->ImportantMessage);
+	StrBuf_ServGetln(Line);
+	if (GetServerStatusMsg(Line, NULL, 1, 4) != 4) {
 		display_aide_menu();
+		FreeStrBuf(&Line);
 		return;
 	}
 
+	FreeStrBuf(&Line);
+
 	for (i=0; i < (sizeof(ServerConfig) / sizeof(CfgMapping)); i ++)
 	{
 		switch (ServerConfig[i].type) {
@@ -297,8 +311,7 @@ void siteconfig(void)
 	FreeStrBuf(&WCC->serv_info->serv_default_cal_zone);
 	WCC->serv_info->serv_default_cal_zone = NewStrBufDup(sbstr("c_default_cal_zone"));
 
-	safestrncpy(WCC->ImportantMessage, _("Your system configuration has been updated."),
-		sizeof WCC->ImportantMessage);
+	AppendImportantMessage(_("Your system configuration has been updated."), -1);
 	DeleteHash(&WCC->ServCfg);
 	display_aide_menu();
 }
@@ -350,31 +363,25 @@ int ConditionalServCfg(StrBuf *Target, WCTemplputParams *TP)
 	else return 0;
 }
 
-int ConditionalServCfgSubst(StrBuf *Target, WCTemplputParams *TP)
+int ConditionalServCfgCTXStrBuf(StrBuf *Target, WCTemplputParams *TP)
 {
 	wcsession *WCC = WC;
 	void *vBuf;
 	StrBuf *Buf;
+	StrBuf *ZoneToCheck = (StrBuf*) CTX;
 
-	if (WCC->is_aide) {
+	if ((WCC->is_aide) || (ZoneToCheck == NULL)) {
 		if (WCC->ServCfg == NULL)
 			load_siteconfig();
 		GetHash(WCC->ServCfg, TKEY(2), &vBuf);
 		if (vBuf == NULL) return 0;
 		Buf = (StrBuf*) vBuf;
 
-		return CompareSubstToStrBuf(Buf, TP->Tokens->Params[3]);
+		return strcmp(ChrPtr(Buf), ChrPtr(ZoneToCheck)) == 0;
 	}
 	else return 0;
 }
 
-void CfgZoneTempl(StrBuf *TemplBuffer, WCTemplputParams *TP)
-{
-	StrBuf *Zone = (StrBuf*) CTX;
-
-	SVPutBuf("ZONENAME", Zone, 1);
-}
-
 void 
 InitModule_SITECONFIG
 (void)
@@ -383,8 +390,8 @@ InitModule_SITECONFIG
 
 	RegisterNamespace("SERV:CFG", 1, 2, tmplput_servcfg, NULL, CTX_NONE);
 	RegisterConditional(HKEY("COND:SERVCFG"), 3, ConditionalServCfg, CTX_NONE);
-	RegisterConditional(HKEY("COND:SERVCFG:SUBST"), 4, ConditionalServCfgSubst, CTX_NONE);
-	RegisterIterator("PREF:ZONE", 0, ZoneHash, NULL, CfgZoneTempl, NULL, CTX_PREF, CTX_NONE, IT_NOFLAG);
+	RegisterConditional(HKEY("COND:SERVCFG:CTXSTRBUF"), 4, ConditionalServCfgCTXStrBuf, CTX_STRBUF);
+	RegisterIterator("PREF:ZONE", 0, ZoneHash, NULL, NULL, NULL, CTX_STRBUF, CTX_NONE, IT_NOFLAG);
 
 	REGISTERTokenParamDefine(roompolicy);
 	REGISTERTokenParamDefine(floorpolicy);
