@@ -1,6 +1,4 @@
 /* 
- * $Id$
- *
  * Display the outbound SMTP queue
  */
 
@@ -106,7 +104,7 @@ void display_queue_msg(long msgnum)
 				if (!IsEmptyStr(thisrecp)) {
 					stresc(thismsg, sizeof thismsg, thisrecp, 1, 1);
 					if (!IsEmptyStr(thisdsn)) {
-						strcat(thismsg, "<br />&nbsp;&nbsp;<i>");
+						strcat(thismsg, "<br>&nbsp;&nbsp;<i>");
 						stresc(&thismsg[strlen(thismsg)], sizeof thismsg,
 							thisdsn, 1, 1);
 						strcat(thismsg, "</i>");
@@ -115,7 +113,7 @@ void display_queue_msg(long msgnum)
 
 					if ((recipients_len + thismsg_len + 100) < sizeof recipients) {
 						if (!IsEmptyStr(recipients)) {
-							strcpy(&recipients[recipients_len], "<br />");
+							strcpy(&recipients[recipients_len], "<br>");
 							recipients_len += 6;
 						}
 						strcpy(&recipients[recipients_len], thismsg);
@@ -130,7 +128,7 @@ void display_queue_msg(long msgnum)
 	}
 
 	wc_printf("<tr><td>");
-	wc_printf("%ld<br />", msgnum);
+	wc_printf("%ld<br>", msgnum);
 	wc_printf(" <a href=\"javascript:DeleteSMTPqueueMsg(%ld,%ld);\">%s</a>", 
 		msgnum, msgid, _("(Delete)")
 	);
@@ -212,15 +210,15 @@ void display_smtpqueue_inner_div(void) {
 
 		}
 		else {
-			wc_printf("<br /><br /><div align=\"center\">");
+			wc_printf("<br><br><div align=\"center\">");
 			wc_printf(_("The queue is empty."));
-			wc_printf("</div><br /><br />");
+			wc_printf("</div><br><br>");
 		}
 	}
 	else {
-		wc_printf("<br /><br /><div align=\"center\">");
+		wc_printf("<br><br><div align=\"center\">");
 		wc_printf(_("You do not have permission to view this resource."));
-		wc_printf("</div><br /><br />");
+		wc_printf("</div><br><br>");
 	}
 	output_headers(0, 0, 0, 0, 0, 0);
 	end_burst();
@@ -241,17 +239,16 @@ void display_smtpqueue(void)
 
 	wc_printf("<div id=\"content\" class=\"service\">\n");
 
-	wc_printf("<div class=\"fix_scrollbar_bug\">"
-		"<table class=\"smtpqueue_background\">"
+	wc_printf("<table class=\"smtpqueue_background\">"
 		"<tr><td valign=top>\n");
 
 	wc_printf("<div id=\"smtpqueue_inner_div\">"
-		"<div align=\"center\"><img src=\"static/throbber.gif\"></div>"
+		"<div align=\"center\"><img src=\"static/webcit_icons/throbber.gif\"></div>"
 		"</div>"
 		"<div align=\"center\">"
 		"<a href=\"javascript:RefreshSMTPqueueDisplay();\">%s</a>"
 		"</div>"
-		"</td></tr></table></div>\n", _("Refresh this page")
+		"</td></tr></table>\n", _("Refresh this page")
 	);
 
 	StrBufAppendPrintf(WC->trailing_javascript, "RefreshSMTPqueueDisplay();\n");
