@@ -1,11 +1,9 @@
 /*
- * $Id$
- *
  * WebCit "system dependent" code.
  *
- * Copyright (c) 1996-2010 by the citadel.org team
+ * Copyright (c) 1996-2011 by the citadel.org team
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is open source software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
@@ -81,10 +79,10 @@ pthread_key_t MyConKey;				/* TSD key for MyContext() */
 pthread_key_t MyReq;				/* TSD key for MyReq() */
 int msock;			/* master listening socket */
 int time_to_die = 0;            /* Nonzero if server is shutting down */
-int verbosity = 9;		/* Logging level */
 
 extern void *context_loop(ParsedHttpHdrs *Hdr);
 extern void *housekeeping_loop(void);
+extern void do_housekeeping(void);
 
 char ctdl_key_dir[PATH_MAX]=SSL_DIR;
 char file_crpt_file_key[PATH_MAX]="";
@@ -102,6 +100,8 @@ char  *static_dirs[]={				/* needs same sort order as the web mapping */
 	(char*)static_icon_dir                  /* our icons... */
 };
 
+int ExitPipe[2];
+
 void InitialiseSemaphores(void)
 {
 	int i;
@@ -110,6 +110,15 @@ void InitialiseSemaphores(void)
 	for (i=0; i<MAX_SEMAPHORES; ++i) {
 		pthread_mutex_init(&Critters[i], NULL);
 	}
+
+	if (pipe(ExitPipe))
+	{
+		syslog(2, "Failed to open exit pipe: %d [%s]\n", 
+		       errno, 
+		       strerror(errno));
+		
+		exit(-1);
+	}
 }
 
 /*
@@ -117,7 +126,6 @@ void InitialiseSemaphores(void)
  */
 void begin_critical_section(int which_one)
 {
-	/* lprintf(CTDL_DEBUG, "begin_critical_section(%d)\n", which_one); */
 	pthread_mutex_lock(&Critters[which_one]);
 }
 
@@ -164,7 +172,16 @@ void worker_entry(void)
 		ssock = -1; 
 		errno = EAGAIN;
 		do {
+			fd_set wset;
 			--num_threads_executing;
+                        FD_ZERO(&wset);
+                        FD_SET(msock, &wset);
+                        FD_SET(ExitPipe[1], &wset);
+
+                        select(msock + 1, NULL, &wset, NULL, NULL);
+			if (time_to_die)
+				break;
+
 			ssock = accept(msock, NULL, 0);
 			++num_threads_executing;
 			if (ssock < 0) fail_this_transaction = 1;
@@ -187,17 +204,17 @@ void worker_entry(void)
 			if (shutdown == 1)
 			{/* we're the one to cleanup the mess. */
 				http_destroy_modules(&Hdr);
-				lprintf(2, "I'm master shutdown: tagging sessions to be killed.\n");
+				syslog(2, "I'm master shutdown: tagging sessions to be killed.\n");
 				shutdown_sessions();
-				lprintf(2, "master shutdown: waiting for others\n");
+				syslog(2, "master shutdown: waiting for others\n");
 				sleeeeeeeeeep(1); /* wait so some others might finish... */
-				lprintf(2, "master shutdown: cleaning up sessions\n");
+				syslog(2, "master shutdown: cleaning up sessions\n");
 				do_housekeeping();
-				lprintf(2, "master shutdown: cleaning up libical\n");
+				syslog(2, "master shutdown: cleaning up libical\n");
 
 				ShutDownWebcit();
 
-				lprintf(2, "master shutdown exiting.\n");				
+				syslog(2, "master shutdown exiting.\n");				
 				exit(0);
 			}
 			break;
@@ -209,7 +226,7 @@ void worker_entry(void)
 		/* Now do something. */
 		if (msock < 0) {
 			if (ssock > 0) close (ssock);
-			lprintf(2, "in between.");
+			syslog(2, "in between.");
 			pthread_exit(NULL);
 		} else {
 			/* Got it? do some real work! */
@@ -231,11 +248,11 @@ void worker_entry(void)
 				int fdflags; 
 				fdflags = fcntl(ssock, F_GETFL);
 				if (fdflags < 0)
-					lprintf(1, "unable to get server socket flags! %s \n",
+					syslog(1, "unable to get server socket flags! %s \n",
 						strerror(errno));
 				fdflags = fdflags | O_NONBLOCK;
 				if (fcntl(ssock, F_SETFL, fdflags) < 0)
-					lprintf(1, "unable to set server socket nonblocking flags! %s \n",
+					syslog(1, "unable to set server socket nonblocking flags! %s \n",
 						strerror(errno));
 			}
 
@@ -265,30 +282,10 @@ void worker_entry(void)
 	} while (!time_to_die);
 
 	http_destroy_modules(&Hdr);
-	lprintf (1, "Thread exiting.\n");
+	syslog(1, "Thread exiting.\n");
 	pthread_exit(NULL);
 }
 
-/*
- * print log messages 
- * logs to stderr if loglevel is lower than the verbosity set at startup
- *
- * loglevel	level of the message
- * format	the printf like format string
- * ...		the strings to put into format
- */
-int lprintf(int loglevel, const char *format, ...)
-{
-	va_list ap;
-
-	if (loglevel <= verbosity) {
-		va_start(ap, format);
-		vfprintf(stderr, format, ap);
-		va_end(ap);
-		fflush(stderr);
-	}
-	return 1;
-}
 
 /*
  * Shut us down the regular way.
@@ -296,7 +293,8 @@ int lprintf(int loglevel, const char *format, ...)
  */
 pid_t current_child;
 void graceful_shutdown_watcher(int signum) {
-	lprintf (1, "Watcher thread exiting.\n");
+	syslog(1, "Watcher thread exiting.\n");
+	write(ExitPipe[0], HKEY("                              "));
 	kill(current_child, signum);
 	if (signum != SIGHUP)
 		exit(0);
@@ -312,7 +310,7 @@ void graceful_shutdown(int signum) {
 	FILE *FD;
 	int fd;
 
-	lprintf (1, "WebCit is being shut down on signal %d.\n", signum);
+	syslog(1, "WebCit is being shut down on signal %d.\n", signum);
 	fd = msock;
 	msock = -1;
 	time_to_die = 1;
@@ -320,6 +318,7 @@ void graceful_shutdown(int signum) {
 	fflush (FD);
 	fclose (FD);
 	close(fd);
+	write(ExitPipe[0], HKEY("                              "));
 }
 
 
@@ -443,13 +442,13 @@ void spawn_another_worker_thread()
 	 * otherwise the MIME parser crashes on FreeBSD.
 	 */
 	if ((ret = pthread_attr_setstacksize(&attr, 1024 * 1024))) {
-		lprintf(1, "pthread_attr_setstacksize: %s\n", strerror(ret));
+		syslog(1, "pthread_attr_setstacksize: %s\n", strerror(ret));
 		pthread_attr_destroy(&attr);
 	}
 
 	/* now create the thread */
 	if (pthread_create(&SessThread, &attr, (void *(*)(void *)) worker_entry, NULL) != 0) {
-		lprintf(1, "Can't create thread: %s\n", strerror(errno));
+		syslog(1, "Can't create thread: %s\n", strerror(errno));
 	}
 
 	/* free up the attributes */
@@ -499,7 +498,7 @@ webcit_calc_dirs_n_files(int relh, const char *basedir, int home, char *webcitdi
 		 ctdl_key_dir);
 
 	/* we should go somewhere we can leave our coredump, if enabled... */
-	lprintf(9, "Changing directory to %s\n", socket_dir);
+	syslog(9, "Changing directory to %s\n", socket_dir);
 	if (chdir(webcitdir) != 0) {
 		perror("chdir");
 	}
@@ -518,26 +517,26 @@ void drop_root(uid_t UID)
 #ifdef HAVE_GETPWUID_R
 #ifdef SOLARIS_GETPWUID
 		pwp = getpwuid_r(UID, &pw, pwbuf, sizeof(pwbuf));
-#else // SOLARIS_GETPWUID
+#else /* SOLARIS_GETPWUID */
 		getpwuid_r(UID, &pw, pwbuf, sizeof(pwbuf), &pwp);
-#endif // SOLARIS_GETPWUID
-#else // HAVE_GETPWUID_R
+#endif /* SOLARIS_GETPWUID */
+#else /* HAVE_GETPWUID_R */
 		pwp = NULL;
-#endif // HAVE_GETPWUID_R
+#endif /* HAVE_GETPWUID_R */
 
 		if (pwp == NULL)
-			lprintf(CTDL_CRIT, "WARNING: getpwuid(%ld): %s\n"
+			syslog(LOG_CRIT, "WARNING: getpwuid(%d): %s\n"
 				"Group IDs will be incorrect.\n", UID,
 				strerror(errno));
 		else {
 			initgroups(pw.pw_name, pw.pw_gid);
 			if (setgid(pw.pw_gid))
-				lprintf(CTDL_CRIT, "setgid(%ld): %s\n", (long)pw.pw_gid,
+				syslog(LOG_CRIT, "setgid(%ld): %s\n", (long)pw.pw_gid,
 					strerror(errno));
 		}
-		lprintf(CTDL_INFO, "Changing uid to %ld\n", (long)UID);
+		syslog(LOG_INFO, "Changing uid to %ld\n", (long)UID);
 		if (setuid(UID) != 0) {
-			lprintf(CTDL_CRIT, "setuid() failed: %s\n", strerror(errno));
+			syslog(LOG_CRIT, "setuid() failed: %s\n", strerror(errno));
 		}
 #if defined (HAVE_SYS_PRCTL_H) && defined (PR_SET_DUMPABLE)
 		prctl(PR_SET_DUMPABLE, 1);
@@ -561,9 +560,9 @@ void wc_backtrace(void)
 	strings = backtrace_symbols(stack_frames, size);
 	for (i = 0; i < size; i++) {
 		if (strings != NULL)
-			lprintf(1, "%s\n", strings[i]);
+			syslog(1, "%s\n", strings[i]);
 		else
-			lprintf(1, "%p\n", stack_frames[i]);
+			syslog(1, "%p\n", stack_frames[i]);
 	}
 	free(strings);
 #endif
