@@ -1,6 +1,21 @@
 /*
- * $Id$
+ * Copyright (c) 1996-2011 by the citadel.org team
+ *
+ * This program is open source software.  You can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
+
 #include "webcit.h"
 
 
@@ -15,14 +30,15 @@
 void display_edit(char *description, char *check_cmd,
 		  char *read_cmd, char *save_cmd, int with_room_banner)
 {
-	char buf[SIZ];
+	StrBuf *Line;
 
 	serv_puts(check_cmd);
-	serv_getln(buf, sizeof buf);
-
-	if (buf[0] != '2') {
-		safestrncpy(WC->ImportantMessage, &buf[4], sizeof WC->ImportantMessage);
+	Line = NewStrBuf();
+	StrBuf_ServGetln(Line);
+	if (GetServerStatusMsg(Line, NULL, 1, 2) != 2) {
+		FreeStrBuf(&Line);
 		display_main_menu();
+		FreeStrBuf(&Line);
 		return;
 	}
 	if (with_room_banner) {
@@ -32,8 +48,9 @@ void display_edit(char *description, char *check_cmd,
 		output_headers(1, 1, 0, 0, 0, 0);
 	}
 
-	svprintf(HKEY("BOXTITLE"), WCS_STRING, _("Edit %s"), description);
-	do_template("beginboxx", NULL);
+	do_template("box_begin_1");
+	StrBufAppendPrintf (WC->WBuf, _("Edit %s"), description);
+	do_template("box_begin_2");
 
 	wc_printf(_("Enter %s below. Text is formatted to the reader's browser."
 		" A newline is forced by preceding the next line by a blank."), description);
@@ -43,17 +60,18 @@ void display_edit(char *description, char *check_cmd,
 	wc_printf("<textarea name=\"msgtext\" wrap=soft "
 		"rows=10 cols=80 width=80>\n");
 	serv_puts(read_cmd);
-	serv_getln(buf, sizeof buf);
-	if (buf[0] == '1')
+	StrBuf_ServGetln(Line);
+	if (GetServerStatusMsg(Line, NULL, 0, 0) == 1)
 		server_to_text();
 	wc_printf("</textarea><div class=\"buttons\" >\n");
 	wc_printf("<input type=\"submit\" name=\"save_button\" value=\"%s\">", _("Save changes"));
 	wc_printf("&nbsp;");
-	wc_printf("<input type=\"submit\" name=\"cancel_button\" value=\"%s\"><br />\n", _("Cancel"));
+	wc_printf("<input type=\"submit\" name=\"cancel_button\" value=\"%s\"><br>\n", _("Cancel"));
 	wc_printf("</div></form>\n");
 
-	do_template("endbox", NULL);
+	do_template("box_end");
 	wDumpContent(1);
+	FreeStrBuf(&Line);
 }
 
 
@@ -65,31 +83,30 @@ void display_edit(char *description, char *check_cmd,
  */
 void save_edit(char *description, char *enter_cmd, int regoto)
 {
-	char buf[SIZ];
+	StrBuf *Line;
 
 	if (!havebstr("save_button")) {
-		sprintf(WC->ImportantMessage,
-			_("Cancelled.  %s was not saved."),
-			description);
+		AppendImportantMessage(_("Cancelled.  %s was not saved."), -1);
 		display_main_menu();
 		return;
 	}
+	Line = NewStrBuf();
 	serv_puts(enter_cmd);
-	serv_getln(buf, sizeof buf);
-	if (buf[0] != '4') {
-		safestrncpy(WC->ImportantMessage, &buf[4], sizeof WC->ImportantMessage);
+	StrBuf_ServGetln(Line);
+	if (GetServerStatusMsg(Line, NULL, 1, 0) != 4) {
+		FreeStrBuf(&Line);
 		display_main_menu();
 		return;
 	}
+	FreeStrBuf(&Line);
 	text_to_server(bstr("msgtext"));
 	serv_puts("000");
 
 	if (regoto) {
 		smart_goto(WC->CurRoom.name);
 	} else {
-		sprintf(WC->ImportantMessage,
-			_("%s has been saved."),
-			description);
+		AppendImportantMessage(description, -1);
+		AppendImportantMessage(_(" has been saved."), -1);
 		display_main_menu();
 		return;
 	}
