@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1987-2011 by the citadel.org team
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is open source software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
@@ -26,13 +26,14 @@
 #include "webserver.h"
 
 extern int DisableGzip;
+long MaxRead = -1; /* should we do READ scattered or all at once? */
 
 /*
  * register the timeout
  */
 RETSIGTYPE timeout(int signum)
 {
-	lprintf(1, "Connection timed out; unable to reach citserver\n");
+	syslog(1, "Connection timed out; unable to reach citserver\n");
 	/* no exit here, since we need to server the connection unreachable thing. exit(3); */
 }
 
@@ -51,12 +52,12 @@ int uds_connectsock(char *sockpath)
 
 	s = socket(AF_UNIX, SOCK_STREAM, 0);
 	if (s < 0) {
-		lprintf(1, "Can't create socket[%s]: %s\n", sockpath, strerror(errno));
+		syslog(1, "Can't create socket [%s]: %s\n", sockpath, strerror(errno));
 		return(-1);
 	}
 
 	if (connect(s, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
-		lprintf(1, "Can't connect [%s]: %s\n", sockpath, strerror(errno));
+		syslog(1, "Can't connect [%s]: %s\n", sockpath, strerror(errno));
 		close(s);
 		return(-1);
 	}
@@ -66,41 +67,83 @@ int uds_connectsock(char *sockpath)
 
 
 /*
- * TCP client - connect to a host/port
+ * TCP client - connect to a host/port 
  */
-int tcp_connectsock(char *host, int port)
+int tcp_connectsock(char *host, char *service)
 {
-	struct sockaddr_in stSockAddr;
-	int rv;
-	int sock;
+	struct in6_addr serveraddr;
+	struct addrinfo hints;
+	struct addrinfo *res = NULL;
+	struct addrinfo *ai = NULL;
+	int rc = (-1);
+	int s = (-1);
 
-	sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
-	if (sock < 0) {
-		lprintf(1, "Can't create socket: %s\n", strerror(errno));
+	if ((host == NULL) || IsEmptyStr(host))
 		return (-1);
+	if ((service == NULL) || IsEmptyStr(service))
+		return (-1);
+
+	syslog(9, "tcp_connectsock(%s,%s)\n", host, service);
+
+	memset(&hints, 0x00, sizeof(hints));
+	hints.ai_flags = AI_NUMERICSERV;
+	hints.ai_family = AF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM;
+
+	/*
+	 * Handle numeric IPv4 and IPv6 addresses
+	 */
+	rc = inet_pton(AF_INET, host, &serveraddr);
+	if (rc == 1) {						/* dotted quad */
+		hints.ai_family = AF_INET;
+		hints.ai_flags |= AI_NUMERICHOST;
+	} else {
+		rc = inet_pton(AF_INET6, host, &serveraddr);
+		if (rc == 1) {					/* IPv6 address */
+			hints.ai_family = AF_INET6;
+			hints.ai_flags |= AI_NUMERICHOST;
+		}
 	}
 
-	memset(&stSockAddr, 0, sizeof(struct sockaddr_in));
-	stSockAddr.sin_family = AF_INET;
-	stSockAddr.sin_port = htons(port);
-	rv = inet_pton(AF_INET, host, &stSockAddr.sin_addr);
+	/* Begin the connection process */
 
-	if (rv <= 0) {
-		lprintf(1, "Can't grok %s: %s\n", host, strerror(errno));
-		return (-1);
+	rc = getaddrinfo(host, service, &hints, &res);
+	if (rc != 0) {
+		syslog(1, "%s: %s\n", host, gai_strerror(rc));
+		freeaddrinfo(res);
+		return(-1);
 	}
 
-	if (connect(sock, (const struct sockaddr *)&stSockAddr, sizeof(struct sockaddr_in)) != 0) {
-		lprintf(1, "Can't connect to %s.%d: %s\n", host, port, strerror(errno));
-		close(sock);
-		return (-1);
-	}
+	/*
+	 * Try all available addresses until we connect to one or until we run out.
+	 */
+	for (ai = res; ai != NULL; ai = ai->ai_next) {
 
-	return (sock);
+		if (ai->ai_family == AF_INET) syslog(9, "Trying IPv4\n");
+		else if (ai->ai_family == AF_INET6) syslog(9, "Trying IPv6\n");
+		else syslog(9, "This is going to fail.\n");
+
+		s = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
+		if (s < 0) {
+			syslog(1, "socket() failed: %s\n", strerror(errno));
+			freeaddrinfo(res);
+			return(-1);
+		}
+		rc = connect(s, ai->ai_addr, ai->ai_addrlen);
+		if (rc >= 0) {
+			freeaddrinfo(res);
+			return(s);
+		}
+		else {
+			syslog(1, "connect() failed: %s\n", strerror(errno));
+			close(s);
+		}
+	}
+        freeaddrinfo(res);
+	return(-1);
 }
 
 
-
 /*
  *  input string from pipe
  */
@@ -118,7 +161,7 @@ int serv_getln(char *strbuf, int bufsize)
 	FlushStrBuf(WCC->MigrateReadLineBuf);
 	strbuf[len] = '\0';
 #ifdef SERV_TRACE
-	lprintf(9, "%3d<<<%s\n", WC->serv_sock, strbuf);
+	syslog(9, "%3d<<<%s\n", WC->serv_sock, strbuf);
 #endif
 	return len;
 }
@@ -139,9 +182,10 @@ int StrBuf_ServGetln(StrBuf *buf)
 					       &ErrStr);
 	if (rc < 0)
 	{
-		lprintf(1, "Server connection broken: %s\n",
+		syslog(1, "StrBuf_ServGetln(): Server connection broken: %s\n",
 			(ErrStr)?ErrStr:"");
 		wc_backtrace();
+		if (WCC->serv_sock > 0) close(WCC->serv_sock);
 		WCC->serv_sock = (-1);
 		WCC->connected = 0;
 		WCC->logged_in = 0;
@@ -152,7 +196,7 @@ int StrBuf_ServGetln(StrBuf *buf)
 		long pos = 0;
 		if (WCC->ReadPos != NULL)
 			pos = WCC->ReadPos - ChrPtr(WCC->ReadBuf);
-		lprintf(9, "%3d<<<[%ld]%s\n", WC->serv_sock, pos, ChrPtr(buf));
+		syslog(9, "%3d<<<[%ld]%s\n", WC->serv_sock, pos, ChrPtr(buf));
 	}
 #endif
 	return rc;
@@ -174,16 +218,17 @@ int StrBuf_ServGetBLOBBuffered(StrBuf *buf, long BlobSize)
 				    &ErrStr);
 	if (rc < 0)
 	{
-		lprintf(1, "Server connection broken: %s\n",
+		syslog(1, "StrBuf_ServGetBLOBBuffered(): Server connection broken: %s\n",
 			(ErrStr)?ErrStr:"");
 		wc_backtrace();
+		if (WCC->serv_sock > 0) close(WCC->serv_sock);
 		WCC->serv_sock = (-1);
 		WCC->connected = 0;
 		WCC->logged_in = 0;
 	}
 #ifdef SERV_TRACE
         else
-                lprintf(9, "%3d<<<BLOB: %ld bytes\n", WC->serv_sock, StrLength(buf));
+                syslog(9, "%3d<<<BLOB: %d bytes\n", WC->serv_sock, StrLength(buf));
 #endif
 
 	return rc;
@@ -199,49 +244,89 @@ int StrBuf_ServGetBLOB(StrBuf *buf, long BlobSize)
 	rc = StrBufReadBLOB(buf, &WCC->serv_sock, 1, BlobSize, &ErrStr);
 	if (rc < 0)
 	{
-		lprintf(1, "Server connection broken: %s\n",
+		syslog(1, "StrBuf_ServGetBLOB(): Server connection broken: %s\n",
 			(ErrStr)?ErrStr:"");
 		wc_backtrace();
+		if (WCC->serv_sock > 0) close(WCC->serv_sock);
 		WCC->serv_sock = (-1);
 		WCC->connected = 0;
 		WCC->logged_in = 0;
 	}
 #ifdef SERV_TRACE
         else
-                lprintf(9, "%3d<<<BLOB: %ld bytes\n", WC->serv_sock, StrLength(buf));
+                syslog(9, "%3d<<<BLOB: %d bytes\n", WC->serv_sock, StrLength(buf));
 #endif
 
 	return rc;
 }
 
+
+void FlushReadBuf (void)
+{
+	long len;
+	const char *pch;
+	const char *pche;
+	wcsession *WCC = WC;
+
+	len = StrLength(WCC->ReadBuf);
+	if ((len > 0) &&
+	    (WCC->ReadPos != NULL) && 
+	    (WCC->ReadPos != StrBufNOTNULL))
+		
+	{
+		pch = ChrPtr(WCC->ReadBuf);
+		pche = pch + len;
+		if (WCC->ReadPos != pche)
+		{
+			syslog(1,
+				"ERROR: somebody didn't eat his soup! Remaing Chars: %ld [%s]\n", 
+				(long)(pche - WCC->ReadPos),
+				pche
+			);
+			syslog(1, 
+				"--------------------------------------------------------------------------------\n"
+				"Whole buf: [%s]\n"
+				"--------------------------------------------------------------------------------\n", 
+				pch);
+			AppendImportantMessage(HKEY("Suppenkasper alert! watch your webcit logfile and get connected to your favourite opensource Crew."));
+		}
+	}
+
+	FlushStrBuf(WCC->ReadBuf);
+	WCC->ReadPos = NULL;
+
+
+}
+
+
 /*
  *  send binary to server
  *  buf the buffer to write to citadel server
  *  nbytes how many bytes to send to citadel server
  */
-void serv_write(const char *buf, int nbytes)
+int serv_write(const char *buf, int nbytes)
 {
 	wcsession *WCC = WC;
 	int bytes_written = 0;
 	int retval;
 
-	FlushStrBuf(WCC->ReadBuf);
-	WCC->ReadPos = NULL;
+	FlushReadBuf();
 	while (bytes_written < nbytes) {
 		retval = write(WCC->serv_sock, &buf[bytes_written],
 			       nbytes - bytes_written);
 		if (retval < 1) {
 			const char *ErrStr = strerror(errno);
-			lprintf(1, "Server connection broken: %s\n",
+			syslog(1, "serv_write(): Server connection broken: %s\n",
 				(ErrStr)?ErrStr:"");
-			close(WCC->serv_sock);
+			if (WCC->serv_sock > 0) close(WCC->serv_sock);
 			WCC->serv_sock = (-1);
 			WCC->connected = 0;
 			WCC->logged_in = 0;
-			return;
+			return 0;
 		}
 		bytes_written = bytes_written + retval;
 	}
+	return 1;
 }
 
 
@@ -249,34 +334,32 @@ void serv_write(const char *buf, int nbytes)
  *  send line to server
  *  string the line to send to the citadel server
  */
-void serv_puts(const char *string)
+int serv_puts(const char *string)
 {
-	wcsession *WCC = WC;
 #ifdef SERV_TRACE
-	lprintf(9, "%3d>>>%s\n", WC->serv_sock, string);
+	syslog(9, "%3d>>>%s\n", WC->serv_sock, string);
 #endif
-	FlushStrBuf(WCC->ReadBuf);
-	WCC->ReadPos = NULL;
+	FlushReadBuf();
 
-	serv_write(string, strlen(string));
-	serv_write("\n", 1);
+	if (!serv_write(string, strlen(string)))
+		return 0;
+	return serv_write("\n", 1);
 }
 
 /*
  *  send line to server
  *  string the line to send to the citadel server
  */
-void serv_putbuf(const StrBuf *string)
+int serv_putbuf(const StrBuf *string)
 {
-	wcsession *WCC = WC;
 #ifdef SERV_TRACE
-	lprintf(9, "%3d>>>%s\n", WC->serv_sock, ChrPtr(string));
+	syslog(9, "%3d>>>%s\n", WC->serv_sock, ChrPtr(string));
 #endif
-	FlushStrBuf(WCC->ReadBuf);
-	WCC->ReadPos = NULL;
+	FlushReadBuf();
 
-	serv_write(ChrPtr(string), StrLength(string));
-	serv_write("\n", 1);
+	if (!serv_write(ChrPtr(string), StrLength(string)))
+		return 0;
+	return serv_write("\n", 1);
 }
 
 
@@ -285,15 +368,14 @@ void serv_putbuf(const StrBuf *string)
  *  format the formatstring
  *  ... the entities to insert into format 
  */
-void serv_printf(const char *format,...)
+int serv_printf(const char *format,...)
 {
-	wcsession *WCC = WC;
 	va_list arg_ptr;
 	char buf[SIZ];
 	size_t len;
+	int rc;
 
-	FlushStrBuf(WCC->ReadBuf);
-	WCC->ReadPos = NULL;
+	FlushReadBuf();
 
 	va_start(arg_ptr, format);
 	vsnprintf(buf, sizeof buf, format, arg_ptr);
@@ -302,14 +384,14 @@ void serv_printf(const char *format,...)
 	len = strlen(buf);
 	buf[len++] = '\n';
 	buf[len] = '\0';
-	serv_write(buf, len);
+	rc = serv_write(buf, len);
 #ifdef SERV_TRACE
-	lprintf(9, ">>>%s", buf);
+	syslog(9, ">>>%s", buf);
 #endif
+	return rc;
 }
 
 
-
 /*
  * Read binary data from server into memory using a series of server READ commands.
  * returns the read content as StrBuf
@@ -332,7 +414,7 @@ int serv_read_binary(StrBuf *Ret, size_t total_len, StrBuf *Buf)
 			return -1; 
 		}
 
-		serv_printf("READ %d|%d", bytes_read, total_len-bytes_read);
+		serv_printf("READ "SIZE_T_FMT"|"SIZE_T_FMT, bytes_read, total_len-bytes_read);
 		if ( (rc = StrBuf_ServGetln(Buf) > 0) && (GetServerStatus(Buf, NULL) == 6) ) 
 		{
 			if (rc < 0)
@@ -341,8 +423,9 @@ int serv_read_binary(StrBuf *Ret, size_t total_len, StrBuf *Buf)
 			this_block = StrTol(Buf);
 			rc = StrBuf_ServGetBLOBBuffered(Ret, this_block);
 			if (rc < 0) {
-				lprintf(1, "Server connection broken during download\n");
+				syslog(1, "Server connection broken during download\n");
 				wc_backtrace();
+				if (WCC->serv_sock > 0) close(WCC->serv_sock);
 				WCC->serv_sock = (-1);
 				WCC->connected = 0;
 				WCC->logged_in = 0;
@@ -365,7 +448,8 @@ int ClientGetLine(ParsedHttpHdrs *Hdr, StrBuf *Target)
 
 	if (is_https) {
 		int ntries = 0;
-		if (StrLength(Hdr->ReadBuf) > 0) {
+		if (StrLength(Hdr->ReadBuf) > 0)
+		{
 			pchs = ChrPtr(Hdr->ReadBuf);
 			pch = strchr(pchs, '\n');
 			if (pch != NULL) {
@@ -388,6 +472,8 @@ int ClientGetLine(ParsedHttpHdrs *Hdr, StrBuf *Target)
 					retval = client_read_sslbuffer(Hdr->ReadBuf, SLEEPING);
 					pchs = ChrPtr(Hdr->ReadBuf);
 					pch = strchr(pchs, '\n');
+					if (pch == NULL)
+						retval = 0;
 				}
 				if (retval == 0) {
 					sleeeeeeeeeep(1);
@@ -420,7 +506,6 @@ int ClientGetLine(ParsedHttpHdrs *Hdr, StrBuf *Target)
 							 &Error);
 }
 
-#ifdef CTDL_IPV6
 
 /* 
  * This is a generic function to set up a master socket for listening on
@@ -430,131 +515,94 @@ int ClientGetLine(ParsedHttpHdrs *Hdr, StrBuf *Target)
  * port_number	port number to bind
  * queue_len	number of incoming connections to allow in the queue
  */
-int ig_tcp_server(char *ip_addr, int port_number, int queue_len)
+int webcit_tcp_server(char *ip_addr, int port_number, int queue_len)
 {
 	struct protoent *p;
-	struct sockaddr_in6 sin;
-	int s, i;
+	struct sockaddr_in6 sin6;
+	struct sockaddr_in sin4;
+	int s, i, b;
+	int ip_version = 6;
 
-	memset(&sin, 0, sizeof(sin));
-	sin.sin6_family = AF_INET6;
+	memset(&sin6, 0, sizeof(sin6));
+	memset(&sin4, 0, sizeof(sin4));
+	sin6.sin6_family = AF_INET6;
+	sin4.sin_family = AF_INET;
 
-	if ((ip_addr == NULL) || (IsEmptyStr(ip_addr)) || (!strcmp(ip_addr, "0.0.0.0"))) {
-		sin.sin6_addr = in6addr_any;
-	} else {
-		char bind_to[256];
-		if ((strchr(ip_addr, '.')) && (!strchr(ip_addr, ':'))) {
-			snprintf(bind_to, sizeof bind_to, "::ffff:%s", ip_addr);
+	if (	(ip_addr == NULL)							/* any IPv6 */
+		|| (IsEmptyStr(ip_addr))
+		|| (!strcmp(ip_addr, "*"))
+	) {
+		ip_version = 6;
+		sin6.sin6_addr = in6addr_any;
+	}
+	else if (!strcmp(ip_addr, "0.0.0.0"))						/* any IPv4 */
+	{
+		ip_version = 4;
+		sin4.sin_addr.s_addr = INADDR_ANY;
+	}
+	else if ((strchr(ip_addr, '.')) && (!strchr(ip_addr, ':')))			/* specific IPv4 */
+	{
+		ip_version = 4;
+		if (inet_pton(AF_INET, ip_addr, &sin4.sin_addr) <= 0) {
+			syslog(1, "Error binding to [%s] : %s\n", ip_addr, strerror(errno));
+			return (-WC_EXIT_BIND);
 		}
-		else {
-			safestrncpy(bind_to, ip_addr, sizeof bind_to);
-		}
-		if (inet_pton(AF_INET6, bind_to, &sin.sin6_addr) <= 0) {
-			lprintf(1, "Error binding to [%s] : %s\n", ip_addr, strerror(errno));
+	}
+	else										/* specific IPv6 */
+	{
+		ip_version = 6;
+		if (inet_pton(AF_INET6, ip_addr, &sin6.sin6_addr) <= 0) {
+			syslog(1, "Error binding to [%s] : %s\n", ip_addr, strerror(errno));
 			return (-WC_EXIT_BIND);
 		}
 	}
 
 	if (port_number == 0) {
-		lprintf(1, "Cannot start: no port number specified.\n");
+		syslog(1, "Cannot start: no port number specified.\n");
 		return (-WC_EXIT_BIND);
 	}
-	sin.sin6_port = htons((u_short) port_number);
+	sin6.sin6_port = htons((u_short) port_number);
+	sin4.sin_port = htons((u_short) port_number);
 
 	p = getprotobyname("tcp");
 
-	s = socket(PF_INET6, SOCK_STREAM, (p->p_proto));
+	s = socket( ((ip_version == 6) ? PF_INET6 : PF_INET), SOCK_STREAM, (p->p_proto));
 	if (s < 0) {
-		lprintf(1, "Can't create an IPv6 socket: %s\n", strerror(errno));
+		syslog(1, "Can't create a listening socket: %s\n", strerror(errno));
 		return (-WC_EXIT_BIND);
 	}
 	/* Set some socket options that make sense. */
 	i = 1;
 	setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));
 
-	#ifndef __APPLE__
-	fcntl(s, F_SETFL, O_NONBLOCK); /* maide: this statement is incorrect
-					  there should be a preceding F_GETFL
-					  and a bitwise OR with the previous
-					  fd flags */
-	#endif
-	
-	if (bind(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
-		lprintf(1, "Can't bind: %s\n", strerror(errno));
+	if (ip_version == 6) {
+		b = bind(s, (struct sockaddr *) &sin6, sizeof(sin6));
+	}
+	else {
+		b = bind(s, (struct sockaddr *) &sin4, sizeof(sin4));
+	}
+
+	if (b < 0) {
+		syslog(1, "Can't bind: %s\n", strerror(errno));
+		close(s);
 		return (-WC_EXIT_BIND);
 	}
+
 	if (listen(s, queue_len) < 0) {
-		lprintf(1, "Can't listen: %s\n", strerror(errno));
+		syslog(1, "Can't listen: %s\n", strerror(errno));
+		close(s);
 		return (-WC_EXIT_BIND);
 	}
 	return (s);
 }
 
-#else /* CTDL_IPV6 */
-
-/* 
- * This is a generic function to set up a master socket for listening on
- * a TCP port.  The server shuts down if the bind fails.
- *
- * ip_addr 	IP address to bind
- * port_number	port number to bind
- * queue_len	number of incoming connections to allow in the queue
- */
-int ig_tcp_server(char *ip_addr, int port_number, int queue_len)
-{
-	struct protoent *p;
-	struct sockaddr_in sin;
-	int s, i;
-
-	memset(&sin, 0, sizeof(sin));
-	sin.sin_family = AF_INET;
-	if (ip_addr == NULL) {
-		sin.sin_addr.s_addr = INADDR_ANY;
-	} else {
-		sin.sin_addr.s_addr = inet_addr(ip_addr);
-	}
-
-	if (sin.sin_addr.s_addr == INADDR_NONE) {
-		sin.sin_addr.s_addr = INADDR_ANY;
-	}
-
-	if (port_number == 0) {
-		lprintf(1, "Cannot start: no port number specified.\n");
-		return (-WC_EXIT_BIND);
-	}
-	sin.sin_port = htons((u_short) port_number);
-
-	p = getprotobyname("tcp");
-
-	s = socket(PF_INET, SOCK_STREAM, (p->p_proto));
-	if (s < 0) {
-		lprintf(1, "Can't create an IPv4 socket: %s\n", strerror(errno));
-		return (-WC_EXIT_BIND);
-	}
-	/* Set some socket options that make sense. */
-	i = 1;
-	setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));
-
-	if (bind(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
-		lprintf(1, "Can't bind: %s\n", strerror(errno));
-		return (-WC_EXIT_BIND);
-	}
-	if (listen(s, queue_len) < 0) {
-		lprintf(1, "Can't listen: %s\n", strerror(errno));
-		return (-WC_EXIT_BIND);
-	}
-	return (s);
-}
-
-#endif /* CTDL_IPV6 */
-
 
 /*
  * Create a Unix domain socket and listen on it
  * sockpath - file name of the unix domain socket
  * queue_len - Number of incoming connections to allow in the queue
  */
-int ig_uds_server(char *sockpath, int queue_len)
+int webcit_uds_server(char *sockpath, int queue_len)
 {
 	struct sockaddr_un addr;
 	int s;
@@ -566,7 +614,7 @@ int ig_uds_server(char *sockpath, int queue_len)
 
 	i = unlink(sockpath);
 	if ((i != 0) && (errno != ENOENT)) {
-		lprintf(1, "webcit: can't unlink %s: %s\n",
+		syslog(1, "webcit: can't unlink %s: %s\n",
 			sockpath, strerror(errno));
 		return (-WC_EXIT_BIND);
 	}
@@ -577,19 +625,19 @@ int ig_uds_server(char *sockpath, int queue_len)
 
 	s = socket(AF_UNIX, SOCK_STREAM, 0);
 	if (s < 0) {
-		lprintf(1, "webcit: Can't create a unix domain socket: %s\n", strerror(errno));
+		syslog(1, "webcit: Can't create a unix domain socket: %s\n", strerror(errno));
 		return (-WC_EXIT_BIND);
 	}
 
 	if (bind(s, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
-		lprintf(1, "webcit: Can't bind: %s\n",
-			strerror(errno));
+		syslog(1, "webcit: Can't bind: %s\n", strerror(errno));
+		close(s);
 		return (-WC_EXIT_BIND);
 	}
 
 	if (listen(s, actual_queue_len) < 0) {
-		lprintf(1, "webcit: Can't listen: %s\n",
-			strerror(errno));
+		syslog(1, "webcit: Can't listen: %s\n", strerror(errno));
+		close(s);
 		return (-WC_EXIT_BIND);
 	}
 
@@ -620,19 +668,23 @@ int client_read_to(ParsedHttpHdrs *Hdr, StrBuf *Target, int bytes, int timeout)
 
 #ifdef HAVE_OPENSSL
 	if (is_https) {
-		long bufremain;
+		long bufremain = 0;
 		long baselen;
 
 		baselen = StrLength(Target);
 
 		if (Hdr->Pos == NULL)
 			Hdr->Pos = ChrPtr(Hdr->ReadBuf);
-		bufremain = StrLength(Hdr->ReadBuf) - (Hdr->Pos - ChrPtr(Hdr->ReadBuf));
 
-		if (bytes < bufremain)
-			bufremain = bytes;
-		StrBufAppendBufPlain(Target, Hdr->Pos, bufremain, 0);
-		StrBufCutLeft(Hdr->ReadBuf, bufremain);
+		if (StrLength(Hdr->ReadBuf) > 0)
+		{
+			bufremain = StrLength(Hdr->ReadBuf) - (Hdr->Pos - ChrPtr(Hdr->ReadBuf));
+			
+			if (bytes < bufremain)
+				bufremain = bytes;
+			StrBufAppendBufPlain(Target, Hdr->Pos, bufremain, 0);
+			StrBufCutLeft(Hdr->ReadBuf, bufremain);
+		}
 
 		if (bytes > bufremain) 
 		{
@@ -641,15 +693,10 @@ int client_read_to(ParsedHttpHdrs *Hdr, StrBuf *Target, int bytes, int timeout)
 				retval = client_read_sslbuffer(Hdr->ReadBuf, timeout);
 			if (retval >= 0) {
 				StrBufAppendBuf(Target, Hdr->ReadBuf, 0); /* todo: Buf > bytes? */
-#ifdef HTTP_TRACING
-				write(2, "\033[32m", 5);
-				write(2, buf, bytes);
-				write(2, "\033[30m", 5);
-#endif
 				return 1;
 			}
 			else {
-				lprintf(2, "client_read_ssl() failed\n");
+				syslog(2, "client_read_ssl() failed\n");
 				return -1;
 			}
 		}
@@ -667,17 +714,12 @@ int client_read_to(ParsedHttpHdrs *Hdr, StrBuf *Target, int bytes, int timeout)
 					O_TERM,
 					&Error);
 	if (retval < 0) {
-		lprintf(2, "client_read() failed: %s\n",
+		syslog(2, "client_read() failed: %s\n",
 			Error);
 		wc_backtrace();
 		return retval;
 	}
 
-#ifdef HTTP_TRACING
-	write(2, "\033[32m", 5);
-	write(2, buf, bytes);
-	write(2, "\033[30m", 5);
-#endif
 	return 1;
 }
 
@@ -710,7 +752,7 @@ long end_burst(void)
 		if (CompressBuffer(WCC->WBuf) > 0)
 			hprintf("Content-encoding: gzip\r\n");
 		else {
-			lprintf(CTDL_ALERT, "Compression failed: %d [%s] sending uncompressed\n", errno, strerror(errno));
+			syslog(LOG_ALERT, "Compression failed: %d [%s] sending uncompressed\n", errno, strerror(errno));
 			wc_backtrace();
 		}
 	}
@@ -736,13 +778,6 @@ long end_burst(void)
 	}
 #endif
 
-	
-#ifdef HTTP_TRACING
-	
-	write(2, "\033[34m", 5);
-	write(2, ptr, StrLength(WCC->WBuf));
-	write(2, "\033[30m", 5);
-#endif
 	if (WCC->Hdr->http_sock == -1)
 		return -1;
 	fdflags = fcntl(WC->Hdr->http_sock, F_GETFL);
@@ -752,7 +787,7 @@ long end_burst(void)
                         FD_ZERO(&wset);
                         FD_SET(WCC->Hdr->http_sock, &wset);
                         if (select(WCC->Hdr->http_sock + 1, NULL, &wset, NULL, NULL) == -1) {
-                                lprintf(2, "client_write: Socket select failed (%s)\n", strerror(errno));
+                                syslog(2, "client_write: Socket select failed (%s)\n", strerror(errno));
                                 return -1;
                         }
                 }
@@ -761,7 +796,7 @@ long end_burst(void)
 		    (res = write(WCC->Hdr->http_sock, 
 				 ptr,
 				 count)) == -1) {
-                        lprintf(2, "client_write: Socket write failed (%s)\n", strerror(errno));
+                        syslog(2, "client_write: Socket write failed (%s)\n", strerror(errno));
 		        wc_backtrace();
                         return res;
                 }
@@ -773,19 +808,12 @@ long end_burst(void)
 	count = StrLength(WCC->WBuf);
 	eptr = ptr + count;
 
-#ifdef HTTP_TRACING
-	
-	write(2, "\033[34m", 5);
-	write(2, ptr, StrLength(WCC->WBuf));
-	write(2, "\033[30m", 5);
-#endif
-
         while ((ptr < eptr) && (WCC->Hdr->http_sock != -1)) {
                 if ((fdflags & O_NONBLOCK) == O_NONBLOCK) {
                         FD_ZERO(&wset);
                         FD_SET(WCC->Hdr->http_sock, &wset);
                         if (select(WCC->Hdr->http_sock + 1, NULL, &wset, NULL, NULL) == -1) {
-                                lprintf(2, "client_write: Socket select failed (%s)\n", strerror(errno));
+                                syslog(2, "client_write: Socket select failed (%s)\n", strerror(errno));
                                 return -1;
                         }
                 }
@@ -794,7 +822,7 @@ long end_burst(void)
 		    (res = write(WCC->Hdr->http_sock, 
 				 ptr,
 				 count)) == -1) {
-                        lprintf(2, "client_write: Socket write failed (%s)\n", strerror(errno));
+                        syslog(2, "client_write: Socket write failed (%s)\n", strerror(errno));
 			wc_backtrace();
                         return res;
                 }
@@ -886,6 +914,8 @@ SessionDestroyModule_TCPSOCKETS
 	FreeStrBuf(&sess->ReadBuf);
 	sess->ReadPos = NULL;
 	FreeStrBuf(&sess->MigrateReadLineBuf);
-	if (sess->serv_sock > 0)
+	if (sess->serv_sock > 0) {
+		syslog(LOG_DEBUG, "Closing socket %d", sess->serv_sock);
 		close(sess->serv_sock);
+	}
 }
