@@ -1,5 +1,19 @@
 /*
- * $Id$
+ * Copyright (c) 1996-2011 by the citadel.org team
+ *
+ * This program is open source software.  You can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "webcit.h"
@@ -14,7 +28,7 @@
 void select_user_to_edit(const char *preselect)
 {
 	output_headers(1, 0, 0, 0, 1, 0);
-	do_template("aide_edituser_select", NULL);
+	do_template("aide_edituser_select");
         end_burst();
 }
 
@@ -31,6 +45,7 @@ typedef struct _UserListEntry {
 	/* Just available for Single users to view: */
 	unsigned int Flags;
 	int DaysTillPurge;
+	int HasBio;
 } UserListEntry;
 
 
@@ -86,6 +101,7 @@ UserListEntry* NewUserListEntry(StrBuf *SerializedUserList)
 	ul->nPosts      = StrBufExtractNext_int( SerializedUserList, &Pos, '|');
 	StrBufExtract_NextToken(ul->Passvoid,    SerializedUserList, &Pos, '|');
 	ul->Flags = 0;
+	ul->HasBio = 0;
 	ul->DaysTillPurge = -1;
 	return ul;
 }
@@ -258,9 +274,9 @@ HashList *iterate_load_userlist(StrBuf *Target, WCTemplputParams *TP)
 	HashList *Hash = NULL;
 	StrBuf *Buf;
 	UserListEntry* ul;
-	char nnn[64];
-	int nUsed;
 	int len;
+	int UID;
+	void *vData;
 	WCTemplputParams SubTP;
 
 	memset(&SubTP, 0, sizeof(WCTemplputParams));	
@@ -268,7 +284,7 @@ HashList *iterate_load_userlist(StrBuf *Target, WCTemplputParams *TP)
 	Buf = NewStrBuf();
 	StrBuf_ServGetln(Buf);
 	if (GetServerStatus(Buf, NULL) == 1) {
-		Hash = NewHash(1, NULL);
+		Hash = NewHash(1, Flathash);
 
 		while (!Done) {
 			len = StrBuf_ServGetln(Buf);
@@ -282,9 +298,29 @@ HashList *iterate_load_userlist(StrBuf *Target, WCTemplputParams *TP)
 			ul = NewUserListEntry(Buf);
 			if (ul == NULL)
 				continue;
-			nUsed = GetCount(Hash);
-			nUsed = snprintf(nnn, sizeof(nnn), "%d", nUsed+1);
-			Put(Hash, nnn, nUsed, ul, DeleteUserListEntry); 
+
+			Put(Hash, IKEY(ul->UID), ul, DeleteUserListEntry); 
+		}
+
+		serv_puts("LBIO 1");
+		StrBuf_ServGetln(Buf);
+		if (GetServerStatus(Buf, NULL) == 1)
+			Done = 0;
+			while (!Done) {
+			len = StrBuf_ServGetln(Buf);
+			if ((len <0) || 
+			    ((len == 3) &&
+			     !strcmp(ChrPtr(Buf), "000")))
+			{
+				Done = 1;
+				break;
+			}
+			UID = atoi(ChrPtr(Buf));
+			if (GetHash(Hash, IKEY(UID), &vData) && vData != 0)
+			{
+				ul = (UserListEntry*)vData;
+				ul->HasBio = 1;
+			}
 		}
 		SubTP.Filter.ContextType = CTX_USERLIST;
 		SortIt = RetrieveSort(&SubTP, HKEY("USER"), HKEY("user:uid"), 0);
@@ -404,6 +440,67 @@ int ConditionalUserAccess(StrBuf *Target, WCTemplputParams *TP)
 		==
 		ul->AccessLevel;
 }
+int ConditionalHaveBIO(StrBuf *Target, WCTemplputParams *TP)
+{
+	UserListEntry *ul = (UserListEntry*) CTX;
+	
+	if (ul == NULL)
+		return 0;
+	return ul->HasBio;
+}
+
+void tmplput_USER_BIO(StrBuf *Target, WCTemplputParams *TP)
+{
+	int Done = 0;
+	StrBuf *Buf;
+	const char *who;
+	long len;
+
+	GetTemplateTokenString(Target, TP, 0, &who, &len);
+
+	Buf = NewStrBuf();
+	serv_printf("RBIO %s", who);
+	StrBuf_ServGetln(Buf);
+	if (GetServerStatus(Buf, NULL) == 1) {
+		StrBuf *BioBuf = NewStrBufPlain(NULL, SIZ);
+		while (!Done && StrBuf_ServGetln(Buf)>=0) {
+			if ( (StrLength(Buf)==3) && 
+			     !strcmp(ChrPtr(Buf), "000")) 
+				Done = 1;
+			else
+				StrBufAppendBuf(BioBuf, Buf, 0);
+		}
+		StrBufAppendTemplate(Target, TP, BioBuf, 1);
+		FreeStrBuf(&BioBuf);
+	}
+	FreeStrBuf(&Buf);
+}
+
+int Conditional_USER_HAS_PIC(StrBuf *Target, WCTemplputParams *TP)
+{
+	StrBuf *Buf;
+	const char *who;
+	long len;
+	int r = 0;
+
+	GetTemplateTokenString(Target, TP, 2, &who, &len);
+
+	Buf = NewStrBuf();
+	serv_printf("OIMG _userpic_|%s", who);
+	StrBuf_ServGetln(Buf);
+	if (GetServerStatus(Buf, NULL) != 2) {
+		r = 1;
+	}
+	else {
+		r = 0;
+	}
+	serv_puts("CLOS");
+	StrBuf_ServGetln(Buf);
+	GetServerStatus(Buf, NULL);
+	FreeStrBuf(&Buf);
+	return(r);
+}
+
 
 /*
  *  Locate the message number of a user's vCard in the current room
@@ -419,7 +516,6 @@ long locate_user_vcard_in_this_room(message_summary **VCMsg, wc_mime_attachment
 	void *vMsg;
 	message_summary *Msg;
 	wc_mime_attachment *Att;
-	int Done;
 	StrBuf *Buf;
 	long vcard_msgnum = (-1L);
 	int already_tried_creating_one = 0;
@@ -434,7 +530,6 @@ TRYAGAIN:
 	Stat.maxload = 10000;
 	Stat.lowest_found = (-1);
 	Stat.highest_found = (-1);
-	Done = 0;
 	/* Search for the user's vCard */
 	if (load_msg_ptrs("MSGS ALL||||1", &Stat, NULL) > 0) {
 		at = GetNewHashPos(WCC->summ, 0);
@@ -485,7 +580,7 @@ TRYAGAIN:
 			serv_puts("000");
 		}
 		else 
-			lprintf(1, "Error while creating user vcard: %s\n", ChrPtr(Buf));
+			syslog(1, "Error while creating user vcard: %s\n", ChrPtr(Buf));
 		goto TRYAGAIN;
 	}
 	FreeStrBuf(&Buf);
@@ -501,7 +596,6 @@ TRYAGAIN:
  *  usernum the citadel-uid of the user
  */
 void display_edit_address_book_entry(const char *username, long usernum) {
-	wcsession *WCC = WC;
 	message_summary *VCMsg = NULL;
 	wc_mime_attachment *VCAtt = NULL;
 	StrBuf *roomname;
@@ -520,9 +614,7 @@ void display_edit_address_book_entry(const char *username, long usernum) {
 		GetServerStatus(Buf, NULL);
 		serv_printf("GOTO %s||1", ChrPtr(roomname));
 		StrBuf_ServGetln(Buf);
-		if (GetServerStatus(Buf, NULL) != 2) {
-			FlushStrBuf(WCC->ImportantMsg);
-			StrBufAppendBuf(WCC->ImportantMsg, Buf, 4);
+		if (GetServerStatusMsg(Buf, NULL, 1, 2) != 2) {
 			select_user_to_edit(username);
 			FreeStrBuf(&Buf);
 			FreeStrBuf(&roomname);
@@ -534,9 +626,7 @@ void display_edit_address_book_entry(const char *username, long usernum) {
 	locate_user_vcard_in_this_room(&VCMsg, &VCAtt);
 
 	if (VCMsg == NULL) {
-		StrBufPlain(WCC->ImportantMsg, 
-			    _("An error occurred while trying to create or edit this address book entry."), 
-			    0);
+		AppendImportantMessage(_("An error occurred while trying to create or edit this address book entry."), -1);
 		select_user_to_edit(username);
 		FreeStrBuf(&roomname);
 		return;
@@ -550,10 +640,25 @@ void display_edit_address_book_entry(const char *username, long usernum) {
 	FreeStrBuf(&roomname);
 }
 
+/*
+ *  burge a user 
+ *  username the name of the user to remove
+ */
+void delete_user(char *username) {
+	StrBuf *Buf;
+	
+	Buf = NewStrBuf();
+	serv_printf("ASUP %s|0|0|0|0|0|", username);
+	StrBuf_ServGetln(Buf);
+	GetServerStatusMsg(Buf, NULL, 1, 2);
+
+	select_user_to_edit( bstr("username"));
+	FreeStrBuf(&Buf);
+}
+		
 
 void display_edituser(const char *supplied_username, int is_new) {
 	const char *Pos;
-	wcsession *WCC = WC;
 	UserListEntry* UL;
 	StrBuf *Buf;
 	char username[256];
@@ -568,9 +673,7 @@ void display_edituser(const char *supplied_username, int is_new) {
 	Buf = NewStrBuf();
 	serv_printf("AGUP %s", username);
 	StrBuf_ServGetln(Buf);
-	if (GetServerStatus(Buf, NULL) != 2) {
-		FlushStrBuf(WCC->ImportantMsg);
-		StrBufAppendBuf(WCC->ImportantMsg, Buf, 4);
+	if (GetServerStatusMsg(Buf, NULL, 1, 2) != 2) {
 		select_user_to_edit(username);
 		FreeStrBuf(&Buf);
 		return;
@@ -603,7 +706,6 @@ void display_edituser(const char *supplied_username, int is_new) {
  *  do the backend operation of the user edit on the server
  */
 void edituser(void) {
-	wcsession *WCC = WC;
 	int is_new = 0;
 	unsigned int flags = 0;
 	const char *username;
@@ -612,7 +714,7 @@ void edituser(void) {
 	username = bstr("username");
 
 	if (!havebstr("ok_button")) {
-		StrBufPlain(WCC->ImportantMsg, _("Changes were not saved."), -1);
+		AppendImportantMessage(_("Changes were not saved."), -1);
 	}	
 	else {
 		StrBuf *Buf = NewStrBuf();
@@ -628,11 +730,7 @@ void edituser(void) {
 		if ((havebstr("newname")) && (strcasecmp(bstr("username"), bstr("newname")))) {
 			serv_printf("RENU %s|%s", bstr("username"), bstr("newname"));
 			StrBuf_ServGetln(Buf);
-			if (GetServerStatus(Buf, NULL) == 2) {
-				FlushStrBuf(WCC->ImportantMsg);
-				StrBufAppendBuf(WCC->ImportantMsg, Buf, 4);				
-			}
-			else {
+			if (GetServerStatusMsg(Buf, NULL, 1, 2) != 2) {
 				username = bstr("newname");
 			}
 		}
@@ -649,9 +747,7 @@ void edituser(void) {
 			bstr("purgedays")
 		);
 		StrBuf_ServGetln(Buf);
-		if (GetServerStatus(Buf, NULL) == 2) {
-			StrBufAppendBuf(WCC->ImportantMsg, Buf, 4);
-		}
+		GetServerStatusMsg(Buf, NULL, 1, 2);
 		FreeStrBuf(&Buf);
 	}
 
@@ -667,24 +763,6 @@ void edituser(void) {
 	}
 }
 
-/*
- *  burge a user 
- *  username the name of the user to remove
- */
-void delete_user(char *username) {
-	wcsession *WCC = WC;
-	StrBuf *Buf;
-	
-	Buf = NewStrBuf();
-	serv_printf("ASUP %s|0|0|0|0|0|", username);
-	StrBuf_ServGetln(Buf);
-	if (GetServerStatus(Buf, NULL) != 2) 
-		StrBufAppendBuf(WCC->ImportantMsg, Buf, 4);
-
-	select_user_to_edit( bstr("username"));
-	FreeStrBuf(&Buf);
-}
-		
 
 
 /*
@@ -692,7 +770,6 @@ void delete_user(char *username) {
  * take the web environment username and create it on the citadel server
  */
 void create_user(void) {
-	wcsession *WCC = WC;
 	long FullState;
 	StrBuf *Buf;
 	const char *username;
@@ -702,19 +779,18 @@ void create_user(void) {
 	serv_printf("CREU %s", username);
 	StrBuf_ServGetln(Buf);
 	if (GetServerStatus(Buf, &FullState) == 2) {
-		sprintf(WC->ImportantMessage, _("A new user has been created."));
+		AppendImportantMessage(_("A new user has been created."), -1);
 		display_edituser(username, 1);
 	}
 	else if (FullState == 570) {
-		StrBufPlain(WCC->ImportantMsg, 
-			    _("You are attempting to create a new user from within Citadel "
-			      "while running in host based authentication mode.  In this mode, "
-			      "you must create new users on the host system, not within Citadel."), 
-			    0);
+		AppendImportantMessage(_("You are attempting to create a new user from within Citadel "
+					 "while running in host based authentication mode.  In this mode, "
+					 "you must create new users on the host system, not within Citadel."), 
+				       -1);
 		select_user_to_edit(NULL);
 	}
 	else {
-		StrBufAppendBuf(WCC->ImportantMsg, Buf, 4);
+		AppendImportantMessage(ChrPtr(Buf) + 4, StrLength(Buf) - 4);
 		select_user_to_edit(NULL);
 	}
 	FreeStrBuf(&Buf);
@@ -730,11 +806,19 @@ void _display_edituser(void) {
 	display_edituser(NULL, 0);
 }
 
+void showuser(void)
+{
+	output_headers(1, 0, 0, 0, 1, 0);
+	do_template("user_show");
+	end_burst();
+}
+
 
 void 
 InitModule_USEREDIT
 (void)
 {
+	WebcitAddUrlHandler(HKEY("showuser"), "", 0, showuser, 0);
 	WebcitAddUrlHandler(HKEY("select_user_to_edit"), "", 0, _select_user_to_edit, 0);
 	WebcitAddUrlHandler(HKEY("display_edituser"), "", 0, _display_edituser, 0);
 	WebcitAddUrlHandler(HKEY("edituser"), "", 0, edituser, 0);
@@ -753,9 +837,14 @@ InitModule_USEREDIT
 	RegisterNamespace("USERLIST:FLAGS",         0, 0, tmplput_USERLIST_Flags, NULL, CTX_USERLIST);
 	RegisterNamespace("USERLIST:DAYSTILLPURGE", 0, 0, tmplput_USERLIST_DaysTillPurge, NULL, CTX_USERLIST);
 
+	RegisterNamespace("USER:BIO", 1, 2, tmplput_USER_BIO,  NULL, CTX_NONE);
+
 	RegisterConditional(HKEY("COND:USERNAME"),  0,    ConditionalUser, CTX_USERLIST);
 	RegisterConditional(HKEY("COND:USERACCESS"), 0,   ConditionalUserAccess, CTX_USERLIST);
 	RegisterConditional(HKEY("COND:USERLIST:FLAG:USE_INTERNET"), 0, ConditionalFlagINetEmail, CTX_USERLIST);
+	RegisterConditional(HKEY("COND:USERLIST:HAVEBIO"), 0, ConditionalHaveBIO, CTX_USERLIST);
+
+	RegisterConditional(HKEY("COND:USER:PIC"), 1, Conditional_USER_HAS_PIC,  CTX_NONE);
 
 	RegisterIterator("USERLIST", 0, NULL, iterate_load_userlist, NULL, DeleteHash, CTX_USERLIST, CTX_NONE, IT_FLAG_DETECT_GROUPCHANGE);
 	
