@@ -1,10 +1,65 @@
 /*
- * $Id$
+ * Copyright (c) 1996-2011 by the citadel.org team
+ *
+ * This program is open source software.  You can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "webcit.h"
+#include "webserver.h"
 #include "calendar.h"
 
+
+
+ConstStr VCStr [] = {
+	{HKEY("n")}, /* N is name, but only if there's no FN already there */
+	{HKEY("fn")}, /* FN (full name) is a true 'display name' field */
+	{HKEY("title")},   /* title */
+	{HKEY("org")},    /* organization */
+	{HKEY("email")},
+	{HKEY("tel")},
+	{HKEY("tel_tel")},
+	{HKEY("tel_work")},
+	{HKEY("tel_home")},
+	{HKEY("tel_cell")},
+	{HKEY("adr")},
+	{HKEY("photo")},
+	{HKEY("version")},
+	{HKEY("rev")},
+	{HKEY("label")}
+};
+
+typedef enum _eVC{
+	VC_n,
+	VC_fn,
+	VC_title,
+	VC_org,
+	VC_email,
+	VC_tel,
+	VC_tel_tel,
+	VC_tel_work,
+	VC_tel_home,
+	VC_tel_cell,
+	VC_adr,
+	VC_photo,
+	VC_version,
+	VC_rev,
+	VC_label
+} eVC;
+
+HashList *VCToEnum = NULL;
+
 /*
  * Record compare function for sorting address book indices
  */
@@ -197,7 +252,7 @@ void fetchname_parsed_vcard(struct vCard *v, char **storename) {
 			}
 		}
 		if (is_qp) {
-			// %ff can become 6 bytes in utf8 
+			/* %ff can become 6 bytes in utf8  */
 			*storename = malloc(len * 2 + 3); 
 			j = CtdlDecodeQuotedPrintable(
 				*storename, name,
@@ -205,7 +260,7 @@ void fetchname_parsed_vcard(struct vCard *v, char **storename) {
 			(*storename)[j] = 0;
 		}
 		else if (is_b64) {
-			// ff will become one byte..
+			/* ff will become one byte.. */
 			*storename = malloc(len + 50);
 			CtdlDecodeBase64(
 				*storename, name,
@@ -267,7 +322,7 @@ void display_parsed_vcard(StrBuf *Target, struct vCard *v, int full, wc_mime_att
 	strcpy(org, "");
 
 	if (!full) {
-		StrBufAppendPrintf(Target, "<TD>");
+		StrBufAppendPrintf(Target, "<td>");
 		name = vcard_get_prop(v, "fn", 1, 0, 0);
 		if (name != NULL) {
 			StrEscAppend(Target, NULL, name, 0, 0);
@@ -280,12 +335,12 @@ void display_parsed_vcard(StrBuf *Target, struct vCard *v, int full, wc_mime_att
 		else {
 			StrBufAppendPrintf(Target, "&nbsp;");
 		}
-		StrBufAppendPrintf(Target, "</TD>");
+		StrBufAppendPrintf(Target, "</td>");
 		return;
 	}
 
-	StrBufAppendPrintf(Target, "<div align=center>"
-		"<table bgcolor=#aaaaaa width=50%%>");
+	StrBufAppendPrintf(Target, "<div align=\"center\">"
+		"<table bgcolor=\"#aaaaaa\" width=\"50%%\">");
 	for (pass=1; pass<=2; ++pass) {
 
 		if (v->numprops) for (i=0; i<(v->numprops); ++i) {
@@ -356,7 +411,7 @@ void display_parsed_vcard(StrBuf *Target, struct vCard *v, int full, wc_mime_att
 	
 			else if (!strcasecmp(firsttoken, "email")) {
 				size_t len;
-				if (!IsEmptyStr(mailto)) strcat(mailto, "<br />");
+				if (!IsEmptyStr(mailto)) strcat(mailto, "<br>");
 				strcat(mailto,
 					"<a href=\"display_enter"
 					"?force_room=_MAIL_?recp=");
@@ -378,7 +433,7 @@ void display_parsed_vcard(StrBuf *Target, struct vCard *v, int full, wc_mime_att
 				strcat(mailto, "</A>");
 			}
 			else if (!strcasecmp(firsttoken, "tel")) {
-				if (!IsEmptyStr(phone)) strcat(phone, "<br />");
+				if (!IsEmptyStr(phone)) strcat(phone, "<br>");
 				strcat(phone, thisvalue);
 				for (j=0; j<num_tokens(thisname, ';'); ++j) {
 					extract_token(buf, thisname, j, ';', sizeof buf);
@@ -399,18 +454,18 @@ void display_parsed_vcard(StrBuf *Target, struct vCard *v, int full, wc_mime_att
 			}
 			else if (!strcasecmp(firsttoken, "adr")) {
 				if (pass == 2) {
-					StrBufAppendPrintf(Target, "<TR><TD>");
+					StrBufAppendPrintf(Target, "<tr><td>");
 					StrBufAppendPrintf(Target, _("Address:"));
-					StrBufAppendPrintf(Target, "</TD><TD>");
+					StrBufAppendPrintf(Target, "</td><td>");
 					for (j=0; j<num_tokens(thisvalue, ';'); ++j) {
 						extract_token(buf, thisvalue, j, ';', sizeof buf);
 						if (!IsEmptyStr(buf)) {
 							StrEscAppend(Target, NULL, buf, 0, 0);
-							if (j<3) StrBufAppendPrintf(Target, "<br />");
+							if (j<3) StrBufAppendPrintf(Target, "<br>");
 							else StrBufAppendPrintf(Target, " ");
 						}
 					}
-					StrBufAppendPrintf(Target, "</TD></TR>\n");
+					StrBufAppendPrintf(Target, "</td></tr>\n");
 				}
 			}
 			/* else if (!strcasecmp(firsttoken, "photo") && full && pass == 2) { 
@@ -448,23 +503,23 @@ void display_parsed_vcard(StrBuf *Target, struct vCard *v, int full, wc_mime_att
 		}
 	
 		if (pass == 1) {
-			StrBufAppendPrintf(Target, "<TR BGCOLOR=\"#AAAAAA\">"
-			"<TD COLSPAN=2 BGCOLOR=\"#FFFFFF\">"
-			"<IMG ALIGN=CENTER src=\"static/viewcontacts_48x.gif\">"
-			"<FONT SIZE=+1><B>");
+			StrBufAppendPrintf(Target, "<tr bgcolor=\"#aaaaaa\">"
+      "<td colspan=2 bgcolor=\"#ffffff\">"
+      "<img align=\"center\" src=\"static/webcit_icons/essen/32x32/contact.png\">"
+      "<font size=\"+1\"><b>");
 			StrEscAppend(Target, NULL, fullname, 0, 0);
-			StrBufAppendPrintf(Target, "</B></FONT>");
+			StrBufAppendPrintf(Target, "</b></font>");
 			if (!IsEmptyStr(title)) {
-				StrBufAppendPrintf(Target, "<div align=right>");
+				StrBufAppendPrintf(Target, "<div align=\"right>\"");
 				StrEscAppend(Target, NULL, title, 0, 0);
 				StrBufAppendPrintf(Target, "</div>");
 			}
 			if (!IsEmptyStr(org)) {
-				StrBufAppendPrintf(Target, "<div align=right>");
+				StrBufAppendPrintf(Target, "<div align=\"right\">");
 				StrEscAppend(Target, NULL, org, 0, 0);
 				StrBufAppendPrintf(Target, "</div>");
 			}
-			StrBufAppendPrintf(Target, "</TD></TR>\n");
+			StrBufAppendPrintf(Target, "</td></tr>\n");
 		
 			if (!IsEmptyStr(phone)) {
 				StrBufAppendPrintf(Target, "<tr><td>");
@@ -483,6 +538,160 @@ void display_parsed_vcard(StrBuf *Target, struct vCard *v, int full, wc_mime_att
 	StrBufAppendPrintf(Target, "</table></div>\n");
 }
 
+/*
+ * html print a vcard
+ * display_vcard() calls this after parsing the textual vCard into
+ * our 'struct vCard' data object.
+ *
+ * Set 'full' to nonzero to display the full card, otherwise it will only
+ * show a summary line.
+ *
+ * This code is a bit ugly, so perhaps an explanation is due: we do this
+ * in two passes through the vCard fields.  On the first pass, we process
+ * fields we understand, and then render them in a pretty fashion at the
+ * end.  Then we make a second pass, outputting all the fields we don't
+ * understand in a simple two-column name/value format.
+ * v		the vCard to display
+ * full		display all items of the vcard?
+ * msgnum	Citadel message pointer
+ */
+void parse_vcard(StrBuf *Target, struct vCard *v, HashList *VC, int full, wc_mime_attachment *Mime)
+{
+	StrBuf *Val = NULL;
+	StrBuf *Swap = NULL;
+	int i, j;
+	char buf[SIZ];
+	int is_qp = 0;
+	int is_b64 = 0;
+	StrBuf *thisname = NULL;
+	char firsttoken[SIZ];
+	void *V;
+
+	Swap = NewStrBuf ();
+	thisname = NewStrBuf();
+	for (i=0; i<(v->numprops); ++i) {
+		is_qp = 0;
+		is_b64 = 0;
+		StrBufPlain(thisname, v->prop[i].name, -1);
+		StrBufLowerCase(thisname);
+		
+		/*len = */extract_token(firsttoken, ChrPtr(thisname), 0, ';', sizeof firsttoken);
+		
+		for (j=0; j<num_tokens(ChrPtr(thisname), ';'); ++j) {
+			extract_token(buf, ChrPtr(thisname), j, ';', sizeof buf);
+			if (!strcasecmp(buf, "encoding=quoted-printable")) {
+				is_qp = 1;
+/*				remove_token(thisname, j, ';');*/
+			}
+			if (!strcasecmp(buf, "encoding=base64")) {
+				is_b64 = 1;
+/*				remove_token(thisname, j, ';');*/
+			}
+		}
+		
+		/* copy over the payload into a StrBuf */
+		Val = NewStrBufPlain(v->prop[i].value, -1);
+			
+		/* if we have some untagged QP, detect it here. */
+		if (is_qp || (strstr(v->prop[i].value, "=?")!=NULL)){
+			StrBuf *b;
+			StrBuf_RFC822_to_Utf8(Swap, Val, NULL, NULL); /* default charset, current charset */
+			b = Val;
+			Val = Swap; 
+			Swap = b;
+			FlushStrBuf(Swap);
+		}
+		else if (is_b64) {
+			StrBufDecodeBase64(Val);
+
+		}
+		syslog(1, "%s [%s][%s]",
+			firsttoken,
+			ChrPtr(Val),
+			v->prop[i].value);
+		if (GetHash(VCToEnum, firsttoken, strlen(firsttoken), &V))
+		{
+			eVC evc = (eVC) V;
+			Put(VC, IKEY(evc), Val, HFreeStrBuf);
+			syslog(1, "[%ul]\n", evc);
+			Val = NULL;
+		}
+		else
+			syslog(1, "[]\n");
+/*
+TODO: check for layer II
+		else 
+		{
+			long max = num_tokens(thisname, ';');
+			firsttoken[len] = '_';
+
+			for (j = 0; j < max; j++) {
+//			firsttoken[len]
+
+				extract_token(buf, thisname, j, ';', sizeof (buf));
+					if (!strcasecmp(buf, "tel"))
+						strcat(phone, "");
+					else if (!strcasecmp(buf, "work"))
+						strcat(phone, _(" (work)"));
+					else if (!strcasecmp(buf, "home"))
+						strcat(phone, _(" (home)"));
+					else if (!strcasecmp(buf, "cell"))
+						strcat(phone, _(" (cell)"));
+					else {
+						strcat(phone, " (");
+						strcat(phone, buf);
+						strcat(phone, ")");
+					}
+				}
+			}
+
+		}
+*/
+	
+		FreeStrBuf(&Val);
+		free(thisname);
+		thisname = NULL;
+	}
+
+}
+
+void tmplput_VCARD_ITEM(StrBuf *Target, WCTemplputParams *TP)
+{
+	HashList *VC = CTX;
+	eVC evc;
+	void *vStr;
+
+	evc = GetTemplateTokenNumber(Target, TP, 0, -1);
+	if (evc != -1)
+	{
+		if (GetHash(VC, IKEY(evc), &vStr))
+		{
+			StrBufAppendTemplate(Target, TP,
+					     (StrBuf*) vStr,
+					     1);
+		}
+	}
+	
+}
+
+void new_vcard (StrBuf *Target, struct vCard *v, int full, wc_mime_attachment *Mime)
+{
+	HashList *VC;
+	WCTemplputParams SubTP;
+
+        memset(&SubTP, 0, sizeof(WCTemplputParams));    
+
+
+	VC = NewHash(0, Flathash);
+	parse_vcard(Target, v, VC, full, Mime);
+
+	SubTP.Filter.ContextType = CTX_VCARD;
+	SubTP.Context = VC;
+
+	DoTemplate(HKEY("test_vcard"), Target, &SubTP);
+	DeleteHash(&VC);
+}
+
 
 
 /*
@@ -533,7 +742,11 @@ void display_vcard(StrBuf *Target,
 		 ((!isalpha(alpha)) && (!isalpha(this_alpha)))
 		) 
 	{
+#ifdef XXX_XXX
+		new_vcard (Target, v, full, Mime);
+#else
 		display_parsed_vcard(Target, v, full, Mime);
+#endif		
 	}
 
 	vcard_free(v);
@@ -562,7 +775,7 @@ void do_addrbook_view(addrbookent *addrbook, int num_ab) {
 	char **tablabels;
 
 	if (num_ab == 0) {
-		wc_printf("<br /><br /><br /><div align=\"center\"><i>");
+		wc_printf("<br><br><br><div align=\"center\"><i>");
 		wc_printf(_("This address book is empty."));
 		wc_printf("</i></div>\n");
 		return;
@@ -576,7 +789,7 @@ void do_addrbook_view(addrbookent *addrbook, int num_ab) {
 
 	tablabels = malloc(num_pages * sizeof (char *));
 	if (tablabels == NULL) {
-		wc_printf("<br /><br /><br /><div align=\"center\"><i>");
+		wc_printf("<br><br><br><div align=\"center\"><i>");
 		wc_printf(_("An internal error has occurred."));
 		wc_printf("</i></div>\n");
 		return;
@@ -604,7 +817,7 @@ void do_addrbook_view(addrbookent *addrbook, int num_ab) {
 				end_tab(page-1, num_pages);
 			}
 			begin_tab(page, num_pages);
-			wc_printf("<table border=0 cellspacing=0 cellpadding=3 width=100%%>\n");
+			wc_printf("<table border=\"0\" cellspacing=\"0\" cellpadding=\"3\" width=\"100%%\">\n");
 			displayed = 0;
 		}
 
@@ -614,7 +827,7 @@ void do_addrbook_view(addrbookent *addrbook, int num_ab) {
 			}
 			bg = 1 - bg;
 			wc_printf("<tr bgcolor=\"#%s\">",
-				(bg ? "DDDDDD" : "FFFFFF")
+				(bg ? "dddddd" : "ffffff")
 			);
 		}
 	
@@ -831,8 +1044,9 @@ void do_edit_vcard(long msgnum, char *partnum,
 	/* Display the form */
 	output_headers(1, 1, 1, 0, 0, 0);
 
-	svput("BOXTITLE", WCS_STRING, _("Edit contact information"));
-	do_template("beginboxx", NULL);
+	do_template("box_begin_1");
+	StrBufAppendBufPlain(WC->WBuf, _("Edit contact information"), -1, 0);
+	do_template("box_begin_2");
 
 	wc_printf("<form method=\"POST\" action=\"submit_vcard\">\n");
 	wc_printf("<input type=\"hidden\" name=\"nonce\" value=\"%d\">\n", WC->nonce);
@@ -843,10 +1057,9 @@ void do_edit_vcard(long msgnum, char *partnum,
 		wc_printf("\">\n");
 	}
 
-	wc_printf("<div class=\"fix_scrollbar_bug\">"
-		"<table class=\"vcard_edit_background\"><tr><td>\n");
+	wc_printf("<table class=\"vcard_edit_background\"><tr><td>\n");
 
-	wc_printf("<table border=0><tr>"
+	wc_printf("<table border=\"0\"><tr>"
 		"<td>%s</td>"
 		"<td>%s</td>"
 		"<td>%s</td>"
@@ -896,7 +1109,7 @@ void do_edit_vcard(long msgnum, char *partnum,
 
 	wc_printf("</td><td>");
 
-	wc_printf("<table border=0>");
+	wc_printf("<table border=\"0\">");
 	wc_printf("<tr><td>");
 	wc_printf(_("PO box:"));
 	wc_printf("</td><td>"
@@ -972,14 +1185,14 @@ void do_edit_vcard(long msgnum, char *partnum,
 	wc_printf("<table border=0><TR>"
 		"<td valign=top>");
 	wc_printf(_("Primary Internet e-mail address"));
-	wc_printf("<br />"
+	wc_printf("<br>"
 		"<input type=\"text\" name=\"primary_inetemail\" "
 		"size=40 maxlength=60 value=\"");
 	escputs(primary_inetemail);
-	wc_printf("\"><br />"
+	wc_printf("\"><br>"
 		"</td><td valign=top>");
 	wc_printf(_("Internet e-mail aliases"));
-	wc_printf("<br />"
+	wc_printf("<br>"
 		"<textarea name=\"other_inetemail\" rows=5 cols=40 width=40>");
 	escputs(other_inetemail);
 	wc_printf("</textarea></td></tr></table>\n");
@@ -1004,7 +1217,7 @@ void do_edit_vcard(long msgnum, char *partnum,
 	);
 	
 	wc_printf("</td></tr></table>\n");
-	do_template("endbox", NULL);
+	do_template("box_end");
 	wDumpContent(1);
 	if (Msg != NULL) {
 		DestroyMessageSummary(Msg);
@@ -1030,11 +1243,11 @@ void edit_vcard(void) {
  *  parse edited vcard from the browser
  */
 void submit_vcard(void) {
-	wcsession *WCC = WC;
 	struct vCard *v;
 	char *serialized_vcard;
 	char buf[SIZ];
 	StrBuf *Buf;
+	const StrBuf *ForceRoom;
 	int i;
 
 	if (!havebstr("ok_button")) { 
@@ -1043,19 +1256,13 @@ void submit_vcard(void) {
 	}
 
 	if (havebstr("force_room")) {
-		if (gotoroom(sbstr("force_room")) != 200) {
-			StrBufAppendBufPlain(WCC->ImportantMsg,
-					     _("Unable to enter the room to save your message"),
-					     -1, 0);
-			StrBufAppendBufPlain(WCC->ImportantMsg,
-					     HKEY(": "), 0);
-			StrBufAppendBuf(WCC->ImportantMsg, sbstr("force_room"), 0);
-			StrBufAppendBufPlain(WCC->ImportantMsg,
-					     HKEY("; "), 0);
-					       
-			StrBufAppendBufPlain(WCC->ImportantMsg,
-					     _("Aborting."),
-					     -1, 0);
+		ForceRoom = sbstr("force_room");
+		if (gotoroom(ForceRoom) != 200) {
+			AppendImportantMessage(_("Unable to enter the room to save your message"), -1);
+			AppendImportantMessage(HKEY(": "));
+			AppendImportantMessage(SKEY(ForceRoom));
+			AppendImportantMessage(HKEY("; "));
+			AppendImportantMessage(_("Aborting."), -1);
 
 			if (!strcmp(bstr("return_to"), "select_user_to_edit")) {
 				select_user_to_edit(NULL);
@@ -1089,10 +1296,7 @@ void submit_vcard(void) {
 	v = VCardLoad(Buf);	/* Start with the extra fields */
 	FreeStrBuf(&Buf);
 	if (v == NULL) {
-		safestrncpy(WCC->ImportantMessage,
-			_("An error has occurred."),
-			sizeof WCC->ImportantMessage
-		);
+		AppendImportantMessage(_("An error has occurred."), -1);
 		edit_vcard();
 		return;
 	}
@@ -1135,10 +1339,7 @@ void submit_vcard(void) {
 	serialized_vcard = vcard_serialize(v);
 	vcard_free(v);
 	if (serialized_vcard == NULL) {
-		safestrncpy(WCC->ImportantMessage,
-			_("An error has occurred."),
-			sizeof WCC->ImportantMessage
-		);
+		AppendImportantMessage(_("An error has occurred."), -1);
 		edit_vcard();
 		return;
 	}
@@ -1287,6 +1488,21 @@ int vcard_Cleanup(void **ViewSpecific)
 	return 0;
 }
 
+void 
+ServerStartModule_VCARD
+(void)
+{
+	VCToEnum = NewHash(0, NULL);
+
+}
+
+void 
+ServerShutdownModule_VCARD
+(void)
+{
+	DeleteHash(&VCToEnum);
+}
+
 void 
 InitModule_VCARD
 (void)
@@ -1302,5 +1518,41 @@ InitModule_VCARD
 	WebcitAddUrlHandler(HKEY("edit_vcard"), "", 0, edit_vcard, 0);
 	WebcitAddUrlHandler(HKEY("submit_vcard"), "", 0, submit_vcard, 0);
 	WebcitAddUrlHandler(HKEY("vcardphoto"), "", 0, display_vcard_photo_img, NEED_URL);
+
+	Put(VCToEnum, HKEY("n"), (void*)VC_n, reference_free_handler);
+	Put(VCToEnum, HKEY("fn"), (void*)VC_fn, reference_free_handler);
+	Put(VCToEnum, HKEY("title"), (void*)VC_title, reference_free_handler);
+	Put(VCToEnum, HKEY("org"), (void*)VC_org, reference_free_handler);
+	Put(VCToEnum, HKEY("email"), (void*)VC_email, reference_free_handler);
+	Put(VCToEnum, HKEY("tel"), (void*)VC_tel, reference_free_handler);
+	Put(VCToEnum, HKEY("tel_tel"), (void*)VC_tel_tel, reference_free_handler);
+	Put(VCToEnum, HKEY("tel_work"), (void*)VC_tel_work, reference_free_handler);
+	Put(VCToEnum, HKEY("tel_home"), (void*)VC_tel_home, reference_free_handler);
+	Put(VCToEnum, HKEY("tel_cell"), (void*)VC_tel_cell, reference_free_handler);
+	Put(VCToEnum, HKEY("adr"), (void*)VC_adr, reference_free_handler);
+	Put(VCToEnum, HKEY("photo"), (void*)VC_photo, reference_free_handler);
+	Put(VCToEnum, HKEY("version"), (void*)VC_version, reference_free_handler);
+	Put(VCToEnum, HKEY("rev"), (void*)VC_rev, reference_free_handler);
+	Put(VCToEnum, HKEY("label"), (void*)VC_label, reference_free_handler);
+
+
+	RegisterNamespace("VC", 1, 2, tmplput_VCARD_ITEM, NULL, CTX_VCARD);
+
+	REGISTERTokenParamDefine(VC_n);
+	REGISTERTokenParamDefine(VC_fn);
+	REGISTERTokenParamDefine(VC_title);
+	REGISTERTokenParamDefine(VC_org);
+	REGISTERTokenParamDefine(VC_email);
+	REGISTERTokenParamDefine(VC_tel);
+	REGISTERTokenParamDefine(VC_tel_tel);
+	REGISTERTokenParamDefine(VC_tel_work);
+	REGISTERTokenParamDefine(VC_tel_home);
+	REGISTERTokenParamDefine(VC_tel_cell);
+	REGISTERTokenParamDefine(VC_adr);
+	REGISTERTokenParamDefine(VC_photo);
+	REGISTERTokenParamDefine(VC_version);
+	REGISTERTokenParamDefine(VC_rev);
+	REGISTERTokenParamDefine(VC_label);
+
 }
 
