@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 1987-2010 by the citadel.org team
+ * Copyright (c) 1987-2012 by the citadel.org team
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
+ * This program is open source software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,7 +11,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "sysdep.h"
@@ -38,10 +36,11 @@
 #ifdef HAVE_LIMITS_H
 #include <limits.h>
 #endif
+#include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <sys/un.h>
 #include <netdb.h>
+#include <sys/un.h>
 #include <sys/poll.h>
 #include <string.h>
 #include <pwd.h>
@@ -49,13 +48,10 @@
 #include <stdarg.h>
 #include <pthread.h>
 #include <signal.h>
+#include <syslog.h>
 #include <sys/utsname.h>
 #include <libcitadel.h>
 
-#ifndef INADDR_NONE
-#define INADDR_NONE 0xffffffff
-#endif
-
 #ifdef HAVE_ICONV
 #include <iconv.h>
 #endif
@@ -72,15 +68,10 @@
 #endif
 
 #define IsEmptyStr(a) ((a)[0] == '\0')
-/*
- * Uncomment to dump an HTTP trace to stderr
-#define HTTP_TRACING 1
- */
 
-#ifdef HTTP_TRACING
-#undef HAVE_ZLIB_H
-#undef HAVE_ZLIB
-#endif
+#define DO_DBG_QR 0
+#define DBG_QR(x) if(DO_DBG_QR) _DBG_QR(x)
+#define DBG_QR2(x) if(DO_DBG_QR) _DBG_QR2(x)
 
 #ifdef HAVE_ZLIB_H
 #include <zlib.h>
@@ -95,6 +86,9 @@
 #undef PACKAGE_TARNAME
 #undef PACKAGE_VERSION
 #undef PACKAGE_BUGREPORT
+
+typedef struct wcsession wcsession;
+
 #include "sysdep.h"
 
 #include "subst.h"
@@ -103,6 +97,8 @@
 #include "roomops.h"
 #include "preferences.h"
 
+#include "tcp_sockets.h"
+#include "utils.h"
 #ifdef HAVE_OPENSSL
 /* Work around RedHat's b0rken OpenSSL includes */
 #define OPENSSL_NO_KRB5
@@ -130,16 +126,16 @@ extern char *ssl_cipher_list;
 
 #define SIZ			4096		/* generic buffer size */
 
-#define TRACE fprintf(stderr, "Checkpoint: %s, %d\n", __FILE__, __LINE__)
+#define TRACE syslog(LOG_DEBUG, "\033[3%dmCHECKPOINT: %s:%d\033[0m", ((__LINE__%6)+1), __FILE__, __LINE__)
 
 #define SLEEPING		180		/* TCP connection timeout */
 #define WEBCIT_TIMEOUT		900		/* WebCit session timeout */
 #define PORT_NUM		2000		/* port number to listen on */
 #define DEVELOPER_ID		0
 #define CLIENT_ID		4
-#define CLIENT_VERSION		786		/* This version of WebCit */
-#define MINIMUM_CIT_VERSION	786		/* min required Citadel ver */
-#define	LIBCITADEL_MIN		786		/* min required libcitadel ver */
+#define CLIENT_VERSION		804		/* This version of WebCit */
+#define MINIMUM_CIT_VERSION	804		/* min required Citadel ver */
+#define	LIBCITADEL_MIN		804		/* min required libcitadel ver */
 #define DEFAULT_HOST		"localhost"	/* Default Citadel server */
 #define DEFAULT_PORT		"504"
 #define TARGET			"webcit01"	/* Target for inline URL's */
@@ -163,43 +159,43 @@ extern char *ssl_cipher_list;
  *
  * bucket one...
  */
-#define QR_PERMANENT	1		/* Room does not purge		*/
+#define QR_PERMANENT	1		/* Room does not purge			*/
 #define QR_INUSE	2		/* Set if in use, clear if avail	*/
 #define QR_PRIVATE	4		/* Set for any type of private room	*/
 #define QR_PASSWORDED	8		/* Set if there's a password too	*/
-#define QR_GUESSNAME	16		/* Set if it's a guessname room	*/
+#define QR_GUESSNAME	16		/* Set if it's a guessname room		*/
 #define QR_DIRECTORY	32		/* Directory room			*/
 #define QR_UPLOAD	64	    	/* Allowed to upload			*/
-#define QR_DOWNLOAD	128		/* Allowed to download		*/
+#define QR_DOWNLOAD	128		/* Allowed to download			*/
 #define QR_VISDIR	256		/* Visible directory			*/
-#define QR_ANONONLY	512		/* Anonymous-Only room		*/
+#define QR_ANONONLY	512		/* Anonymous-Only room			*/
 #define QR_ANONOPT	1024		/* Anonymous-Option room		*/
-#define QR_NETWORK	2048		/* Shared network room		*/
+#define QR_NETWORK	2048		/* Shared network room			*/
 #define QR_PREFONLY	4096		/* Preferred status needed to enter	*/
-#define QR_READONLY	8192		/* Aide status required to post	*/
+#define QR_READONLY	8192		/* Aide status required to post		*/
 #define QR_MAILBOX	16384		/* Set if this is a private mailbox	*/
 
 /*
  * bucket two...
  */
-#define QR2_SYSTEM	1		/* System room; hide by default	*/
+#define QR2_SYSTEM	1		/* System room; hide by default		*/
 #define QR2_SELFLIST	2		/* Self-service mailing list mgmt	*/
-#define QR2_COLLABDEL	4		/* Anyone who can post can also delete*/
-#define QR2_SUBJECTREQ  8               /* Subject strongly recommended */
-#define QR2_SMTP_PUBLIC 16              /* smtp public postable room */
-#define QR2_MODERATED	32		/* Listservice aide has to permit posts  */
+#define QR2_COLLABDEL	4		/* Anyone who can post can also delete	*/
+#define QR2_SUBJECTREQ  8               /* Subject strongly recommended		*/
+#define QR2_SMTP_PUBLIC 16              /* smtp public postable room		*/
+#define QR2_MODERATED	32		/* Listservice aide has to permit posts	*/
 
 /*
  * user/room access
  */
-#define UA_KNOWN	2
-#define UA_GOTOALLOWED	4
-#define UA_HASNEWMSGS	8
-#define UA_ZAPPED	16
+#define UA_KNOWN		2
+#define UA_GOTOALLOWED		4
+#define UA_HASNEWMSGS		8
+#define UA_ZAPPED		16
 #define UA_POSTALLOWED          32
 #define UA_ADMINALLOWED         64
 #define UA_DELETEALLOWED        128
-#define UA_ISTRASH              256 /* Only available in room view... */
+#define UA_ISTRASH              256	/* Only available in room view... */
 
 
 /*
@@ -224,7 +220,7 @@ extern char *ssl_cipher_list;
 
 
 
-
+#define SRV_STATUS_MSG(ServerLineBuf) (ChrPtr(ServerLineBuf) + 4), (StrLength(ServerLineBuf) - 4)
 #define MAJORCODE(a) (((int)(a / 100) ) * 100)
 
 #define LISTING_FOLLOWS	100
@@ -260,9 +256,10 @@ extern char *ssl_cipher_list;
 #define NO_SUCH_SYSTEM 			73	
 #define ALREADY_EXISTS 			74	
 #define MESSAGE_NOT_FOUND 		75
+
 /*
  * NLI is the string that shows up in a who's online listing for sessions
- * that are active, but for which no user has yet authenticated.
+ * that are active but do not (yet) have a user logged in.
  */
 #define NLI	"(not logged in)"
 
@@ -296,20 +293,20 @@ struct urlcontent {
 typedef struct _serv_info {
 	int serv_pid;			/* Process ID of the Citadel server */
 	StrBuf *serv_nodename;		/* Node name of the Citadel server */
-	StrBuf *serv_humannode;	        /* human readable node name of the Citadel server */
-	StrBuf *serv_fqdn;		/* fully quallified Domain Name (such as uncensored.citadel.org) */
-	StrBuf *serv_software;		/* What version does our connected citadel server use */
-	int serv_rev_level;		/* Whats the citadel server revision */
+	StrBuf *serv_humannode;	        /* Juman readable node name of the Citadel server */
+	StrBuf *serv_fqdn;		/* Fully qualified Domain Name (such as uncensored.citadel.org) */
+	StrBuf *serv_software;		/* Free form text description of the server software in use */
+	int serv_rev_level;		/* Server version number (times 100) */
 	StrBuf *serv_bbs_city;		/* Geographic location of the Citadel server */
 	StrBuf *serv_sysadm;		/* Name of system administrator */
-	StrBuf *serv_moreprompt;	/* Whats the commandline textprompt */
 	int serv_supports_ldap;		/* is the server linked against an ldap tree for adresses? */
 	int serv_newuser_disabled;	/* Has the server disabled self-service new user creation? */
 	StrBuf *serv_default_cal_zone;  /* Default timezone for unspecified calendar items */
-	int serv_supports_sieve;	/* Does the server support Sieve mail filtering? */
-	int serv_fulltext_enabled;	/* Does the server have the full text index enabled? */
-	StrBuf *serv_svn_revision;	/* SVN revision of the server */
-	int serv_supports_openid;	/* Does the server support authentication via OpenID? */
+	int serv_supports_sieve;	/* Server supports Sieve mail filtering */
+	int serv_fulltext_enabled;	/* Full text index is enabled */
+	StrBuf *serv_svn_revision;	/* svn or git revision of the server */
+	int serv_supports_openid;	/* Server supports authentication via OpenID */
+	int serv_supports_guest;	/* Server supports unauthenticated guest logins */
 } ServInfo;
 
 
@@ -357,6 +354,14 @@ typedef struct _addrbookent {
 #define PARSE_REST_URL (1<<9)
 #define PROHIBIT_STARTPAGE (1<<10)
 
+
+#define DATEFMT_FULL 0
+#define DATEFMT_BRIEF 1
+#define DATEFMT_RAWDATE 2
+#define DATEFMT_LOCALEDATE 3
+void webcit_fmt_date(char *buf, size_t siz, time_t thetime, int Format);
+
+
 typedef enum _RESTDispatchID {
 	ExistsID,
 	PutID,
@@ -432,7 +437,6 @@ typedef struct _HdrRefs {
 	StrBuf *ContentType;
 	StrBuf *RawCookie;
 	StrBuf *ReqLine;
-	StrBuf *http_host;			/* HTTP Host: header */
 	StrBuf *browser_host;
 	StrBuf *browser_language;
 	StrBuf *user_agent;
@@ -453,6 +457,7 @@ typedef struct _ParsedHttpHdrs {
 	StrBuf *c_language;
 	StrBuf *this_page;			/* URL of current page */
 	StrBuf *PlainArgs; 
+	StrBuf *HostHeader;
 
 	HashList *urlstrings;		        /* variables passed to webcit in a URL */
 	HashList *HTTPHeaders;                  /* the headers the client sent us */
@@ -466,21 +471,19 @@ typedef struct _ParsedHttpHdrs {
  * One of these is kept for each active Citadel session.
  * HTTP transactions are bound to one at a time.
  */
-typedef struct wcsession wcsession;
 struct wcsession {
 /* infrastructural members */
 	wcsession *next;			/* Linked list */
 	pthread_mutex_t SessionMutex;		/* mutex for exclusive access */
 	int wc_session;				/* WebCit session ID */
 	int killthis;				/* Nonzero == purge this session */
-	int is_mobile;			        /* Client is a handheld browser */
 	int ctdl_pid;				/* Session ID on the Citadel server */
 	int nonce;				/* session nonce (to prevent session riding) */
-	int SessionKey;
+	int inuse;				/* set to nonzero if bound to a running thread */
 
 /* Session local Members */
 	int serv_sock;				/* Client socket to Citadel server */
-	StrBuf *ReadBuf;                        /* here we keep our stuff while reading linebuffered from the server. */
+	StrBuf *ReadBuf;                        /* linebuffered reads from the server */
 	StrBuf *MigrateReadLineBuf;             /* here we buffer legacy server read stuff */
 	const char *ReadPos;                    /* whats our read position in ReadBuf? */
 	int last_chat_seq;			/* When in chat - last message seq# we saw */
@@ -488,6 +491,7 @@ struct wcsession {
 	time_t last_pager_check;		/* last time we polled for instant msgs */
 	ServInfo *serv_info;			/* Information about the citserver we're connected to */
 	int is_ajax;                            /* are we doing an ajax request? */
+	StrBuf *PushedDestination;		/* Where to go after login, registration, etc. */
 
 /* Request local Members */
 	StrBuf *CLineBuf;                       /* linebuffering client stuff */
@@ -495,13 +499,10 @@ struct wcsession {
 	StrBuf *WBuf;                           /* Our output buffer */
 	StrBuf *HBuf;                           /* Our HeaderBuffer */
 	StrBuf *WFBuf;                          /* Wildfire error logging buffer */
-
-	HashList *vars; 			/* HTTP variable substitutions for this page */
 	StrBuf *trailing_javascript;		/* extra javascript to be appended to page */
-	char ImportantMessage[SIZ];
 	StrBuf *ImportantMsg;
 	HashList *Directory;			/* Parts of the directory URL in snippets */
-	const Floor *CurrentFloor;              /**< when Parsing REST, which floor are we on? */
+	const Floor *CurrentFloor;              /* when Parsing REST, which floor are we on? */
 
 /* accounting */
 	StrBuf *wc_username;			/* login name of current user */
@@ -538,7 +539,6 @@ struct wcsession {
 	StrBuf *upload_filename;		/* filename of http-uploaded data */
 	char upload_content_type[256];		/* content type of http-uploaded data */
 
-	int new_mail;				/* user has new mail waiting */
 	int remember_new_mail;			/* last count of new mail messages */
 
 /* Roomiew control */
@@ -556,19 +556,27 @@ struct wcsession {
 	char last_chat_user[256];
 
 /* Iconbar controls */
-	struct __ofolder *cache_fold;		/* cache the iconbar room list */
 	int cache_max_folders;
 	int cache_num_floors;
-	time_t cache_timestamp;
 	long *IBSettingsVec;                    /* which icons should be shown / not shown? */
 	const StrBuf *floordiv_expanded;	/* which floordiv currently expanded */
+	int ib_wholist_expanded;
+	int ib_roomlist_expanded;
 
+/* our known Sieve scripts; loaded by SIEVE:SCRIPTS iterator. */
+	HashList *KnownSieveScripts;
 
+/* Transcoding cache buffers; used to avoid to frequent realloc */
+	StrBuf *ConvertBuf1;
+	StrBuf *ConvertBuf2;
 
 /* cache stuff for templates. TODO: find a smartrer way */
 	HashList *ServCfg;                      /* cache our server config for editing */
 	HashList *InetCfg;                      /* Our inet server config for editing */
 	ExpirePolicy Policy[maxpolicy];
+
+/* used by the blog viewer */
+	int bptlid;				/* hash of thread currently being rendered */
 };
 
 
@@ -589,11 +597,12 @@ enum {
 	MAX_SEMAPHORES
 };
 
-
 #ifndef num_parms
 #define num_parms(source)		num_tokens(source, '|') 
 #endif
 
+#define site_prefix	(WC ? (WC->Hdr->HostHeader) : NULL)
+
 /* Per-session data */
 #define WC ((struct wcsession *)pthread_getspecific(MyConKey))
 extern pthread_key_t MyConKey;
@@ -606,25 +615,61 @@ extern char ctdl_key_dir[PATH_MAX];
 extern char file_crpt_file_key[PATH_MAX];
 extern char file_crpt_file_csr[PATH_MAX];
 extern char file_crpt_file_cer[PATH_MAX];
+
+void init_ssl(void);
+void endtls(void);
+void ssl_lock(int mode, int n, const char *file, int line);
+int starttls(int sock);
+extern SSL_CTX *ssl_ctx;  
+int client_read_sslbuffer(StrBuf *buf, int timeout);
+void client_write_ssl(const StrBuf *Buf);
 #endif
 
-extern char floorlist[128][SIZ];
-extern char *axdefs[];
-extern char *ctdlhost, *ctdlport;
-extern int http_port;
-extern char *server_cookie;
 extern int is_https;
-extern int setup_wizard;
-extern char wizard_filename[];
 extern int follow_xff;
+extern char *server_cookie;
+extern char *ctdlhost, *ctdlport;
+extern char *axdefs[];
 extern int num_threads_existing;
 extern int num_threads_executing;
+extern int setup_wizard;
+extern char wizard_filename[];
 
 void InitialiseSemaphores(void);
 void begin_critical_section(int which_one);
 void end_critical_section(int which_one);
 
-void stuff_to_cookie(int unset_cookie);
+
+extern void do_404(void);
+void http_redirect(const char *);
+
+
+#ifdef UBER_VERBOSE_DEBUGGING
+#define wc_printf(...) wcc_printf(__FILE__, __FUNCTION__, __LINE__, __VA_ARGS__)
+void wcc_printf(const char *FILE, const char *FUNCTION, long LINE, const char *format, ...);
+#else 
+void wc_printf(const char *format,...)__attribute__((__format__(__printf__,1,2)));
+#endif
+
+void hprintf(const char *format,...)__attribute__((__format__(__printf__,1,2)));
+
+void CheckAuthBasic(ParsedHttpHdrs *hdr);
+void GetAuthBasic(ParsedHttpHdrs *hdr);
+
+void sleeeeeeeeeep(int);
+
+size_t wc_strftime(char *s, size_t max, const char *format, const struct tm *tm);
+void fmt_time(char *buf, size_t siz, time_t thetime);
+void httpdate(char *buf, time_t thetime);
+time_t httpdate_to_timestamp(StrBuf *buf);
+
+
+
+
+void end_webcit_session(void);
+
+
+
 
 void cookie_to_stuff(StrBuf *cookie,
 		int *session,
@@ -635,45 +680,30 @@ void cookie_to_stuff(StrBuf *cookie,
 );
 void locate_host(StrBuf *TBuf, int);
 void become_logged_in(const StrBuf *user, const StrBuf *pass, StrBuf *serv_response);
-void openid_manual_create(void);
+
 void display_login(void);
 void display_openids(void);
+void display_default_landing_page(void);
 void do_welcome(void);
-void do_logout(void);
+
+void display_reg(int during_login);
 void display_main_menu(void);
 void display_aide_menu(void);
-void display_advanced_menu(void);
-void slrp_highest(void);
-ServInfo *get_serv_info(StrBuf *, StrBuf *);
+
 void RegisterEmbeddableMimeType(const char *MimeType, long MTLen, int Priority);
 void CreateMimeStr(void);
-int GetConnected(void);
-void DeleteServInfo(ServInfo **FreeMe);
-int uds_connectsock(char *);
-int tcp_connectsock(char *, int);
-int serv_getln(char *strbuf, int bufsize);
-int StrBuf_ServGetln(StrBuf *buf);
-int GetServerStatus(StrBuf *Line, long* FullState);
-void serv_puts(const char *string);
-void who(void);
-void who_inner_div(void);
-void ajax_mini_calendar(void);
-void fmout(char *align);
-void _fmout(StrBuf *Targt, char *align);
-void FmOut(StrBuf *Target, char *align, StrBuf *Source);
-void pullquote_fmout(void);
+
+
+void pop_destination(void);
+
+void fmout(const char *align);
+void _fmout(StrBuf *Targt, const char *align);
+void FmOut(StrBuf *Target, const char *align, const StrBuf *Source);
 void wDumpContent(int);
 
 
+void PutRequestLocalMem(void *Data, DeleteHashDataFunc DeleteIt);
 
-void UrlescPutStrBuf(const StrBuf *strbuf);
-void StrEscPuts(const StrBuf *strbuf);
-void StrEscputs1(const StrBuf *strbuf, int nbsp, int nolinebreaks);
-
-void urlescputs(const char *);
-void hurlescputs(const char *);
-void jsesc(char *, size_t, char *);
-void jsescputs(char *);
 void output_headers(    int do_httpheaders,
 			int do_htmlhead,
 			int do_room_banner,
@@ -681,79 +711,23 @@ void output_headers(    int do_httpheaders,
 			int suppress_check,
 			int cache);
 void output_custom_content_header(const char *ctype);
+void cdataout(char *rawdata);
 
-#ifdef UBER_VERBOSE_DEBUGGING
-#define wc_printf(...) wcc_printf(__FILE__, __FUNCTION__, __LINE__, __VA_ARGS__)
-void wcc_printf(const char *FILE, const char *FUNCTION, long LINE, const char *format, ...);
-#else 
-void wc_printf(const char *format,...)__attribute__((__format__(__printf__,1,2)));
-#endif
 
-void hprintf(const char *format,...)__attribute__((__format__(__printf__,1,2)));
-void output_static(const char* What);
-
-long stresc(char *target, long tSize, char *strbuf, int nbsp, int nolinebreaks);
-void escputs(const char *strbuf);
 void url(char *buf, size_t bufsize);
 void UrlizeText(StrBuf* Target, StrBuf *Source, StrBuf *WrkBuf);
-void escputs1(const char *strbuf, int nbsp, int nolinebreaks);
-void msgesc(char *target, size_t tlen, char *strbuf);
-void msgescputs(char *strbuf);
-void msgescputs1(char *strbuf);
-void dump_vars(void);
-void embed_main_menu(void);
 
-void do_addrbook_view(addrbookent *addrbook, int num_ab);
-void fetch_ab_name(message_summary *Msg, char **namebuf);
+
 void display_vcard(StrBuf *Target, wc_mime_attachment *Mime, char alpha, int full, char **storename, long msgnum);
-void jsonMessageList(void);
-void new_summary_view(void);
-void getseen(void);
-void text_to_server(char *ptr);
-void text_to_server_qp(char *ptr);
-void confirm_delete_msg(void);
-void display_success(char *);
-void CheckAuthBasic(ParsedHttpHdrs *hdr);
-void GetAuthBasic(ParsedHttpHdrs *hdr);
-void server_to_text(void);
-void save_edit(char *description, char *enter_cmd, int regoto);
-void display_edit(char *description, char *check_cmd,
-		  char *read_cmd, char *save_cmd, int with_room_banner);
-long gotoroom(const StrBuf *gname);
-void remove_march(const StrBuf *aaa);
-void dotskip(void);
-void confirm_delete_room(void);
-void validate(void);
-void display_graphics_upload(char *, char *, char *);
-void do_graphics_upload(char *upl_cmd);
-void serv_gets(char *strbuf);
-void serv_write(const char *buf, int nbytes);
-void serv_putbuf(const StrBuf *string);
-void serv_printf(const char *format,...)__attribute__((__format__(__printf__,1,2)));
-void load_floorlist(StrBuf *Buf);
+
+void display_success(const char *successmessage);
+
 void shutdown_sessions(void);
-void do_housekeeping(void);
-void smart_goto(const StrBuf *);
-void worker_entry(void);
-void session_loop(void);
-size_t wc_strftime(char *s, size_t max, const char *format, const struct tm *tm);
-void fmt_time(char *buf, size_t siz, time_t thetime);
-void httpdate(char *buf, time_t thetime);
-time_t httpdate_to_timestamp(StrBuf *buf);
-void end_webcit_session(void);
-void page_popup(void);
-void http_redirect(const char *);
-void clear_substs(struct wcsession *wc);
-void clear_local_substs(void);
 
 
 
-int lingering_close(int fd);
-long extract_token(char *dest, const char *source, int parmnum, char separator, int maxlen);
-void remove_token(char *source, int parmnum, char separator);
 StrBuf *load_mimepart(long msgnum, char *partnum);
 void MimeLoadData(wc_mime_attachment *Mime);
-int pattern2(char *search, char *patn);
 void do_edit_vcard(long msgnum, char *partnum, 
 		   message_summary *VCMsg,
 		   wc_mime_attachment *VCAtt,
@@ -761,123 +735,53 @@ void do_edit_vcard(long msgnum, char *partnum,
 		   const char *force_room);
 
 void select_user_to_edit(const char *preselect);
-void delete_user(char *);
-void do_change_view(int);
-void folders(void);
 
-
-
-void offer_start_page(StrBuf *Target, WCTemplputParams *TP);
 void convenience_page(const char *titlebarcolor, const char *titlebarmsg, const char *messagetext);
 void output_html(const char *, int, int, StrBuf *, StrBuf *);
-void do_listsub(void);
+
 ssize_t write(int fd, const void *buf, size_t count);
 void cal_process_attachment(wc_mime_attachment *Mime);
-void display_calendar(message_summary *Msg, int unread);
-void display_note(message_summary *Msg, int unread);
-void updatenote(void);
-void do_tasks_view(void);
-int calendar_summary_view(void);
-void free_march_list(wcsession *wcf);
-void display_rules_editor_inner_div(void);
+
 void generate_uuid(char *);
-void CtdlMakeTempFileName(char *, int);
+
 void address_book_popup(void);
 void begin_ajax_response(void);
 void end_ajax_response(void);
-void initialize_viewdefs(void);
-void initialize_axdefs(void);
-void burn_folder_cache(time_t age);
-void list_all_rooms_by_floor(const char *viewpref);
-void display_pictureview(void);
-
-void display_edit_task(void);
-void display_edit_event(void);
-icaltimezone *get_default_icaltimezone(void);
-void display_icaltimetype_as_webform(struct icaltimetype *, char *, int);
-void icaltime_from_webform(struct icaltimetype *result, char *prefix);
-void icaltime_from_webform_dateonly(struct icaltimetype *result, char *prefix);
-void partstat_as_string(char *buf, icalproperty *attendee);
-icalcomponent *ical_encapsulate_subcomponent(icalcomponent *subcomp);
-void check_attendee_availability(icalcomponent *supplied_vevent);
-int ical_ctdl_is_overlap(
-                        struct icaltimetype t1start,
-                        struct icaltimetype t1end,
-                        struct icaltimetype t2start,
-                        struct icaltimetype t2end
-);
 
 
 extern char *months[];
 extern char *days[];
-int serv_read_binary(StrBuf *Ret, size_t total_len, StrBuf *Buf);
-int StrBuf_ServGetBLOB(StrBuf *buf, long BlobSize);
-int StrBuf_ServGetBLOBBuffered(StrBuf *buf, long BlobSize);
-int read_server_text(StrBuf *Buf, long *nLines);
 long locate_user_vcard_in_this_room(message_summary **VCMsg,
 				    wc_mime_attachment **VCAtt);
-void sleeeeeeeeeep(int);
 void http_transmit_thing(const char *content_type, int is_static);
 long unescape_input(char *buf);
-void do_selected_iconbar(void);
 void check_thread_pool_size(void);
-void spawn_another_worker_thread(void);
 void StrEndTab(StrBuf *Target, int tabnum, int num_tabs);
-void StrBeginTab(StrBuf *Target, int tabnum, int num_tabs);
+void StrBeginTab(StrBuf *Target, int tabnum, int num_tabs, StrBuf **Names);
 void StrTabbedDialog(StrBuf *Target, int num_tabs, StrBuf *tabnames[]);
 void tabbed_dialog(int num_tabs, char *tabnames[]);
 void begin_tab(int tabnum, int num_tabs);
 void end_tab(int tabnum, int num_tabs);
-void str_wiki_index(char *s);
-long guess_calhourformat(void);
+
+
 int get_time_format_cached (void);
-const char *get_selected_language(void);
 void display_wiki_pagelist(void);
-
-#define DATEFMT_FULL 0
-#define DATEFMT_BRIEF 1
-#define DATEFMT_RAWDATE 2
-#define DATEFMT_LOCALEDATE 3
-void webcit_fmt_date(char *buf, size_t siz, time_t thetime, int Format);
-int fetch_http(char *url, char *target_buf, int maxbytes);
-void free_attachments(wcsession *sess);
-void summary(void);
-
-int is_mobile_ua(char *user_agent);
-
-void embed_room_banner(char *, int);
-HashList *GetFloorListHash(StrBuf *Target, WCTemplputParams *TP);
-HashList *GetRoomListHash(StrBuf *Target, WCTemplputParams *TP);
-int SortRoomsByListOrder(const void *room1, const void *room2);
-/* navbar types that can be passed to embed_room_banner */
-enum {
-	navbar_none,
-	navbar_default
-};
+HashList *GetRoomListHashLKRA(StrBuf *Target, WCTemplputParams *TP);
 
 /* actual supported locales */
 void TmplGettext(StrBuf *Target, WCTemplputParams *TP);
-void offer_languages(StrBuf *Target, int nArgs, WCTemplateToken *Token, void *Context, int ContextType);
+
 void set_selected_language(const char *);
 void go_selected_language(void);
 void stop_selected_language(void);
-
-#ifdef HAVE_OPENSSL
-void init_ssl(void);
-void endtls(void);
-void ssl_lock(int mode, int n, const char *file, int line);
-int starttls(int sock);
-extern SSL_CTX *ssl_ctx;  
-int client_read_sslbuffer(StrBuf *buf, int timeout);
-void client_write_ssl(const StrBuf *Buf);
-#endif
+const char *get_selected_language(void);
 
 void utf8ify_rfc822_string(char **buf);
 
 void begin_burst(void);
 long end_burst(void);
 
-extern char *hourname[];	/* Names of hours (12am, 1am, etc.) */
+void AppendImportantMessage(const char *pch, long len);
 
 void http_datestring(char *buf, size_t n, time_t xtime);
 
@@ -899,3 +803,26 @@ void http_datestring(char *buf, size_t n, time_t xtime);
 #define WC_TIMEFORMAT_24 2
 
 extern int time_to_die;			/* Nonzero if server is shutting down */
+
+/* 
+ * Array type for a blog post.  The first message is the post; the rest are comments
+ */
+struct blogpost {
+	int top_level_id;
+	long *msgs;		/* Array of msgnums for messages we are displaying */
+	int num_msgs;		/* Number of msgnums stored in 'msgs' */
+	int alloc_msgs;		/* Currently allocated size of array */
+};
+
+
+/*
+ * Data which gets returned from a call to blogview_learn_thread_references()
+ */
+struct bltr {
+	int id;
+	int refs;
+};
+
+
+struct bltr blogview_learn_thread_references(long msgnum);
+void tmplput_blog_permalink(StrBuf *Target, WCTemplputParams *TP);
