@@ -1,13 +1,19 @@
 /*
- * $Id$
+ * Copyright (c) 1996-2011 by the citadel.org team
  *
- * This contains a simple multithreaded TCP server manager.  It sits around
- * waiting on the specified port for incoming HTTP connections.  When a
- * connection is established, it calls context_loop() from context_loop.c.
+ * This program is open source software.  You can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
+ * License, or (at your option) any later version.
  *
- * Copyright (c) 1996-2010 by the citadel.org developers.
- * This program is released under the terms of the GNU General Public License v3.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "webcit.h"
@@ -18,39 +24,34 @@
 int vsnprintf(char *buf, size_t max, const char *fmt, va_list argp);
 #endif
 
-
-extern int msock;			/* master listening socket */
-extern int verbosity;		/* Logging level */
+extern int msock;				/* master listening socket */
 extern char static_icon_dir[PATH_MAX];          /* where should we find our mime icons */
-int is_https = 0;		/* Nonzero if I am an HTTPS service */
-int follow_xff = 0;		/* Follow X-Forwarded-For: header */
-int home_specified = 0;		/* did the user specify a homedir? */
+int is_https = 0;				/* Nonzero if I am an HTTPS service */
+int follow_xff = 0;				/* Follow X-Forwarded-For: header? */
 int DisableGzip = 0;
+char *default_landing_page = NULL;
 extern pthread_mutex_t SessionListMutex;
 extern pthread_key_t MyConKey;
 
 extern void *housekeeping_loop(void);
-extern int ig_tcp_server(char *ip_addr, int port_number, int queue_len);
-extern int ig_uds_server(char *sockpath, int queue_len);
+extern int webcit_tcp_server(char *ip_addr, int port_number, int queue_len);
+extern int webcit_uds_server(char *sockpath, int queue_len);
 extern void graceful_shutdown_watcher(int signum);
 extern void graceful_shutdown(int signum);
 extern void start_daemon(char *pid_file);
 extern void webcit_calc_dirs_n_files(int relh, const char *basedir, int home, char *webcitdir, char *relhome);
-
+extern void worker_entry(void);
 extern void drop_root(uid_t UID);
 
-char socket_dir[PATH_MAX];			/* where to talk to our citadel server */
-
+char socket_dir[PATH_MAX];	/* where to talk to our citadel server */
 char *server_cookie = NULL;	/* our Cookie connection to the client */
 int http_port = PORT_NUM;	/* Port to listen on */
-char *ctdlhost = DEFAULT_HOST;	/* our name */
-char *ctdlport = DEFAULT_PORT;	/* our Port */
-int setup_wizard = 0;		/* should we run the setup wizard? \todo */
-char wizard_filename[PATH_MAX];	/* where's the setup wizard? */
+char *ctdlhost = DEFAULT_HOST;	/* Host name or IP address of Citadel server */
+char *ctdlport = DEFAULT_PORT;	/* Port number of Citadel server */
+int setup_wizard = 0;		/* should we run the setup wizard? */
+char wizard_filename[PATH_MAX];	/* location of file containing the last webcit version against which we ran setup wizard */
 int running_as_daemon = 0;	/* should we deamonize on startup? */
 
-
-
 /* #define DBG_PRINNT_HOOKS_AT_START */
 #ifdef DBG_PRINNT_HOOKS_AT_START
 extern HashList *HandlerHash;
@@ -58,15 +59,13 @@ const char foobuf[32];
 const char *nix(void *vptr) {snprintf(foobuf, 32, "%0x", (long) vptr); return foobuf;}
 #endif 
 extern int dbg_analyze_msg;
-extern int dbg_bactrace_template_errors;
+extern int dbg_backtrace_template_errors;
 extern int DumpTemplateI18NStrings;
 extern StrBuf *I18nDump;
 void InitTemplateCache(void);
 extern int LoadTemplates;
 
 
-
-
 /*
  * Here's where it all begins.
  */
@@ -77,11 +76,9 @@ int main(int argc, char **argv)
 	pthread_t SessThread;		/* Thread descriptor */
 	pthread_attr_t attr;		/* Thread attributes */
 	int a;		        	/* General-purpose variable */
-	char tracefile[PATH_MAX];
-	char ip_addr[256]="0.0.0.0";
+	char ip_addr[256]="*";
 	int relh=0;
 	int home=0;
-	int home_specified=0;
 	char relhome[PATH_MAX]="";
 	char webcitdir[PATH_MAX] = DATADIR;
 	char *pidfile = NULL;
@@ -89,12 +86,11 @@ int main(int argc, char **argv)
 	const char *basedir = NULL;
 	char uds_listen_path[PATH_MAX];	/* listen on a unix domain socket? */
 	const char *I18nDumpFile = NULL;
-	FILE *rvfp = NULL;
 	int rv = 0;
 
 	WildFireInitBacktrace(argv[0], 2);
 
-	start_modules ();
+	start_modules();
 
 #ifdef DBG_PRINNT_HOOKS_AT_START
 /*	dbg_PrintHash(HandlerHash, nix, NULL);*/
@@ -113,9 +109,9 @@ int main(int argc, char **argv)
 
 	/* Parse command line */
 #ifdef HAVE_OPENSSL
-	while ((a = getopt(argc, argv, "u:h:i:p:t:T:B:x:dD:G:cfsS:Z")) != EOF)
+	while ((a = getopt(argc, argv, "u:h:i:p:t:T:B:x:g:dD:G:cfsS:Z")) != EOF)
 #else
-	while ((a = getopt(argc, argv, "u:h:i:p:t:T:B:x:dD:G:cfZ")) != EOF)
+	while ((a = getopt(argc, argv, "u:h:i:p:t:T:B:x:g:dD:G:cfZ")) != EOF)
 #endif
 		switch (a) {
 		case 'u':
@@ -131,7 +127,6 @@ int main(int argc, char **argv)
 				safestrncpy(relhome, relhome, sizeof relhome);
 			}
 			/* free(hdir); TODO: SHOULD WE DO THIS? */
-			home_specified = 1;
 			home=1;
 			break;
 		case 'd':
@@ -141,6 +136,9 @@ int main(int argc, char **argv)
 			pidfile = strdup(optarg);
 			running_as_daemon = 1;
 			break;
+		case 'g':
+			default_landing_page = strdup(optarg);
+			break;
 		case 'B': /* Basesize */
 			basesize = atoi(optarg);
 			if (basesize > 2)
@@ -156,21 +154,18 @@ int main(int argc, char **argv)
 			}
 			break;
 		case 't':
-			safestrncpy(tracefile, optarg, sizeof tracefile);
-			rvfp = freopen(tracefile, "w", stdout);
-			rvfp = freopen(tracefile, "w", stderr);
-			rvfp = freopen(tracefile, "r", stdin);
+			/* no longer used, but ignored so old scripts don't break */
 			break;
 		case 'T':
 			LoadTemplates = atoi(optarg);
 			dbg_analyze_msg = (LoadTemplates && (1<<1)) != 0;
-			dbg_bactrace_template_errors = (LoadTemplates && (1<<2)) != 0;
+			dbg_backtrace_template_errors = (LoadTemplates && (1<<2)) != 0;
 			break;
 		case 'Z':
 			DisableGzip = 1;
 			break;
 		case 'x':
-			verbosity = atoi(optarg);
+			/* no longer used, but ignored so old scripts don't break */
 			break;
 		case 'f':
 			follow_xff = 1;
@@ -184,8 +179,7 @@ int main(int argc, char **argv)
 				if (gethostname
 				    (&server_cookie[strlen(server_cookie)],
 				     200) != 0) {
-					lprintf(2, "gethostname: %s\n",
-						strerror(errno));
+					syslog(2, "gethostname: %s", strerror(errno));
 					free(server_cookie);
 				}
 			}
@@ -207,7 +201,7 @@ int main(int argc, char **argv)
 		default:
 			fprintf(stderr, "usage: webcit "
 				"[-i ip_addr] [-p http_port] "
-				"[-t tracefile] [-c] [-f] "
+				"[-c] [-f] "
 				"[-T Templatedebuglevel] "
 				"[-d] [-Z] [-G i18ndumpfile] "
 #ifdef HAVE_OPENSSL
@@ -217,6 +211,12 @@ int main(int argc, char **argv)
 			return 1;
 		}
 
+	/* Start the logger */
+	openlog("webcit",
+		( running_as_daemon ? (LOG_PID) : (LOG_PID | LOG_PERROR) ),
+		LOG_DAEMON
+	);
+
 	if (optind < argc) {
 		ctdlhost = argv[optind];
 		if (++optind < argc)
@@ -236,34 +236,42 @@ int main(int argc, char **argv)
 	LoadIconDir(static_icon_dir);
 
 	/* Tell 'em who's in da house */
-	lprintf(1, PACKAGE_STRING "\n");
-	lprintf(1, "Copyright (C) 1996-2010 by the Citadel development team.\n"
-		"This software is distributed under the terms of the "
-		"GNU General Public License.\n\n"
-	);
+	syslog(1, "%s", PACKAGE_STRING);
+	syslog(1, "Copyright (C) 1996-2011 by the citadel.org team");
+	syslog(1, " ");
+	syslog(1, "This program is open source software: you can redistribute it and/or");
+	syslog(1, "modify it under the terms of the GNU General Public License as published");
+	syslog(1, "by the Free Software Foundation, either version 3 of the License, or");
+	syslog(1, "(at your option) any later version.");
+	syslog(1, " ");
+	syslog(1, "This program is distributed in the hope that it will be useful,");
+	syslog(1, "but WITHOUT ANY WARRANTY; without even the implied warranty of");
+	syslog(1, "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the");
+	syslog(1, "GNU General Public License for more details.");
+	syslog(1, " ");
+	syslog(1, "You should have received a copy of the GNU General Public License");
+	syslog(1, "along with this program.  If not, see <http://www.gnu.org/licenses/>.");
+	syslog(1, " ");
 
 
-	/* initialize the International Bright Young Thing */
+	/* initialize various subsystems */
 
 	initialise_modules();
-	initialize_viewdefs();
-	initialize_axdefs();
-
 	InitTemplateCache();
 	if (DumpTemplateI18NStrings) {
 		FILE *fd;
 		StrBufAppendBufPlain(I18nDump, HKEY("}\n"), 0);
 	        if (StrLength(I18nDump) < 50) {
-			lprintf(1, "********************************************************************************\n");
-			lprintf(1, "*        No strings found in templates!  Are you sure they're there?           *\n");
-			lprintf(1, "********************************************************************************\n");
+			syslog(1, "*******************************************************************\n");
+			syslog(1, "*   No strings found in templates!  Are you sure they're there?   *\n");
+			syslog(1, "*******************************************************************\n");
 			return -1;
 		}
 		fd = fopen(I18nDumpFile, "w");
 	        if (fd == NULL) {
-			lprintf(1, "********************************************************************************\n");
-			lprintf(1, "*                  unable to open I18N dumpfile [%s]         *\n", I18nDumpFile);
-			lprintf(1, "********************************************************************************\n");
+			syslog(1, "***********************************************\n");
+			syslog(1, "*   unable to open I18N dumpfile [%s]         *\n", I18nDumpFile);
+			syslog(1, "***********************************************\n");
 			return -1;
 		}
 		rv = fwrite(ChrPtr(I18nDump), 1, StrLength(I18nDump), fd);
@@ -271,7 +279,6 @@ int main(int argc, char **argv)
 		return 0;
 	}
 
-
 	/* Tell libical to return an error instead of aborting if it sees badly formed iCalendar data. */
 	icalerror_errors_are_fatal = 0;
 
@@ -284,9 +291,9 @@ int main(int argc, char **argv)
 	 * wcsession struct to which the thread is currently bound.
 	 */
 	if (pthread_key_create(&MyConKey, NULL) != 0) {
-		lprintf(1, "Can't create TSD key: %s\n", strerror(errno));
+		syslog(1, "Can't create TSD key: %s", strerror(errno));
 	}
-	InitialiseSemaphores ();
+	InitialiseSemaphores();
 
 	/*
 	 * Set up a place to put thread-specific SSL data.
@@ -296,23 +303,23 @@ int main(int argc, char **argv)
 	 */
 #ifdef HAVE_OPENSSL
 	if (pthread_key_create(&ThreadSSL, NULL) != 0) {
-		lprintf(1, "Can't create TSD key: %s\n", strerror(errno));
+		syslog(1, "Can't create TSD key: %s", strerror(errno));
 	}
 #endif
 
 	/*
 	 * Bind the server to our favorite port.
-	 * There is no need to check for errors, because ig_tcp_server()
+	 * There is no need to check for errors, because webcit_tcp_server()
 	 * exits if it doesn't succeed.
 	 */
 
 	if (!IsEmptyStr(uds_listen_path)) {
-		lprintf(2, "Attempting to create listener socket at %s...\n", uds_listen_path);
-		msock = ig_uds_server(uds_listen_path, LISTEN_QUEUE_LENGTH);
+		syslog(2, "Attempting to create listener socket at %s...", uds_listen_path);
+		msock = webcit_uds_server(uds_listen_path, LISTEN_QUEUE_LENGTH);
 	}
 	else {
-		lprintf(2, "Attempting to bind to port %d...\n", http_port);
-		msock = ig_tcp_server(ip_addr, http_port, LISTEN_QUEUE_LENGTH);
+		syslog(2, "Attempting to bind to port %d...", http_port);
+		msock = webcit_tcp_server(ip_addr, http_port, LISTEN_QUEUE_LENGTH);
 	}
 	if (msock < 0)
 	{
@@ -320,7 +327,7 @@ int main(int argc, char **argv)
 		return -msock;
 	}
 
-	lprintf(2, "Listening on socket %d\n", msock);
+	syslog(2, "Listening on socket %d", msock);
 	signal(SIGPIPE, SIG_IGN);
 
 	pthread_mutex_init(&SessionListMutex, NULL);
@@ -330,9 +337,7 @@ int main(int argc, char **argv)
 	 */
 	pthread_attr_init(&attr);
 	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-	pthread_create(&SessThread, &attr,
-		       (void *(*)(void *)) housekeeping_loop, NULL);
-
+	pthread_create(&SessThread, &attr, (void *(*)(void *)) housekeeping_loop, NULL);
 
 	/*
 	 * If this is an HTTPS server, fire up SSL
@@ -346,7 +351,7 @@ int main(int argc, char **argv)
 
 	/* Become a worker thread.  More worker threads will be spawned as they are needed. */
 	worker_entry();
-	ShutDownLibCitadel ();
+	ShutDownLibCitadel();
 	return 0;
 }
 
