@@ -1,11 +1,25 @@
 /*
- * $Id$
- *
  * Functions pertaining to rooms with a wiki view
+ *
+ * Copyright (c) 2009-2011 by the citadel.org team
+ *
+ * This program is open source software.  You can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "webcit.h"
-#include "groupdav.h"
+#include "dav.h"
 
 /* 
  * Convert a string to something suitable as a wiki index
@@ -87,7 +101,7 @@ void display_wiki_page_backend(const StrBuf *roomname, char *pagename, char *rev
 		return;
 	}
 
-	wc_printf("<br /><br />"
+	wc_printf("<br><br>"
 		"<div align=\"center\">"
 		"<table border=\"0\" bgcolor=\"#ffffff\" cellpadding=\"10\">"
 		"<tr><td align=\"center\">"
@@ -162,9 +176,7 @@ void tmplput_display_wiki_history(StrBuf *Target, WCTemplputParams *TP)
 		StrBuf *author = NewStrBuf();
 		StrBuf *node = NewStrBuf();
 
-		wc_printf("<div class=\"fix_scrollbar_bug\">"
-			"<table class=\"wiki_history_background\">"
-		);
+		wc_printf("<table class=\"wiki_history_background\">");
 
 		wc_printf("<th>%s</th>", _("Date"));
 		wc_printf("<th>%s</th>", _("Author"));
@@ -205,7 +217,7 @@ void tmplput_display_wiki_history(StrBuf *Target, WCTemplputParams *TP)
 					ChrPtr(rev_uuid),
 					_("(show)")
 				);
-				wc_printf("<td><a href=\"wiki?page=%s?rev=%s?revert=1\">%s</a></td>",
+				wc_printf("<td><a href=\"javascript:GetLoggedInFirst(encodeURIComponent('wiki?page=%s?rev=%s?revert=1'))\">%s</a></td>",
 					bstr("page"),
 					ChrPtr(rev_uuid),
 					_("(revert)")
@@ -244,7 +256,7 @@ void tmplput_display_wiki_history(StrBuf *Target, WCTemplputParams *TP)
 void display_wiki_history(void)
 {
 	output_headers(1, 1, 1, 0, 0, 0);
-	do_template("wiki_history", NULL);
+	do_template("wiki_history");
 	wDumpContent(1);
 }
 
@@ -284,10 +296,7 @@ void tmplput_display_wiki_pagelist(StrBuf *Target, WCTemplputParams *TP)
 	if (GetServerStatus(Buf, NULL) == 1) {
 		StrBuf *pagetitle = NewStrBuf();
 
-		wc_printf("<div class=\"fix_scrollbar_bug\">"
-			"<table class=\"wiki_pagelist_background\">"
-		);
-
+		wc_printf("<table class=\"wiki_pagelist_background\">");
 		wc_printf("<th>%s</th>", _("Page title"));
 
 		while((StrBuf_ServGetln(Buf) >= 0) && strcmp(ChrPtr(Buf), "000")) {
@@ -318,7 +327,7 @@ void tmplput_display_wiki_pagelist(StrBuf *Target, WCTemplputParams *TP)
 void display_wiki_pagelist(void)
 {
 	output_headers(1, 1, 1, 0, 0, 0);
-	do_template("wiki_pagelist", NULL);
+	do_template("wiki_pagelist");
 	wDumpContent(1);
 }
 
@@ -332,6 +341,30 @@ int wiki_Cleanup(void **ViewSpecific)
 	return 0;
 }
 
+
+int ConditionalHaveWikiPage(StrBuf *Target, WCTemplputParams *TP)
+{
+	const char *page;
+	const char *pch;
+	long len;
+
+	page = BSTR("page");
+	GetTemplateTokenString(Target, TP, 2, &pch, &len);
+	return strcasecmp(page, pch) == 0;
+}
+
+
+int ConditionalHavewikiType(StrBuf *Target, WCTemplputParams *TP)
+{
+	wcsession *WCC = WC;
+	const char *pch;
+	long len;
+
+	GetTemplateTokenString(Target, TP, 2, &pch, &len);
+	return bmstrcasestr((char *)ChrPtr(WCC->Hdr->HR.ReqLine), pch) != NULL;
+}
+
+
 void 
 InitModule_WIKI
 (void)
@@ -351,4 +384,6 @@ InitModule_WIKI
 	WebcitAddUrlHandler(HKEY("wiki_pagelist"), "", 0, display_wiki_pagelist, 0);
 	RegisterNamespace("WIKI:DISPLAYHISTORY", 0, 0, tmplput_display_wiki_history, NULL, CTX_NONE);
 	RegisterNamespace("WIKI:DISPLAYPAGELIST", 0, 0, tmplput_display_wiki_pagelist, NULL, CTX_NONE);
+	RegisterConditional(HKEY("COND:WIKI:PAGE"), 1, ConditionalHaveWikiPage, CTX_NONE);
+	RegisterConditional(HKEY("COND:WIKI:TYPE"), 1, ConditionalHavewikiType, CTX_NONE);
 }
